@@ -558,7 +558,7 @@ rasqal_algebra_write_indent(raptor_iostream *iostr, int indent)
 {
   while(indent > 0) {
     int sp = (indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
-    raptor_iostream_write_bytes(iostr, spaces, sizeof(char), sp);
+    raptor_iostream_write_bytes(spaces, sizeof(char), sp, iostr);
     indent -= sp;
   }
 }
@@ -572,14 +572,14 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   int indent_delta;
   
   if(node->op == RASQAL_ALGEBRA_OPERATOR_BGP && !node->triples) {
-    raptor_iostream_write_byte(iostr, 'Z');
+    raptor_iostream_write_byte('Z', iostr);
     return 0;
   }
   
   indent_delta = strlen(op_string);
 
-  raptor_iostream_write_counted_string(iostr, op_string, indent_delta);
-  raptor_iostream_write_counted_string(iostr, "(\n", 2);
+  raptor_iostream_counted_string_write(op_string, indent_delta, iostr);
+  raptor_iostream_counted_string_write("(\n", 2, iostr);
   indent_delta++;
   
   indent += indent_delta;
@@ -592,7 +592,7 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
       rasqal_triple *t;
       t = (rasqal_triple*)raptor_sequence_get_at(node->triples, i);
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_algebra_write_indent(iostr, indent);
       }
       rasqal_triple_write(t, iostr);
@@ -601,14 +601,14 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   }
   if(node->node1) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
     rasqal_algebra_algebra_node_write_internal(node->node1, iostr, indent);
     arg_count++;
     if(node->node2) {
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_algebra_write_indent(iostr, indent);
       }
       rasqal_algebra_algebra_node_write_internal(node->node2, iostr, indent);
@@ -619,7 +619,7 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   /* look for assignment var */
   if(node->var) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
     rasqal_variable_write(node->var, iostr);
@@ -629,7 +629,7 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   /* look for FILTER expression */
   if(node->expr) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
     rasqal_expression_write(node->expr, iostr);
@@ -642,19 +642,19 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
       int i;
       
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_algebra_write_indent(iostr, indent);
       }
-      raptor_iostream_write_counted_string(iostr, "Conditions([ ", 13);
+      raptor_iostream_counted_string_write("Conditions([ ", 13, iostr);
       for(i = 0; i < order_size; i++) {
         rasqal_expression* e;
         e = (rasqal_expression*)raptor_sequence_get_at(node->seq, i);
         if(i > 0)
-          raptor_iostream_write_counted_string(iostr, ", ", 2);
+          raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_expression_write(e, iostr);
         arg_count++;
       }
-      raptor_iostream_write_counted_string(iostr, " ])", 3);
+      raptor_iostream_counted_string_write(" ])", 3, iostr);
     }
   }
 
@@ -663,50 +663,50 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
     int i;
       
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
-    raptor_iostream_write_counted_string(iostr, "Variables([ ", 12);
+    raptor_iostream_counted_string_write("Variables([ ", 12, iostr);
     for(i = 0; i < vars_size; i++) {
       rasqal_variable* v;
       v = (rasqal_variable*)raptor_sequence_get_at(node->vars_seq, i);
       if(i > 0)
-        raptor_iostream_write_counted_string(iostr, ", ", 2);
+        raptor_iostream_counted_string_write(", ", 2, iostr);
       rasqal_variable_write(v, iostr);
       arg_count++;
     }
-    raptor_iostream_write_counted_string(iostr, " ])", 3);
+    raptor_iostream_counted_string_write(" ])", 3, iostr);
   }
 
   if(node->op == RASQAL_ALGEBRA_OPERATOR_SLICE) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
-    raptor_iostream_write_string(iostr, "slice start ");
-    raptor_iostream_write_decimal(iostr, node->start);
-    raptor_iostream_write_string(iostr, " length ");
-    raptor_iostream_write_decimal(iostr, node->length);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_string_write("slice start ", iostr);
+    raptor_iostream_decimal_write(node->start, iostr);
+    raptor_iostream_string_write(" length ", iostr);
+    raptor_iostream_decimal_write(node->length, iostr);
+    raptor_iostream_write_byte('\n', iostr);
     arg_count++;
   }
 
   if(node->op == RASQAL_ALGEBRA_OPERATOR_GRAPH) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
-    raptor_iostream_write_string(iostr, "origin ");
+    raptor_iostream_string_write("origin ", iostr);
     rasqal_literal_write(node->graph, iostr);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
     arg_count++;
   }
 
-  raptor_iostream_write_byte(iostr, '\n');
+  raptor_iostream_write_byte('\n', iostr);
   indent-= indent_delta;
 
   rasqal_algebra_write_indent(iostr, indent);
-  raptor_iostream_write_byte(iostr, ')');
+  raptor_iostream_write_byte(')', iostr);
 
   return 0;
 }
@@ -723,21 +723,24 @@ rasqal_algebra_algebra_node_write(rasqal_algebra_node *node,
 /**
  * rasqal_algebra_node_print:
  * @gp: the #rasqal_algebra_node object
- * @fh: the #FILE* handle to print to
+ * @fh: the FILE* handle to print to
  *
  * Print a #rasqal_algebra_node in a debug format.
  * 
  * The print debug format may change in any release.
  * 
+ * Return value: non-0 on failure
  **/
-void
+int
 rasqal_algebra_node_print(rasqal_algebra_node* node, FILE* fh)
 {
   raptor_iostream* iostr;
 
-  iostr = raptor_new_iostream_to_file_handle(fh);
+  iostr = raptor_new_iostream_to_file_handle(node->query->world->raptor_world_ptr, fh);
   rasqal_algebra_algebra_node_write(node, iostr);
   raptor_free_iostream(iostr);
+
+  return 0;
 }
 
 
@@ -1240,6 +1243,42 @@ rasqal_algebra_remove_znodes(rasqal_query* query, rasqal_algebra_node* node,
 }
 
 
+static raptor_sequence*
+rasqal_algebra_get_variables_mentioned_in(rasqal_query* query,
+                                          int row_index)
+{
+  raptor_sequence* seq; /* sequence of rasqal_variable* */
+  int width;
+  short *row;
+  int i;
+  
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                            (raptor_data_print_handler)rasqal_variable_print);
+#else
+  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                            (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
+  if(!seq)
+    return NULL;
+
+  width = rasqal_variables_table_get_total_variables_count(query->vars_table);
+  row = &query->variables_use_map[row_index * width];
+
+  for(i = 0; i < width; i++) {
+    rasqal_variable* v;
+
+    if(!(row[i] & RASQAL_VAR_USE_MENTIONED_HERE))
+      continue;
+
+    v = rasqal_variables_table_get(query->vars_table, i);
+    raptor_sequence_push(seq, rasqal_new_variable_from_variable(v));
+  }
+
+  return seq;
+}
+
+
 /**
  * rasqal_algebra_query_to_algebra:
  * @query: #rasqal_query to operate on
@@ -1284,7 +1323,11 @@ rasqal_algebra_query_to_algebra(rasqal_query* query)
       /* Make a deep copy of the query order conditions sequence for
        * the ORDERBY algebra node
        */
+#ifdef HAVE_RAPTOR2_API
+      seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_expression, (raptor_data_print_handler)rasqal_expression_print);
+#else
       seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_expression, (raptor_sequence_print_handler*)rasqal_expression_print);
+#endif
       if(!seq) {
         rasqal_free_algebra_node(node);
         return NULL;
@@ -1313,24 +1356,38 @@ rasqal_algebra_query_to_algebra(rasqal_query* query)
    */
   if(1) {
     int vars_size = 0;
-    raptor_sequence* seq = NULL;
+    raptor_sequence* seq = NULL;  /* sequence of rasqal_variable* */
     raptor_sequence* vars_seq;
     int i;
 
-    if(query->verb == RASQAL_QUERY_VERB_SELECT ||
-       query->verb == RASQAL_QUERY_VERB_CONSTRUCT)
-      /* sequence of rasqal_variable* */
+    if(query->verb == RASQAL_QUERY_VERB_SELECT)
+      /* project all selected variables */
       seq = query->selects;
+    else if(query->verb == RASQAL_QUERY_VERB_CONSTRUCT) {
+      /* project all variables mentioned in CONSTRUCT  */
+      seq = rasqal_algebra_get_variables_mentioned_in(query, 
+                                                      RASQAL_VAR_USE_MAP_OFFSET_VERBS);
+      if(!seq) {
+        rasqal_free_algebra_node(node);
+        return NULL;
+      }
+    }
 
     if(seq)
       vars_size = raptor_sequence_size(seq);
     
+#ifdef HAVE_RAPTOR2_API
+    vars_seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                   (raptor_data_print_handler)rasqal_variable_print);
+#else
     vars_seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
                                    (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
     if(!vars_seq) {
       rasqal_free_algebra_node(node);
       return NULL;
     }
+
     for(i = 0; i < vars_size; i++) {
       rasqal_variable* v;
       v = (rasqal_variable*)raptor_sequence_get_at(seq, i);
@@ -1341,12 +1398,14 @@ rasqal_algebra_query_to_algebra(rasqal_query* query)
     modified = 1;
 
 #if RASQAL_DEBUG > 1
-  RASQAL_DEBUG2("modified=%d after adding project node, algebra node now:\n  ", modified);
-  rasqal_algebra_node_print(node, stderr);
-  fputs("\n", stderr);
+    RASQAL_DEBUG2("modified=%d after adding project node, algebra node now:\n  ", modified);
+    rasqal_algebra_node_print(node, stderr);
+    fputs("\n", stderr);
 #endif
-  }
 
+    if(query->verb == RASQAL_QUERY_VERB_CONSTRUCT && seq) 
+      raptor_free_sequence(seq);
+  }
 
   if(query->distinct) {
     node = rasqal_new_distinct_algebra_node(query, node);
@@ -1421,11 +1480,7 @@ main(int argc, char *argv[]) {
   uri_string = raptor_uri_filename_to_uri_string("");
   if(!uri_string)
     FAIL;
-#ifdef RAPTOR_V2_AVAILABLE
-  base_uri = raptor_new_uri_v2(world->raptor_world_ptr, uri_string);
-#else
-  base_uri = raptor_new_uri(uri_string);
-#endif
+  base_uri = raptor_new_uri(world->raptor_world_ptr, uri_string);
   if(!base_uri)
     FAIL;
   raptor_free_memory(uri_string);
@@ -1596,7 +1651,13 @@ main(int argc, char *argv[]) {
   /* This is an artificial order conditions sequence equivalent to
    * ORDER BY 1, 2 which would probably never appear in a query.
    */
-  conditions = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_expression, (raptor_sequence_print_handler*)rasqal_expression_print);
+#ifdef HAVE_RAPTOR2_API
+  conditions = raptor_new_sequence((raptor_data_free_handler)rasqal_free_expression,
+                                   (raptor_data_print_handler)rasqal_expression_print);
+#else
+  conditions = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_expression,
+                                   (raptor_sequence_print_handler*)rasqal_expression_print);
+#endif
   if(!conditions)
     FAIL;
   lit3 = rasqal_new_integer_literal(world, RASQAL_LITERAL_INTEGER, 1);
@@ -1671,11 +1732,7 @@ main(int argc, char *argv[]) {
   if(query)
     rasqal_free_query(query);
   if(base_uri)
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_free_uri_v2(world->raptor_world_ptr, base_uri);
-#else
     raptor_free_uri(base_uri);
-#endif
   if(world)
     rasqal_free_world(world);
   
