@@ -2,7 +2,7 @@
  *
  * rasqal_expr.c - Rasqal general expression support
  *
- * Copyright (C) 2003-2009, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2003-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -100,8 +100,7 @@ rasqal_new_0op_expression(rasqal_world* world, rasqal_op op)
  * @RASQAL_EXPR_LANGMATCHES
  * @RASQAL_EXPR_DATATYPE @RASQAL_EXPR_ISURI @RASQAL_EXPR_ISBLANK
  * @RASQAL_EXPR_ISLITERAL @RASQAL_EXPR_ORDER_COND_ASC
- * @RASQAL_EXPR_ORDER_COND_DESC @RASQAL_EXPR_GROUP_COND_ASC
- * @RASQAL_EXPR_GROUP_COND_DESC @RASQAL_EXPR_COUNT @RASQAL_EXPR_SUM
+ * @RASQAL_EXPR_ORDER_COND_DESC @RASQAL_EXPR_COUNT @RASQAL_EXPR_SUM
  * @RASQAL_EXPR_AVG @RASQAL_EXPR_MIN @RASQAL_EXPR_MAX
  *
  * @RASQAL_EXPR_BANG and @RASQAL_EXPR_UMINUS are used by RDQL and
@@ -111,7 +110,8 @@ rasqal_new_0op_expression(rasqal_world* world, rasqal_op op)
  * Return value: a new #rasqal_expression object or NULL on failure
  **/
 rasqal_expression*
-rasqal_new_1op_expression(rasqal_world* world, rasqal_op op, rasqal_expression* arg)
+rasqal_new_1op_expression(rasqal_world* world, rasqal_op op,
+                          rasqal_expression* arg)
 {
   rasqal_expression* e = NULL;
 
@@ -313,34 +313,30 @@ rasqal_new_literal_expression(rasqal_world* world, rasqal_literal *literal)
 }
 
 
-/**
- * rasqal_new_function_expression:
- * @world: rasqal_world object
- * @name: function name
- * @args: sequence of #rasqal_expression function arguments
- * 
- * Constructor - create a new expression for a function with expression arguments.
- * Takes ownership of the function uri and arguments.
- * 
- * Return value: a new #rasqal_expression object or NULL on failure
- **/
-rasqal_expression*
-rasqal_new_function_expression(rasqal_world* world,
-                               raptor_uri* name,
-                               raptor_sequence* args)
+static rasqal_expression*
+rasqal_new_function_expression_common(rasqal_world* world,
+                                      rasqal_op op,
+                                      raptor_uri* name,
+                                      rasqal_expression* arg1,
+                                      raptor_sequence* args,
+                                      raptor_sequence* params,
+                                      unsigned int flags)
 {
   rasqal_expression* e = NULL;
 
-  if(!world || !name || !args)
+  if(!world || (arg1 && args) || (name && !args)|| (!name && args))
     goto tidy;
   
   e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
-    e->op = RASQAL_EXPR_FUNCTION;
+    e->op = op;
     e->name = name; name = NULL;
+    e->arg1 = arg1; arg1 = NULL;
     e->args = args; args = NULL;
+    e->params = params; params = NULL;
+    e->flags = flags;
   }
   
   tidy:
@@ -348,11 +344,98 @@ rasqal_new_function_expression(rasqal_world* world,
     raptor_free_uri(name);
   if(args)
     raptor_free_sequence(args);
+  if(params)
+    raptor_free_sequence(params);
 
   return e;
 }
 
 
+/**
+ * rasqal_new_function_expression2:
+ * @world: rasqal_world object
+ * @name: function name
+ * @args: sequence of #rasqal_expression function arguments
+ * @params: sequence of #rasqal_expression function parameters (or NULL)
+ * @flags: extension function bitflags
+ * 
+ * Constructor - create a new expression for a URI-named function with arguments and optional parameters.
+ *
+ * Takes ownership of the @name, @args and @params arguments.
+ * 
+ * Return value: a new #rasqal_expression object or NULL on failure
+ **/
+rasqal_expression*
+rasqal_new_function_expression2(rasqal_world* world,
+                                raptor_uri* name,
+                                raptor_sequence* args,
+                                raptor_sequence* params,
+                                unsigned int flags)
+{
+  return rasqal_new_function_expression_common(world, RASQAL_EXPR_FUNCTION,
+                                               name,
+                                               NULL /* expr */, args,
+                                               params,
+                                               flags);
+}
+
+
+/**
+ * rasqal_new_function_expression:
+ * @world: rasqal_world object
+ * @name: function name
+ * @args: arguments
+ * 
+ * Constructor - create a new expression for a function with expression arguments.
+ * Takes ownership of the @name and @args
+ * 
+ * @Deprecated: use rasqal_new_function_expression2() with extra args
+ *
+ * Return value: a new #rasqal_expression object or NULL on failure
+ **/
+rasqal_expression*
+rasqal_new_function_expression(rasqal_world* world,
+                               raptor_uri* name,
+                               raptor_sequence* args)
+{
+  return rasqal_new_function_expression_common(world, RASQAL_EXPR_FUNCTION,
+                                               name, 
+                                               NULL /* expr */, args,
+                                               NULL /* params */,
+                                               0 /* flags */);
+}
+
+
+
+/**
+ * rasqal_new_aggregate_function_expression:
+ * @world: rasqal_world object
+ * @op:  built-in aggregate function expression operator
+ * @arg1: #rasqal_expression argument to aggregate function
+ * @params: sequence of #rasqal_expression function parameters (or NULL)
+ * @flags: extension function bitflags
+ * 
+ * Constructor - create a new 1-arg aggregate function expression for a builtin aggregate function
+ *
+ * Takes ownership of the @args and @params
+ * 
+ * Return value: a new #rasqal_expression object or NULL on failure
+ **/
+rasqal_expression*
+rasqal_new_aggregate_function_expression(rasqal_world* world,
+                                         rasqal_op op,
+                                         rasqal_expression* arg1,
+                                         raptor_sequence* params,
+                                         unsigned int flags)
+{
+  return rasqal_new_function_expression_common(world, op,
+                                               NULL /* name */,
+                                               arg1, NULL /* args */,
+                                               params,
+                                               flags | RASQAL_EXPR_FLAG_AGGREGATE);
+}
+
+
 /**
  * rasqal_new_cast_expression:
  * @world: rasqal_world object
@@ -427,6 +510,95 @@ rasqal_new_coalesce_expression(rasqal_world* world, raptor_sequence* args)
 }
 
 
+/**
+ * rasqal_new_set_expression:
+ * @world: rasqal_world object
+ * @op: list operation
+ * @arg1: expression to look for in list
+ * @args: sequence of #rasqal_expression list arguments
+ * 
+ * Constructor - create a new set IN/NOT IN operation with expression arguments.
+ *
+ * Takes ownership of the @arg1 and @args
+ * 
+ * Return value: a new #rasqal_expression object or NULL on failure
+ **/
+rasqal_expression*
+rasqal_new_set_expression(rasqal_world* world, rasqal_op op,
+                          rasqal_expression* arg1,
+                          raptor_sequence* args)
+{
+  rasqal_expression* e = NULL;
+
+  if(!world || !arg1 || !args)
+    goto tidy;
+  
+  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  if(e) {
+    e->usage = 1;
+    e->world = world;
+    e->op = op;
+    e->arg1 = arg1; arg1 = NULL;
+    e->args = args; args = NULL;
+  }
+  
+  tidy:
+  if(arg1)
+    rasqal_free_expression(arg1);
+  if(args)
+    raptor_free_sequence(args);
+
+  return e;
+}
+
+
+/**
+ * rasqal_new_group_concat_expression:
+ * @world: rasqal_world object
+ * @flags: bitset of flags.  Only #RASQAL_EXPR_FLAG_DISTINCT is defined
+ * @args: sequence of #rasqal_expression list arguments
+ * @separator: SEPARATOR string literal or NULL
+ * 
+ * Constructor - create a new SPARQL group concat expression
+ *
+ * Takes an optional distinct flag, a list of expressions and an optional separator string.
+ *
+ * Takes ownership of the @args and @separator
+ * 
+ * Return value: a new #rasqal_expression object or NULL on failure
+ **/
+rasqal_expression*
+rasqal_new_group_concat_expression(rasqal_world* world, 
+                                   int flags,
+                                   raptor_sequence* args,
+                                   rasqal_literal* separator)
+{
+  rasqal_expression* e = NULL;
+
+  if(!world || !args)
+    goto tidy;
+  
+  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  if(e) {
+    e->usage = 1;
+    e->world = world;
+    /* Discard any flags except RASQAL_EXPR_FLAG_DISTINCT */
+    e->flags = flags & RASQAL_EXPR_FLAG_DISTINCT;
+    e->op = RASQAL_EXPR_GROUP_CONCAT;
+    e->args = args; args = NULL;
+    e->literal = separator; separator = NULL;
+  }
+  
+  tidy:
+  if(args)
+    raptor_free_sequence(args);
+  if(separator)
+    rasqal_free_literal(separator);
+
+  return e;
+}
+
+
 /**
  * rasqal_expression_clear:
  * @e: expression
@@ -459,10 +631,13 @@ rasqal_expression_clear(rasqal_expression* e)
     case RASQAL_EXPR_STR_NEQ:
     case RASQAL_EXPR_LANGMATCHES:
     case RASQAL_EXPR_SAMETERM:
+    case RASQAL_EXPR_STRLANG:
+    case RASQAL_EXPR_STRDT:
       rasqal_free_expression(e->arg1);
       rasqal_free_expression(e->arg2);
       break;
     case RASQAL_EXPR_REGEX:
+    case RASQAL_EXPR_IF:
       rasqal_free_expression(e->arg1);
       rasqal_free_expression(e->arg2);
       if(e->arg3)
@@ -487,7 +662,13 @@ rasqal_expression_clear(rasqal_expression* e)
     case RASQAL_EXPR_AVG:
     case RASQAL_EXPR_MIN:
     case RASQAL_EXPR_MAX:
-      rasqal_free_expression(e->arg1);
+    case RASQAL_EXPR_URI:
+    case RASQAL_EXPR_IRI:
+    case RASQAL_EXPR_BNODE:
+    case RASQAL_EXPR_SAMPLE:
+      /* arg1 is optional for RASQAL_EXPR_BNODE */
+      if(e->arg1)
+        rasqal_free_expression(e->arg1);
       break;
     case RASQAL_EXPR_STR_MATCH:
     case RASQAL_EXPR_STR_NMATCH:
@@ -497,8 +678,11 @@ rasqal_expression_clear(rasqal_expression* e)
       rasqal_free_literal(e->literal);
       break;
     case RASQAL_EXPR_FUNCTION:
+    case RASQAL_EXPR_GROUP_CONCAT:
       raptor_free_uri(e->name);
       raptor_free_sequence(e->args);
+      if(e->literal) /* GROUP_CONCAT() SEPARATOR */
+        rasqal_free_literal(e->literal);
       break;
     case RASQAL_EXPR_CAST:
       raptor_free_uri(e->name);
@@ -513,6 +697,12 @@ rasqal_expression_clear(rasqal_expression* e)
       raptor_free_sequence(e->args);
       break;
 
+    case RASQAL_EXPR_IN:
+    case RASQAL_EXPR_NOT_IN:
+      rasqal_free_expression(e->arg1);
+      raptor_free_sequence(e->args);
+      break;
+
     case RASQAL_EXPR_UNKNOWN:
     default:
       RASQAL_FATAL2("Unknown operation %d", e->op);
@@ -549,7 +739,8 @@ rasqal_new_expression_from_expression(rasqal_expression* e)
 void
 rasqal_free_expression(rasqal_expression* e)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(e, rasqal_expression);
+  if(!e)
+    return;
   
   if(--e->usage)
     return;
@@ -567,9 +758,10 @@ rasqal_free_expression(rasqal_expression* e)
  * 
  * Visit a user function over a #rasqal_expression
  * 
- * If the user function @fn returns 0, the visit is truncated.
+ * If the user function @fn returns non-0, the visit is truncated
+ * and the value is returned.
  *
- * Return value: 0 if the visit was truncated.
+ * Return value: non-0 if the visit was truncated.
  **/
 int
 rasqal_expression_visit(rasqal_expression* e, 
@@ -577,7 +769,7 @@ rasqal_expression_visit(rasqal_expression* e,
                         void *user_data)
 {
   int i;
-  int result=0;
+  int result = 0;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(e, rasqal_expression, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(fn, rasqal_expression_visit_fn, 1);
@@ -605,10 +797,13 @@ rasqal_expression_visit(rasqal_expression* e,
     case RASQAL_EXPR_STR_NEQ:
     case RASQAL_EXPR_LANGMATCHES:
     case RASQAL_EXPR_SAMETERM:
+    case RASQAL_EXPR_STRLANG:
+    case RASQAL_EXPR_STRDT:
       return rasqal_expression_visit(e->arg1, fn, user_data) ||
              rasqal_expression_visit(e->arg2, fn, user_data);
       break;
     case RASQAL_EXPR_REGEX:
+    case RASQAL_EXPR_IF:
       return rasqal_expression_visit(e->arg1, fn, user_data) ||
              rasqal_expression_visit(e->arg2, fn, user_data) ||
              (e->arg3 && rasqal_expression_visit(e->arg3, fn, user_data));
@@ -633,7 +828,12 @@ rasqal_expression_visit(rasqal_expression* e,
     case RASQAL_EXPR_AVG:
     case RASQAL_EXPR_MIN:
     case RASQAL_EXPR_MAX:
-      return rasqal_expression_visit(e->arg1, fn, user_data);
+    case RASQAL_EXPR_URI:
+    case RASQAL_EXPR_IRI:
+    case RASQAL_EXPR_BNODE:
+    case RASQAL_EXPR_SAMPLE:
+      /* arg1 is optional for RASQAL_EXPR_BNODE */
+      return (e->arg1) ? rasqal_expression_visit(e->arg1, fn, user_data) : 1;
       break;
     case RASQAL_EXPR_STR_MATCH:
     case RASQAL_EXPR_STR_NMATCH:
@@ -643,13 +843,13 @@ rasqal_expression_visit(rasqal_expression* e,
       return 0;
     case RASQAL_EXPR_FUNCTION:
     case RASQAL_EXPR_COALESCE:
+    case RASQAL_EXPR_GROUP_CONCAT:
       for(i = 0; i < raptor_sequence_size(e->args); i++) {
         rasqal_expression* e2;
         e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
-        if(!rasqal_expression_visit(e2, fn, user_data)) {
-          result = 0;
+        result = rasqal_expression_visit(e2, fn, user_data);
+        if(result)
           break;
-        }
       }
       return result;
       break;
@@ -659,11 +859,29 @@ rasqal_expression_visit(rasqal_expression* e,
       return 0;
       break;
       
+    case RASQAL_EXPR_IN:
+    case RASQAL_EXPR_NOT_IN:
+      result = rasqal_expression_visit(e->arg1, fn, user_data);
+      if(result)
+        return result;
+
+      for(i = 0; i < raptor_sequence_size(e->args); i++) {
+        rasqal_expression* e2;
+        e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+        result = rasqal_expression_visit(e2, fn, user_data);
+        if(result)
+          break;
+      }
+      return result;
+      break;
+
     case RASQAL_EXPR_UNKNOWN:
     default:
       RASQAL_FATAL2("Unknown operation %d", e->op);
       return -1; /* keep some compilers happy */
   }
+
+  return result;
 }
 
 
@@ -874,11 +1092,7 @@ rasqal_expression_evaluate_strmatch(rasqal_world *world,
 
 #ifdef RASQAL_REGEX_NONE
   rasqal_log_error_simple(world,
-#ifdef RAPTOR_V2_AVAILABLE
                           RAPTOR_LOG_LEVEL_WARN,
-#else
-                          RAPTOR_LOG_LEVEL_WARNING,
-#endif
                           locator,
                           "Regex support missing, cannot compare '%s' to '%s'", match_string, pattern);
   b=1;
@@ -942,7 +1156,10 @@ rasqal_expression_evaluate(rasqal_world *world, raptor_locator *locator,
     const unsigned char *s;
     unsigned char *new_s;
     rasqal_variable *v;
+    rasqal_expression *e;
+    struct { void *dummy_do_not_mask; int found; } flags;
   } vars;
+  int i; /* for looping */
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(e, rasqal_expression, NULL);
@@ -1599,11 +1816,7 @@ rasqal_expression_evaluate(rasqal_world *world, raptor_locator *locator,
 
     case RASQAL_EXPR_FUNCTION:
       rasqal_log_error_simple(world,
-#ifdef RAPTOR_V2_AVAILABLE
                               RAPTOR_LOG_LEVEL_WARN,
-#else
-                              RAPTOR_LOG_LEVEL_WARNING,
-#endif
                               locator,
                               "No function expressions support at present.  Returning false.");
       result=rasqal_new_boolean_literal(world, 0);
@@ -1662,15 +1875,240 @@ rasqal_expression_evaluate(rasqal_world *world, raptor_locator *locator,
       break;
       
     case RASQAL_EXPR_COALESCE:
-      for(vars.i = 0; vars.i < raptor_sequence_size(e->args); vars.i++) {
-        rasqal_expression* e2;
-        e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, vars.i);
-        result = rasqal_expression_evaluate(world, locator, e2, flags);
+      for(i = 0; i < raptor_sequence_size(e->args); i++) {
+        vars.e = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+        result = rasqal_expression_evaluate(world, locator, vars.e, flags);
         if(result)
           break;
       }
       break;
 
+    case RASQAL_EXPR_IF:
+      l1 = rasqal_expression_evaluate(world, locator, e->arg1, flags);
+      if(!l1)
+        goto failed;
+
+      /* IF condition */
+      vars.b = rasqal_literal_as_boolean(l1, &errs.e);
+      rasqal_free_literal(l1);
+
+      if(errs.e)
+        goto failed;
+
+      /* condition is true: evaluate arg2 or false: evaluate arg3 */
+      result = rasqal_expression_evaluate(world, locator,
+                                          vars.b ? e->arg2 : e->arg3,
+                                          flags);
+      break;
+
+    case RASQAL_EXPR_URI:
+    case RASQAL_EXPR_IRI:
+      l1 = rasqal_expression_evaluate(world, locator, e->arg1, flags);
+      if(!l1)
+        goto failed;
+
+      s = rasqal_literal_as_string_flags(l1, flags, &errs.e);
+      if(errs.e) {
+        rasqal_free_literal(l1);
+        goto failed;
+      }
+
+      vars.dt_uri = raptor_new_uri(world->raptor_world_ptr, s);
+      rasqal_free_literal(l1);
+      if(!vars.dt_uri)
+        goto failed;
+      
+      result = rasqal_new_uri_literal(world, vars.dt_uri);
+      /* vars.dt_uri becomes owned by the result literal */
+      break;
+
+    case RASQAL_EXPR_STRLANG:
+      l1 = rasqal_expression_evaluate(world, locator, e->arg1, flags);
+      if(!l1)
+        goto failed;
+      
+      s = rasqal_literal_as_string_flags(l1, flags, &errs.e);
+      if(errs.e) {
+        rasqal_free_literal(l1);
+        goto failed;
+      }
+
+      l2 = rasqal_expression_evaluate(world, locator, e->arg2, flags);
+      if(!l2) {
+        rasqal_free_literal(l1);
+        goto failed;
+      }
+
+      vars.s = rasqal_literal_as_string_flags(l1, flags, &errs.e);
+      if(errs.e) {
+        rasqal_free_literal(l1);
+        rasqal_free_literal(l2);
+        goto failed;
+      }
+
+      result = rasqal_new_string_literal(world, s, (const char*)vars.s,
+                                         /*datatype */ NULL, /* qname */ NULL);
+
+      rasqal_free_literal(l1);
+      rasqal_free_literal(l2);
+      break;
+
+    case RASQAL_EXPR_STRDT:
+      l1 = rasqal_expression_evaluate(world, locator, e->arg1, flags);
+      if(!l1)
+        goto failed;
+      
+      s = rasqal_literal_as_string_flags(l1, flags, &errs.e);
+      if(errs.e) {
+        rasqal_free_literal(l1);
+        goto failed;
+      }
+
+      l2 = rasqal_expression_evaluate(world, locator, e->arg2, flags);
+      if(!l2) {
+        rasqal_free_literal(l1);
+        goto failed;
+      }
+
+      vars.dt_uri = rasqal_literal_as_uri(l2);
+      if(vars.dt_uri) {
+        vars.dt_uri = raptor_uri_copy(vars.dt_uri);
+      } else {
+        const unsigned char *uri_string;
+        uri_string = rasqal_literal_as_string_flags(l2, flags, &errs.e);
+        if(errs.e) {
+          rasqal_free_literal(l1);
+          rasqal_free_literal(l2);
+          goto failed;
+        }
+        vars.dt_uri = raptor_new_uri(world->raptor_world_ptr, uri_string);
+        if(!vars.dt_uri) {
+          rasqal_free_literal(l1);
+          rasqal_free_literal(l2);
+          goto failed;
+        }
+      }
+      
+      result = rasqal_new_string_literal(world, s, /* language */ NULL,
+                                         vars.dt_uri, /* qname */ NULL);
+
+      rasqal_free_literal(l1);
+      rasqal_free_literal(l2);
+      break;
+
+    case RASQAL_EXPR_BNODE:
+      if(e->arg1) {
+        l1 = rasqal_expression_evaluate(world, locator, e->arg1, flags);
+        if(!l1)
+          goto failed;
+
+        s = rasqal_literal_as_string_flags(l1, flags, &errs.e);
+        if(errs.e) {
+          rasqal_free_literal(l1);
+          goto failed;
+        }
+      } else {
+        s = rasqal_world_generate_bnodeid(world, NULL);
+        if(!s)
+          goto failed;
+      }
+
+      result = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, s);
+      break;
+
+    case RASQAL_EXPR_SAMPLE:
+      rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
+                              locator,
+                              "Evaluation of SPARQL SAMPLE() expression is not implemented yet, returning error.");
+      errs.e = 1;
+      goto failed;
+      break;
+      
+    case RASQAL_EXPR_GROUP_CONCAT:
+      rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
+                              locator,
+                              "Evaluation of SPARQL GROUP_CONCAT() expression is not implemented yet, returning error.");
+      errs.e = 1;
+      goto failed;
+      break;
+
+    case RASQAL_EXPR_IN:
+      l1 = rasqal_expression_evaluate(world, locator, e->arg1, flags);
+      if(!l1)
+        goto failed;
+
+      if(1) {
+        vars.flags.found = 0;
+        for(i = 0; i < raptor_sequence_size(e->args); i++) {
+          vars.e = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+          l2 = rasqal_expression_evaluate(world, locator, vars.e, flags);
+          if(!l2) {
+            rasqal_free_literal(l1);
+            goto failed;
+          }
+
+          vars.b = (rasqal_literal_equals_flags(l1, l2, flags, &errs.e) != 0);
+#if RASQAL_DEBUG > 1
+          if(errs.e)
+            RASQAL_DEBUG1("rasqal_literal_equals_flags returned: FAILURE\n");
+          else
+            RASQAL_DEBUG2("rasqal_literal_equals_flags returned: %d\n", vars.b);
+#endif
+          rasqal_free_literal(l2);
+          if(errs.e) {
+            rasqal_free_literal(l1);
+            goto failed;
+          }
+          if(vars.b) {
+            /* found - so succeeded */
+            vars.flags.found = 1;
+            break;
+          }
+        }
+        rasqal_free_literal(l1);
+
+        result = rasqal_new_boolean_literal(world, vars.flags.found);
+      }
+      break;
+
+    case RASQAL_EXPR_NOT_IN:
+      l1 = rasqal_expression_evaluate(world, locator, e->arg1, flags);
+      if(!l1)
+        goto failed;
+
+      if(1) {
+        vars.flags.found = 0;
+        for(i = 0; i < raptor_sequence_size(e->args); i++) {
+          vars.e = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+          l2 = rasqal_expression_evaluate(world, locator, vars.e, flags);
+          if(!l2) {
+            rasqal_free_literal(l1);
+            goto failed;
+          }
+
+          vars.b = (rasqal_literal_equals_flags(l1, l2, flags, &errs.e) != 0);
+#if RASQAL_DEBUG > 1
+          if(errs.e)
+            RASQAL_DEBUG1("rasqal_literal_equals_flags returned: FAILURE\n");
+          else
+            RASQAL_DEBUG2("rasqal_literal_equals_flags returned: %d\n", vars.b);
+#endif
+          rasqal_free_literal(l2);
+          if(errs.e) {
+            rasqal_free_literal(l1);
+            goto failed;
+          }
+          if(vars.b) {
+            /* found - so failed */
+            vars.flags.found = 1;
+            break;
+          }
+        }
+        rasqal_free_literal(l1);
+        result = rasqal_new_boolean_literal(world, !vars.flags.found);
+      }
+      break;
+
     case RASQAL_EXPR_UNKNOWN:
     default:
       RASQAL_FATAL2("Unknown operation %d", e->op);
@@ -1746,7 +2184,17 @@ static const char* const rasqal_op_labels[RASQAL_EXPR_LAST+1]={
   "avg",
   "min",
   "max",
-  "coalesce"
+  "coalesce",
+  "if",
+  "uri",
+  "iri",
+  "strlang",
+  "strdt",
+  "bnode",
+  "group_concat",
+  "sample",
+  "in",
+  "not in"
 };
 
 
@@ -1769,7 +2217,7 @@ rasqal_expression_write_op(rasqal_expression* e, raptor_iostream* iostr)
   op = e->op;
   if(op > RASQAL_EXPR_LAST)
     op = RASQAL_EXPR_UNKNOWN;
-  raptor_iostream_write_string(iostr, rasqal_op_labels[(int)op]);
+  raptor_iostream_string_write(rasqal_op_labels[(int)op], iostr);
 }
 
 
@@ -1814,7 +2262,7 @@ rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr)
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(e, rasqal_expression);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(iostr, raptor_iostr);
 
-  raptor_iostream_write_counted_string(iostr, "expr(", 5);
+  raptor_iostream_counted_string_write("expr(", 5, iostr);
   switch(e->op) {
     case RASQAL_EXPR_AND:
     case RASQAL_EXPR_OR:
@@ -1834,28 +2282,31 @@ rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr)
     case RASQAL_EXPR_LANGMATCHES:
     case RASQAL_EXPR_REGEX:
     case RASQAL_EXPR_SAMETERM:
-      raptor_iostream_write_counted_string(iostr, "op ", 3);
+    case RASQAL_EXPR_IF:
+    case RASQAL_EXPR_STRLANG:
+    case RASQAL_EXPR_STRDT:
+      raptor_iostream_counted_string_write("op ", 3, iostr);
       rasqal_expression_write_op(e, iostr);
-      raptor_iostream_write_byte(iostr, '(');
+      raptor_iostream_write_byte('(', iostr);
       rasqal_expression_write(e->arg1, iostr);
-      raptor_iostream_write_counted_string(iostr, ", ", 2);
+      raptor_iostream_counted_string_write(", ", 2, iostr);
       rasqal_expression_write(e->arg2, iostr);
-      /* There is only one 3-op expression and it's handled here */
-      if(e->op == RASQAL_EXPR_REGEX && e->arg3) {
-        raptor_iostream_write_counted_string(iostr, ", ", 2);
+      /* There are two 3-op expressions - both handled here */
+      if((e->op == RASQAL_EXPR_REGEX || e->op == RASQAL_EXPR_IF) && e->arg3) {
+        raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_expression_write(e->arg3, iostr);
       }
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_write_byte(')', iostr);
       break;
     case RASQAL_EXPR_STR_MATCH:
     case RASQAL_EXPR_STR_NMATCH:
-      raptor_iostream_write_counted_string(iostr, "op ", 3);
+      raptor_iostream_counted_string_write("op ", 3, iostr);
       rasqal_expression_write_op(e, iostr);
-      raptor_iostream_write_byte(iostr, '(');
+      raptor_iostream_write_byte('(', iostr);
       rasqal_expression_write(e->arg1, iostr);
-      raptor_iostream_write_counted_string(iostr, ", ", 2);
+      raptor_iostream_counted_string_write(", ", 2, iostr);
       rasqal_literal_write(e->literal, iostr);
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_write_byte(')', iostr);
       break;
     case RASQAL_EXPR_TILDE:
     case RASQAL_EXPR_BANG:
@@ -1876,11 +2327,17 @@ rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr)
     case RASQAL_EXPR_AVG:
     case RASQAL_EXPR_MIN:
     case RASQAL_EXPR_MAX:
-      raptor_iostream_write_counted_string(iostr, "op ", 3);
+    case RASQAL_EXPR_URI:
+    case RASQAL_EXPR_IRI:
+    case RASQAL_EXPR_BNODE:
+    case RASQAL_EXPR_SAMPLE:
+      raptor_iostream_counted_string_write("op ", 3, iostr);
       rasqal_expression_write_op(e, iostr);
-      raptor_iostream_write_byte(iostr, '(');
-      rasqal_expression_write(e->arg1, iostr);
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_write_byte('(', iostr);
+      /* arg1 is optional for RASQAL_EXPR_BNODE */
+      if(e->arg1)
+        rasqal_expression_write(e->arg1, iostr);
+      raptor_iostream_write_byte(')', iostr);
       break;
 
     case RASQAL_EXPR_LITERAL:
@@ -1888,48 +2345,84 @@ rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr)
       break;
 
     case RASQAL_EXPR_FUNCTION:
-      raptor_iostream_write_counted_string(iostr, "function(uri=", 13);
-      raptor_iostream_write_uri(iostr, e->name);
-      raptor_iostream_write_counted_string(iostr, ", args=", 7);
+      raptor_iostream_counted_string_write("function(uri=", 13, iostr);
+      raptor_uri_write(e->name, iostr);
+      raptor_iostream_counted_string_write(", args=", 7, iostr);
       for(i=0; i<raptor_sequence_size(e->args); i++) {
         rasqal_expression* e2;
         if(i>0)
-          raptor_iostream_write_counted_string(iostr, ", ", 2);
+          raptor_iostream_counted_string_write(", ", 2, iostr);
         e2=(rasqal_expression*)raptor_sequence_get_at(e->args, i);
         rasqal_expression_write(e2, iostr);
       }
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_write_byte(')', iostr);
       break;
 
     case RASQAL_EXPR_CAST:
-      raptor_iostream_write_counted_string(iostr, "cast(type=", 10);
-      raptor_iostream_write_uri(iostr, e->name);
-      raptor_iostream_write_counted_string(iostr, ", value=", 8);
+      raptor_iostream_counted_string_write("cast(type=", 10, iostr);
+      raptor_uri_write(e->name, iostr);
+      raptor_iostream_counted_string_write(", value=", 8, iostr);
       rasqal_expression_write(e->arg1, iostr);
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_write_byte(')', iostr);
       break;
 
     case RASQAL_EXPR_VARSTAR:
-      raptor_iostream_write_counted_string(iostr, "varstar", 7);
+      raptor_iostream_counted_string_write("varstar", 7, iostr);
       break;
       
     case RASQAL_EXPR_COALESCE:
-      raptor_iostream_write_counted_string(iostr, "coalesce(", 9);
+      raptor_iostream_counted_string_write("coalesce(", 9, iostr);
       for(i = 0; i < raptor_sequence_size(e->args); i++) {
         rasqal_expression* e2;
         if(i > 0)
-          raptor_iostream_write_counted_string(iostr, ", ", 2);
+          raptor_iostream_counted_string_write(", ", 2, iostr);
         e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
         rasqal_expression_write(e2, iostr);
       }
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_write_byte(')', iostr);
+      break;
+
+    case RASQAL_EXPR_GROUP_CONCAT:
+      raptor_iostream_counted_string_write("group_concat(", 13, iostr);
+      if(e->flags & RASQAL_EXPR_FLAG_DISTINCT)
+        raptor_iostream_counted_string_write("distinct,", 9, iostr);
+      raptor_iostream_counted_string_write("args=", 5, iostr);
+      for(i = 0; i < raptor_sequence_size(e->args); i++) {
+        rasqal_expression* e2;
+        if(i > 0)
+          raptor_iostream_counted_string_write(", ", 2, iostr);
+        e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+        rasqal_expression_write(e2, iostr);
+      }
+      if(e->literal) {
+        raptor_iostream_counted_string_write(",separator=", 11, iostr);
+        rasqal_literal_write(e->literal, iostr);
+      }
+      raptor_iostream_write_byte(')', iostr);
+      break;
+
+    case RASQAL_EXPR_IN:
+    case RASQAL_EXPR_NOT_IN:
+      raptor_iostream_counted_string_write("op ", 3, iostr);
+      rasqal_expression_write_op(e, iostr);
+      raptor_iostream_counted_string_write("(expr=", 6, iostr);
+      rasqal_expression_write(e->arg1, iostr);
+      raptor_iostream_counted_string_write(", args=", 7, iostr);
+      for(i = 0; i < raptor_sequence_size(e->args); i++) {
+        rasqal_expression* e2;
+        if(i > 0)
+          raptor_iostream_counted_string_write(", ", 2, iostr);
+        e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+        rasqal_expression_write(e2, iostr);
+      }
+      raptor_iostream_write_byte(')', iostr);
       break;
 
     case RASQAL_EXPR_UNKNOWN:
     default:
       RASQAL_FATAL2("Unknown operation %d", e->op);
   }
-  raptor_iostream_write_byte(iostr, ')');
+  raptor_iostream_write_byte(')', iostr);
 }
 
 
@@ -1970,14 +2463,17 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
     case RASQAL_EXPR_LANGMATCHES:
     case RASQAL_EXPR_REGEX:
     case RASQAL_EXPR_SAMETERM:
+    case RASQAL_EXPR_IF:
+    case RASQAL_EXPR_STRLANG:
+    case RASQAL_EXPR_STRDT:
       fputs("op ", fh);
       rasqal_expression_print_op(e, fh);
       fputc('(', fh);
       rasqal_expression_print(e->arg1, fh);
       fputs(", ", fh);
       rasqal_expression_print(e->arg2, fh);
-      /* There is only one 3-op expression and it's handled here */
-      if(e->op == RASQAL_EXPR_REGEX && e->arg3) {
+      /* There are two 3-op expressions - both handled here */
+      if((e->op == RASQAL_EXPR_REGEX || e->op == RASQAL_EXPR_IF) && e->arg3) {
         fputs(", ", fh);
         rasqal_expression_print(e->arg3, fh);
       }
@@ -2012,10 +2508,16 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
     case RASQAL_EXPR_AVG:
     case RASQAL_EXPR_MIN:
     case RASQAL_EXPR_MAX:
+    case RASQAL_EXPR_URI:
+    case RASQAL_EXPR_IRI:
+    case RASQAL_EXPR_BNODE:
+    case RASQAL_EXPR_SAMPLE:
       fputs("op ", fh);
       rasqal_expression_print_op(e, fh);
       fputc('(', fh);
-      rasqal_expression_print(e->arg1, fh);
+      /* arg1 is optional for RASQAL_EXPR_BNODE */
+      if(e->arg1)
+        rasqal_expression_print(e->arg1, fh);
       fputc(')', fh);
       break;
 
@@ -2049,6 +2551,30 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
       fputc(')', fh);
       break;
 
+    case RASQAL_EXPR_GROUP_CONCAT:
+      fputs("group_concat(", fh);
+      if(e->flags & RASQAL_EXPR_FLAG_DISTINCT)
+        fputs("distinct,", fh);
+      fputs("args=", fh);
+      raptor_sequence_print(e->args, fh);
+      if(e->literal) {
+        fputs(",separator=", fh);
+        rasqal_literal_print(e->literal, fh);
+      }
+      fputc(')', fh);
+      break;
+
+    case RASQAL_EXPR_IN:
+    case RASQAL_EXPR_NOT_IN:
+      fputs("op ", fh);
+      rasqal_expression_print_op(e, fh);
+      fputs("(expr=", fh);
+      rasqal_expression_print(e->arg1, fh);
+      fputs(", args=", fh);
+      raptor_sequence_print(e->args, fh);
+      fputc(')', fh);
+      break;
+
     case RASQAL_EXPR_UNKNOWN:
     default:
       RASQAL_FATAL2("Unknown operation %d", e->op);
@@ -2085,7 +2611,7 @@ int
 rasqal_expression_is_constant(rasqal_expression* e) 
 {
   int i;
-  int result=0;
+  int result = 0;
   
   switch(e->op) {
     case RASQAL_EXPR_AND:
@@ -2105,18 +2631,21 @@ rasqal_expression_is_constant(rasqal_expression* e)
     case RASQAL_EXPR_STR_NEQ:
     case RASQAL_EXPR_LANGMATCHES:
     case RASQAL_EXPR_SAMETERM:
-      result=rasqal_expression_is_constant(e->arg1) &&
-             rasqal_expression_is_constant(e->arg2);
+    case RASQAL_EXPR_STRLANG:
+    case RASQAL_EXPR_STRDT:
+      result = rasqal_expression_is_constant(e->arg1) &&
+               rasqal_expression_is_constant(e->arg2);
       break;
     case RASQAL_EXPR_REGEX:
-      result=rasqal_expression_is_constant(e->arg1) &&
-             rasqal_expression_is_constant(e->arg2) &&
-             (e->arg3 && rasqal_expression_is_constant(e->arg3));
+    case RASQAL_EXPR_IF:
+      result = rasqal_expression_is_constant(e->arg1) &&
+               rasqal_expression_is_constant(e->arg2) &&
+               (e->arg3 && rasqal_expression_is_constant(e->arg3));
       break;
     case RASQAL_EXPR_STR_MATCH:
     case RASQAL_EXPR_STR_NMATCH:
-      result=rasqal_expression_is_constant(e->arg1) &&
-             rasqal_literal_is_constant(e->literal);
+      result = rasqal_expression_is_constant(e->arg1) &&
+               rasqal_literal_is_constant(e->literal);
       break;
     case RASQAL_EXPR_TILDE:
     case RASQAL_EXPR_BANG:
@@ -2137,7 +2666,12 @@ rasqal_expression_is_constant(rasqal_expression* e)
     case RASQAL_EXPR_AVG:
     case RASQAL_EXPR_MIN:
     case RASQAL_EXPR_MAX:
-      result=rasqal_expression_is_constant(e->arg1);
+    case RASQAL_EXPR_URI:
+    case RASQAL_EXPR_IRI:
+    case RASQAL_EXPR_BNODE:
+    case RASQAL_EXPR_SAMPLE:
+      /* arg1 is optional for RASQAL_EXPR_BNODE and result is always constant */
+      result = (e->arg1) ? rasqal_expression_is_constant(e->arg1) : 1;
       break;
 
     case RASQAL_EXPR_LITERAL:
@@ -2146,6 +2680,7 @@ rasqal_expression_is_constant(rasqal_expression* e)
 
     case RASQAL_EXPR_FUNCTION:
     case RASQAL_EXPR_COALESCE:
+    case RASQAL_EXPR_GROUP_CONCAT:
       result = 1;
       for(i = 0; i < raptor_sequence_size(e->args); i++) {
         rasqal_expression* e2;
@@ -2155,6 +2690,7 @@ rasqal_expression_is_constant(rasqal_expression* e)
           break;
         }
       }
+      /* e->literal is always a string constant - do not need to check */
       break;
 
     case RASQAL_EXPR_CAST:
@@ -2165,6 +2701,23 @@ rasqal_expression_is_constant(rasqal_expression* e)
       result=0;
       break;
       
+    case RASQAL_EXPR_IN:
+    case RASQAL_EXPR_NOT_IN:
+      result = rasqal_expression_is_constant(e->arg1);
+      if(!result)
+        break;
+      
+      result = 1;
+      for(i = 0; i < raptor_sequence_size(e->args); i++) {
+        rasqal_expression* e2;
+        e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+        if(!rasqal_expression_is_constant(e2)) {
+          result = 0;
+          break;
+        }
+      }
+      break;
+      
     case RASQAL_EXPR_UNKNOWN:
     default:
       RASQAL_FATAL2("Unknown operation %d", e->op);
@@ -2244,7 +2797,7 @@ main(int argc, char *argv[])
   int error=0;
   rasqal_world *world;
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   raptor_world* raptor_world_ptr;
   raptor_world_ptr = raptor_new_world();
   if(!raptor_world_ptr || raptor_world_open(raptor_world_ptr))
@@ -2254,7 +2807,7 @@ main(int argc, char *argv[])
 #endif
 
   world = rasqal_new_world();
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   rasqal_world_set_raptor(world, raptor_world_ptr);
 #endif
   /* no rasqal_world_open() */
@@ -2302,7 +2855,7 @@ main(int argc, char *argv[])
   
   RASQAL_FREE(rasqal_world, world);
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   raptor_free_world(raptor_world_ptr);
 #else
   raptor_finish();
