@@ -85,7 +85,7 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 {
   int rc=1;
   rasqal_query* query = rasqal_query_results_get_query(results);
-#ifndef RAPTOR_V2_AVAILABLE
+#ifndef HAVE_RAPTOR2_API
   const raptor_uri_handler *uri_handler;
   void *uri_context;
 #endif
@@ -110,11 +110,8 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
   }
   
 
-#ifdef RAPTOR_V2_AVAILABLE  
-  nstack = raptor_new_namespaces_v2(query->world->raptor_world_ptr,
-                                    (raptor_simple_message_handler)rasqal_query_simple_error, query,
-                                    1);
-
+#ifdef HAVE_RAPTOR2_API  
+  nstack = raptor_new_namespaces(query->world->raptor_world_ptr, 1);
 #else
   raptor_uri_get_handler(&uri_handler, &uri_context);
   nstack = raptor_new_namespaces(uri_handler, uri_context,
@@ -124,12 +121,10 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
   if(!nstack)
     return 1;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  xml_writer = raptor_new_xml_writer_v2(query->world->raptor_world_ptr,
-                                        nstack,
-                                        iostr,
-                                        (raptor_simple_message_handler)rasqal_query_simple_error, query,
-                                        1);
+#ifdef HAVE_RAPTOR2_API
+  xml_writer = raptor_new_xml_writer(query->world->raptor_world_ptr,
+                                     nstack,
+                                     iostr);
 #else
   xml_writer = raptor_new_xml_writer(nstack,
                                      uri_handler, uri_context,
@@ -184,11 +179,11 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef RAPTOR_V2_AVAILABLE
-      attrs[0] = raptor_new_qname_from_namespace_local_name_v2(query->world->raptor_world_ptr,
-                                                               res_ns, 
-                                                               (const unsigned char*)"name",
-                                                               (const unsigned char*)name); /* attribute value */
+#ifdef HAVE_RAPTOR2_API
+      attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+                                                            res_ns,
+                                                            (const unsigned char*)"name",
+                                                            (const unsigned char*)name); /* attribute value */
 #else
       attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
                                                             (const unsigned char*)"name",
@@ -284,11 +279,11 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef RAPTOR_V2_AVAILABLE
-      attrs[0] = raptor_new_qname_from_namespace_local_name_v2(query->world->raptor_world_ptr,
-                                                               res_ns, 
-                                                               (const unsigned char*)"name",
-                                                               name);
+#ifdef HAVE_RAPTOR2_API
+      attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+                                                            res_ns,
+                                                            (const unsigned char*)"name",
+                                                            name);
 
 #else
       attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
@@ -323,11 +318,7 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
             goto tidy;
           
           raptor_xml_writer_start_element(xml_writer, element1);
-#ifdef RAPTOR_V2_AVAILABLE
-          raptor_xml_writer_cdata(xml_writer, (const unsigned char*)raptor_uri_as_string_v2(query->world->raptor_world_ptr, l->value.uri));
-#else
           raptor_xml_writer_cdata(xml_writer, (const unsigned char*)raptor_uri_as_string(l->value.uri));
-#endif
           raptor_xml_writer_end_element(xml_writer, element1);
 
           break;
@@ -359,15 +350,17 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
             if(l->language)
               attrs[0]=raptor_new_qname(nstack,
                                         (const unsigned char*)"xml:lang",
-                                        (const unsigned char*)l->language,
-                                        (raptor_simple_message_handler)rasqal_query_simple_error, query);
+                                        (const unsigned char*)l->language
+#ifndef HAVE_RAPTOR2_API
+                                        , (raptor_simple_message_handler)rasqal_query_simple_error, query
+#endif
+                                        );
             else
-#ifdef RAPTOR_V2_AVAILABLE
-              attrs[0] = raptor_new_qname_from_namespace_local_name_v2(query->world->raptor_world_ptr,
-                                                                       res_ns, 
-                                                                       (const unsigned char*)"datatype",
-                                                                       (const unsigned char*)raptor_uri_as_string_v2(query->world->raptor_world_ptr,
-                                                                                                                     l->datatype));
+#ifdef HAVE_RAPTOR2_API
+              attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+                                                                    res_ns,
+                                                                    (const unsigned char*)"datatype",
+                                                                    (const unsigned char*)raptor_uri_as_string(l->datatype));
 #else
               attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
                                                                     (const unsigned char*)"datatype",
@@ -403,6 +396,7 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
         case RASQAL_LITERAL_DECIMAL:
         case RASQAL_LITERAL_DATETIME:
         case RASQAL_LITERAL_UDT:
+        case RASQAL_LITERAL_INTEGER_SUBTYPE:
 
         case RASQAL_LITERAL_UNKNOWN:
         default:
@@ -521,7 +515,9 @@ typedef struct
   raptor_sax2* sax2;
   raptor_locator locator;
   int depth; /* element depth */
+#ifndef HAVE_RAPTOR2_API
   raptor_error_handlers error_handlers; /* SAX2 error handler */
+#endif
 
   /* SPARQL XML Results parsing */
   rasqal_sparql_xml_read_state state; /* state */
@@ -633,7 +629,7 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
         rasqal_variable *v;
         
         var_name = (unsigned char*)RASQAL_MALLOC(cstring, con->name_length+1);
-        strncpy((char*)var_name, con->name, con->name_length+1);
+        memcpy(var_name, con->name, con->name_length + 1);
 
         v = rasqal_variables_table_add(con->vars_table,
                                        RASQAL_VARIABLE_TYPE_NORMAL,
@@ -748,19 +744,17 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         char* language_str=NULL;
 
         lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
-        strncpy((char*)lvalue, con->value, con->value_len+1);
+        memcpy(lvalue, con->value, con->value_len + 1);
         if(con->datatype)
-#ifdef RAPTOR_V2_AVAILABLE
-          datatype_uri = raptor_new_uri_v2(con->world->raptor_world_ptr, (const unsigned char*)con->datatype);
-#else
-          datatype_uri = raptor_new_uri((const unsigned char*)con->datatype);
-#endif
+          datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->datatype);
         if(con->language) {
           language_str=(char*)RASQAL_MALLOC(cstring, strlen(con->language)+1);
           strcpy(language_str, con->language);
         }
-        l=rasqal_new_string_literal_node(con->world, lvalue, language_str, datatype_uri);
+        l = rasqal_new_string_literal_node(con->world, lvalue, language_str,
+                                           datatype_uri);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
+        rasqal_free_literal(l);
         RASQAL_DEBUG3("Saving row result %d string value at offset %d\n",
                       con->offset, con->result_offset);
       }
@@ -771,9 +765,10 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         rasqal_literal* l;
         unsigned char* lvalue;
         lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
-        strncpy((char*)lvalue, con->value, con->value_len+1);
-        l=rasqal_new_simple_literal(con->world, RASQAL_LITERAL_BLANK, lvalue);
+        memcpy(lvalue, con->value, con->value_len + 1);
+        l = rasqal_new_simple_literal(con->world, RASQAL_LITERAL_BLANK, lvalue);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
+        rasqal_free_literal(l);
         RASQAL_DEBUG3("Saving row result %d bnode value at offset %d\n",
                       con->offset, con->result_offset);
       }
@@ -783,13 +778,10 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
       if(1) {
         raptor_uri* uri;
         rasqal_literal* l;
-#ifdef RAPTOR_V2_AVAILABLE
-        uri = raptor_new_uri_v2(con->world->raptor_world_ptr, (const unsigned char*)con->value);
-#else
-        uri = raptor_new_uri((const unsigned char*)con->value);
-#endif
+        uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->value);
         l = rasqal_new_uri_literal(con->world, uri);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
+        rasqal_free_literal(l);
         RASQAL_DEBUG3("Saving row result %d uri value at offset %d\n",
                       con->offset, con->result_offset);
       }
@@ -849,11 +841,7 @@ rasqal_rowsource_sparql_xml_finish(rasqal_rowsource* rowsource, void *user_data)
   con=(rasqal_rowsource_sparql_xml_context*)user_data;
 
   if(con->base_uri)
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_free_uri_v2(con->world->raptor_world_ptr, con->base_uri);
-#else
     raptor_free_uri(con->base_uri);
-#endif
 
   if(con->sax2)
     raptor_free_sax2(con->sax2);
@@ -880,8 +868,9 @@ rasqal_rowsource_sparql_xml_process(rasqal_rowsource_sparql_xml_context* con)
   while(!raptor_iostream_read_eof(con->iostr)) {
     size_t read_len;
     
-    read_len=raptor_iostream_read_bytes(con->iostr, (char*)con->buffer,
-                                        1, FILE_READ_BUF_SIZE);
+    read_len = raptor_iostream_read_bytes((char*)con->buffer, 1,
+                                          FILE_READ_BUF_SIZE,
+                                          con->iostr);
     if(read_len > 0) {
       RASQAL_DEBUG2("processing %d bytes\n", (int)read_len);
       raptor_sax2_parse_chunk(con->sax2, con->buffer, read_len, 0);
@@ -977,23 +966,19 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
     return NULL;
 
   con->world=world;
-#ifdef RAPTOR_V2_AVAILABLE
-  con->base_uri = base_uri ? raptor_uri_copy_v2(world->raptor_world_ptr, base_uri) : NULL;
-#else
   con->base_uri = base_uri ? raptor_uri_copy(base_uri) : NULL;
-#endif
   con->iostr=iostr;
 
   con->locator.uri=base_uri;
 
-  con->error_handlers.locator=&con->locator;
-#ifdef RAPTOR_V2_AVAILABLE
-  raptor_error_handlers_init_v2(world->raptor_world_ptr, &con->error_handlers);
+#ifdef HAVE_RAPTOR2_API
+  con->sax2 = raptor_new_sax2(world->raptor_world_ptr, &con->locator, con);
 #else
+  con->error_handlers.locator = &con->locator;
   raptor_error_handlers_init(&con->error_handlers);
+
+  con->sax2 = raptor_new_sax2(con, &con->error_handlers);
 #endif
-  
-  con->sax2=raptor_new_sax2(con, &con->error_handlers);
   if(!con->sax2)
     return NULL;
   
@@ -1001,13 +986,15 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
                                         rasqal_sparql_xml_sax2_start_element_handler);
   raptor_sax2_set_characters_handler(con->sax2,
                                      rasqal_sparql_xml_sax2_characters_handler);
-  raptor_sax2_set_characters_handler(con->sax2,
-                                     (raptor_sax2_characters_handler)rasqal_sparql_xml_sax2_characters_handler);
 
   raptor_sax2_set_end_element_handler(con->sax2,
                                       rasqal_sparql_xml_sax2_end_element_handler);
 
-  con->results_sequence=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row, (raptor_sequence_print_handler*)rasqal_row_print);
+#ifdef HAVE_RAPTOR2_API
+  con->results_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row, (raptor_data_print_handler)rasqal_row_print);
+#else
+  con->results_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row, (raptor_sequence_print_handler*)rasqal_row_print);
+#endif
 
   con->vars_table = rasqal_new_variables_table_from_variables_table(vars_table);
   
