@@ -2,7 +2,7 @@
  *
  * rasqal_general.c - Rasqal library startup, shutdown and factories
  *
- * Copyright (C) 2004-2009, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2004-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -47,9 +47,9 @@ static void rasqal_delete_query_language_factories(rasqal_world*);
 
 /* statics */
 
-const char * const rasqal_short_copyright_string = "Copyright 2003-2009 David Beckett.  Copyright 2003-2005 University of Bristol";
+const char * const rasqal_short_copyright_string = "Copyright 2003-2010 David Beckett.  Copyright 2003-2005 University of Bristol";
 
-const char * const rasqal_copyright_string = "Copyright (C) 2003-2009 David Beckett - http://www.dajobe.org/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
+const char * const rasqal_copyright_string = "Copyright (C) 2003-2010 David Beckett - http://www.dajobe.org/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
 
 const char * const rasqal_license_string = "LGPL 2.1 or newer, GPL 2 or newer, Apache 2.0 or newer.\nSee http://librdf.org/rasqal/LICENSE.html for full terms.";
 
@@ -140,7 +140,7 @@ rasqal_world_open(rasqal_world *world)
   if(world->opened++)
     return 0; /* not an error */
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   /* Create and init a raptor_world unless one is provided externally with rasqal_world_set_raptor() */
   if(!world->raptor_world_ptr) {
     world->raptor_world_ptr = raptor_new_world();
@@ -163,11 +163,13 @@ rasqal_world_open(rasqal_world *world)
   if(rc)
     return rc;
 
+#ifndef HAVE_RAPTOR2_API
 /* FIXME */
 #ifndef RAPTOR_ERROR_HANDLER_MAGIC
 #define RAPTOR_ERROR_HANDLER_MAGIC 0xD00DB1FF
 #endif
-  world->error_handlers.magic=RAPTOR_ERROR_HANDLER_MAGIC;
+  world->error_handlers.magic = RAPTOR_ERROR_HANDLER_MAGIC;
+#endif
 
   /* last one declared is the default - RDQL */
 
@@ -224,7 +226,8 @@ rasqal_world_open(rasqal_world *world)
 void
 rasqal_free_world(rasqal_world* world) 
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(world, rasqal_world);
+  if(!world)
+    return;
   
   rasqal_finish_result_formats(world);
   rasqal_finish_query_results();
@@ -239,7 +242,7 @@ rasqal_free_world(rasqal_world* world)
 
   rasqal_uri_finish(world);
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   if(world->raptor_world_ptr && world->raptor_world_allocated_here)
     raptor_free_world(world->raptor_world_ptr);
 #else
@@ -289,6 +292,27 @@ rasqal_world_get_raptor(rasqal_world* world)
 }
 
 
+#ifdef HAVE_RAPTOR2_API
+/**
+ * rasqal_world_set_log_handler:
+ * @world: rasqal_world object
+ * @user_data: user data for log handler function
+ * @handler: log handler function
+ *
+ * Set the log handler for this rasqal_world.
+ **/
+void
+rasqal_world_set_log_handler(rasqal_world* world, void *user_data,
+                             raptor_log_handler handler)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(world, rasqal_world);
+
+  world->log_handler = handler;
+  world->log_handler_user_data = user_data;
+}
+#endif
+
+
 /* helper functions */
 
 /*
@@ -440,6 +464,8 @@ rasqal_get_query_language_factory(rasqal_world *world, const char *name,
 {
   rasqal_query_language_factory *factory;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
+
   /* for compatibility with old API that does not call this - FIXME Remove V2 */
   rasqal_world_open(world);
   
@@ -490,6 +516,10 @@ rasqal_languages_enumerate(rasqal_world *world,
   unsigned int i;
   rasqal_query_language_factory *factory;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
+  if(!name && !label && !uri_string)
+    return 1;
+  
   /* for compatibility with old API that does not call this - FIXME Remove V2 */
   rasqal_world_open(world);
   
@@ -523,19 +553,35 @@ rasqal_languages_enumerate(rasqal_world *world,
  * Return value: non 0 if name is a known query language
  */
 int
-rasqal_language_name_check(rasqal_world* world, const char *name) {
+rasqal_language_name_check(rasqal_world* world, const char *name)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 0);
+
   return (rasqal_get_query_language_factory(world, name, NULL) != NULL);
 }
 
 
 static const char* const rasqal_log_level_labels[RAPTOR_LOG_LEVEL_LAST+1]={
+#ifdef HAVE_RAPTOR2_API
+  /* raptor2 levels */
+ "none",
+ "trace",
+ "debug",
+ "info",
+ "warn",
+ "error",
+ "fatal"
+#else
+  /* raptor1 levels */
   "none",
   "fatal error",
   "error",
   "warning"
+#endif
 };
 
 
+
 /* internal */
 void
 rasqal_log_error_simple(rasqal_world* world, raptor_log_level level,
@@ -559,8 +605,14 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
 {
   char *buffer;
   size_t length;
-  raptor_message_handler handler=world->error_handlers.handlers[level].handler;
-  void* handler_data=world->error_handlers.handlers[level].user_data;
+#ifdef HAVE_RAPTOR2_API
+  raptor_log_message logmsg;
+  raptor_log_handler handler = world->log_handler;
+  void* handler_data = world->log_handler_user_data;
+#else
+  raptor_message_handler handler = world->error_handlers.handlers[level].handler;
+  void* handler_data = world->error_handlers.handlers[level].user_data;
+#endif
   
   if(level == RAPTOR_LOG_LEVEL_NONE)
     return;
@@ -568,8 +620,8 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
   buffer=raptor_vsnprintf(message, arguments);
   if(!buffer) {
     if(locator) {
-#ifdef RAPTOR_V2_AVAILABLE
-      raptor_print_locator_v2(world->raptor_world_ptr, stderr, locator);
+#ifdef HAVE_RAPTOR2_API
+      raptor_locator_print(locator, stderr);
 #else
       raptor_print_locator(stderr, locator);
 #endif
@@ -587,15 +639,25 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
   if(buffer[length-1]=='\n')
     buffer[length-1]='\0';
   
-  if(handler)
+  if(handler) {
     /* This is the single place in rasqal that the user error handler
      * functions are called.
      */
+#ifdef HAVE_RAPTOR2_API
+    /* raptor2 raptor_log_handler */
+    logmsg.code = -1; /* no information to put as code */
+    logmsg.level = level;
+    logmsg.locator = locator;
+    logmsg.text = buffer;
+    handler(handler_data, &logmsg);
+#else
+    /* raptor1 raptor_message_handler */
     handler(handler_data, locator, buffer);
-  else {
+#endif
+  } else {
     if(locator) {
-#ifdef RAPTOR_V2_AVAILABLE
-      raptor_print_locator_v2(world->raptor_world_ptr, stderr, locator);
+#ifdef HAVE_RAPTOR2_API
+      raptor_locator_print(locator, stderr);
 #else
       raptor_print_locator(stderr, locator);
 #endif
@@ -668,21 +730,23 @@ rasqal_basename(const char *name)
 unsigned char*
 rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
                                    size_t *dest_lenp,
-                                   raptor_simple_message_handler error_handler,
-                                   void *error_data)
+                                   int (*error_handler)(rasqal_query *error_data, const char *message, ...),
+                                   rasqal_query* error_data)
 {
   const unsigned char *p=src;
   size_t ulen=0;
   unsigned long unichar=0;
   unsigned char *result;
   unsigned char *dest;
+  unsigned char *endp;
   int n;
   
   result=(unsigned char*)RASQAL_MALLOC(cstring, len+1);
   if(!result)
     return NULL;
 
-  dest=result;
+  dest = result;
+  endp = result + len;
 
   /* find end of string, fixing backslashed characters on the way */
   while(len > 0) {
@@ -690,7 +754,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
 
     if(c > 0x7f) {
       /* just copy the UTF-8 bytes through */
-      size_t unichar_len=raptor_utf8_to_unicode_char(NULL, (const unsigned char*)p, len+1);
+      size_t unichar_len = raptor_unicode_utf8_string_get_char((const unsigned char*)p, len+1, NULL);
       if(unichar_len > len) {
         if(error_handler)
           error_handler(error_data, "UTF-8 encoding error at character %d (0x%02X) found.", c, c);
@@ -752,7 +816,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
           break;
         }
           
-        dest+=raptor_unicode_char_to_utf8(unichar, dest);
+        dest += raptor_unicode_utf8_string_put_char(unichar, dest, endp - dest);
         break;
 
       default:
@@ -778,24 +842,13 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
 int
 rasqal_uri_init(rasqal_world* world) 
 {
-#ifdef RAPTOR_V2_AVAILABLE
-  world->rdf_namespace_uri = raptor_new_uri_v2(world->raptor_world_ptr, raptor_rdf_namespace_uri);
-#else
-  world->rdf_namespace_uri = raptor_new_uri(raptor_rdf_namespace_uri);
-#endif
+  world->rdf_namespace_uri = raptor_new_uri(world->raptor_world_ptr, raptor_rdf_namespace_uri);
   if(!world->rdf_namespace_uri)
     goto oom;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  world->rdf_first_uri = raptor_new_uri_from_uri_local_name_v2(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"first");
-  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name_v2(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"rest");
-  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name_v2(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"nil");
-#else
-  world->rdf_first_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"first");
-  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"rest");
-  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"nil");
-#endif
-
+  world->rdf_first_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"first");
+  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"rest");
+  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"nil");
   if(!world->rdf_first_uri || !world->rdf_rest_uri || !world->rdf_nil_uri)
     goto oom;
 
@@ -812,24 +865,6 @@ rasqal_uri_init(rasqal_world* world)
 void
 rasqal_uri_finish(rasqal_world* world) 
 {
-#ifdef RAPTOR_V2_AVAILABLE
-  if(world->rdf_first_uri) {
-    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_first_uri);
-    world->rdf_first_uri = NULL;
-  }
-  if(world->rdf_rest_uri) {
-    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_rest_uri);
-    world->rdf_rest_uri = NULL;
-  }
-  if(world->rdf_nil_uri) {
-    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_nil_uri);
-    world->rdf_nil_uri = NULL;
-  }
-  if(world->rdf_namespace_uri) {
-    raptor_free_uri_v2(world->raptor_world_ptr, world->rdf_namespace_uri);
-    world->rdf_namespace_uri = NULL;
-  }
-#else
   if(world->rdf_first_uri) {
     raptor_free_uri(world->rdf_first_uri);
     world->rdf_first_uri=NULL;
@@ -846,16 +881,17 @@ rasqal_uri_finish(rasqal_world* world)
     raptor_free_uri(world->rdf_namespace_uri);
     world->rdf_namespace_uri=NULL;
   }
-#endif
 }
 
 
 /**
- * rasqal_query_set_default_generate_bnodeid_parameters - Set default bnodeid generation parameters
- * @rdf_query: #rasqal_query object
+ * rasqal_world_set_default_generate_bnodeid_parameters:
+ * @world: #rasqal_world object
  * @prefix: prefix string
  * @base: integer base identifier
  *
+ * Set default bnodeid generation parameters
+ *
  * Sets the parameters for the default algorithm used to generate
  * blank node IDs.  The default algorithm uses both @prefix and @base
  * to generate a new identifier.  The exact identifier generated is
@@ -863,73 +899,86 @@ rasqal_uri_finish(rasqal_world* world)
  * but will use both parts.
  *
  * For finer control of the generated identifiers, use
- * rasqal_set_default_generate_bnodeid_handler()
+ * rasqal_world_set_generate_bnodeid_handler()
  *
  * If prefix is NULL, the default prefix is used (currently "bnodeid")
  * If base is less than 1, it is initialised to 1.
- * 
+ *
+ * Return value: non-0 on failure
  **/
-void
-rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, 
+int
+rasqal_world_set_default_generate_bnodeid_parameters(rasqal_world* world, 
                                                      char *prefix, int base)
 {
-  char *prefix_copy=NULL;
-  size_t length=0;
+  char *prefix_copy = NULL;
+  size_t length = 0;
 
-  if(--base<0)
-    base=0;
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
+
+  if(--base < 0)
+    base = 0;
 
   if(prefix) {
-    length=strlen(prefix);
+    length = strlen(prefix);
     
-    prefix_copy=(char*)RASQAL_MALLOC(cstring, length+1);
+    prefix_copy = (char*)RASQAL_MALLOC(cstring, length+1);
     if(!prefix_copy)
-      return;
+      return 1;
     strcpy(prefix_copy, prefix);
   }
   
-  if(rdf_query->default_generate_bnodeid_handler_prefix)
-    RASQAL_FREE(cstring, rdf_query->default_generate_bnodeid_handler_prefix);
+  if(world->default_generate_bnodeid_handler_prefix)
+    RASQAL_FREE(cstring, world->default_generate_bnodeid_handler_prefix);
 
-  rdf_query->default_generate_bnodeid_handler_prefix=prefix_copy;
-  rdf_query->default_generate_bnodeid_handler_prefix_length=length;
-  rdf_query->default_generate_bnodeid_handler_base=base;
+  world->default_generate_bnodeid_handler_prefix = prefix_copy;
+  world->default_generate_bnodeid_handler_prefix_length = length;
+  world->default_generate_bnodeid_handler_base = base;
+
+  return 0;
 }
 
 
 /**
- * rasqal_query_set_generate_bnodeid_handler:
- * @query: #rasqal_query query object
+ * rasqal_world_set_generate_bnodeid_handler:
+ * @world: #rasqal_world object
  * @user_data: user data pointer for callback
  * @handler: generate blank ID callback function
  *
- * Set the generate blank node ID handler function for the query.
+ * Set the generate blank node ID handler function
  *
- * Sets the function to generate blank node IDs for the query.
- * The handler is called with a pointer to the rasqal_query, the
+ * Sets the function to generate blank node IDs.
+ * The handler is called with a pointer to the rasqal_world, the
  * @user_data pointer and a user_bnodeid which is the value of
  * a user-provided blank node identifier (may be NULL).
  * It can either be returned directly as the generated value when present or
  * modified.  The passed in value must be free()d if it is not used.
  *
  * If handler is NULL, the default method is used
- * 
+ *
+ * If set, this overrides any bnodeid handler set via
+ * rasqal_query_set_generate_bnodeid_handler().
+ *  
+ * Return value: non-0 on failure
  **/
-void
-rasqal_query_set_generate_bnodeid_handler(rasqal_query* query,
+int
+rasqal_world_set_generate_bnodeid_handler(rasqal_world* world,
                                           void *user_data,
-                                          rasqal_generate_bnodeid_handler handler)
+                                          rasqal_generate_bnodeid_handler2 handler)
 {
-  query->generate_bnodeid_handler_user_data=user_data;
-  query->generate_bnodeid_handler=handler;
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
+
+  world->generate_bnodeid_handler_user_data = user_data;
+  world->generate_bnodeid_handler = handler;
+
+  return 0;  
 }
 
 
-static unsigned char*
-rasqal_default_generate_bnodeid_handler(void *user_data,
-                                        unsigned char *user_bnodeid) 
+unsigned char*
+rasqal_world_default_generate_bnodeid_handler(void *user_data,
+                                              unsigned char *user_bnodeid) 
 {
-  rasqal_query *rdf_query=(rasqal_query *)user_data;
+  rasqal_world *world = (rasqal_world*)user_data;
   int id;
   unsigned char *buffer;
   int length;
@@ -938,25 +987,25 @@ rasqal_default_generate_bnodeid_handler(void *user_data,
   if(user_bnodeid)
     return user_bnodeid;
 
-  id=++rdf_query->default_generate_bnodeid_handler_base;
+  id = ++world->default_generate_bnodeid_handler_base;
 
-  tmpid=id;
-  length=2; /* min length 1 + \0 */
-  while(tmpid/=10)
+  tmpid = id;
+  length = 2; /* min length 1 + \0 */
+  while(tmpid /= 10)
     length++;
 
-  if(rdf_query->default_generate_bnodeid_handler_prefix)
-    length += rdf_query->default_generate_bnodeid_handler_prefix_length;
+  if(world->default_generate_bnodeid_handler_prefix)
+    length += world->default_generate_bnodeid_handler_prefix_length;
   else
     length += 7; /* bnodeid */
   
-  buffer=(unsigned char*)RASQAL_MALLOC(cstring, length);
+  buffer = (unsigned char*)RASQAL_MALLOC(cstring, length);
   if(!buffer)
     return NULL;
-  if(rdf_query->default_generate_bnodeid_handler_prefix) {
-    strncpy((char*)buffer, rdf_query->default_generate_bnodeid_handler_prefix,
-            rdf_query->default_generate_bnodeid_handler_prefix_length);
-    sprintf((char*)buffer + rdf_query->default_generate_bnodeid_handler_prefix_length,
+  if(world->default_generate_bnodeid_handler_prefix) {
+    memcpy(buffer, world->default_generate_bnodeid_handler_prefix,
+           world->default_generate_bnodeid_handler_prefix_length);
+    sprintf((char*)buffer + world->default_generate_bnodeid_handler_prefix_length,
             "%d", id);
   } else 
     sprintf((char*)buffer, "bnodeid%d", id);
@@ -966,22 +1015,22 @@ rasqal_default_generate_bnodeid_handler(void *user_data,
 
 
 /*
- * rasqal_query_generate_bnodeid - Default generate id - internal
+ * rasqal_generate_bnodeid:
+ *
+ * Internal - Default generate ID
  */
 unsigned char*
-rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
+rasqal_world_generate_bnodeid(rasqal_world* world,
                               unsigned char *user_bnodeid)
 {
-  if(rdf_query->generate_bnodeid_handler)
-    return rdf_query->generate_bnodeid_handler(rdf_query, 
-                                               rdf_query->generate_bnodeid_handler_user_data, user_bnodeid);
+  if(world->generate_bnodeid_handler)
+    return world->generate_bnodeid_handler(world, 
+                                           world->generate_bnodeid_handler_user_data, user_bnodeid);
   else
-    return rasqal_default_generate_bnodeid_handler(rdf_query, user_bnodeid);
+    return rasqal_world_default_generate_bnodeid_handler(world, user_bnodeid);
 }
 
 
-
-
 /**
  * rasqal_free_memory:
  * @ptr: memory pointer
@@ -996,7 +1045,8 @@ rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
 void
 rasqal_free_memory(void *ptr)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(ptr, memory);
+  if(!ptr)
+    return;
   
   RASQAL_FREE(void, ptr);
 }
@@ -1111,24 +1161,3 @@ rasqal_sign_free(void *ptr)
   free(p);
 }
 #endif
-
-
-#if defined (RASQAL_DEBUG) && defined(HAVE_DMALLOC_H) && defined(RASQAL_MEMORY_DEBUG_DMALLOC)
-
-#undef malloc
-void*
-rasqal_system_malloc(size_t size)
-{
-  return malloc(size);
-}
-
-#undef free
-void
-rasqal_system_free(void *ptr)
-{
-  return free(ptr);
-  
-}
-
-#endif
-
