@@ -60,10 +60,9 @@ rasqal_new_graph_pattern(rasqal_query* query,
 {
   rasqal_graph_pattern* gp;
 
-  if(!query)
-    return NULL;
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
   
-  gp=(rasqal_graph_pattern*)RASQAL_CALLOC(rasqal_graph_pattern, 1, sizeof(rasqal_graph_pattern));
+  gp = (rasqal_graph_pattern*)RASQAL_CALLOC(rasqal_graph_pattern, 1, sizeof(rasqal_graph_pattern));
   if(!gp)
     return NULL;
 
@@ -100,10 +99,10 @@ rasqal_new_basic_graph_pattern(rasqal_query* query,
 {
   rasqal_graph_pattern* gp;
 
-  if(!triples)
-    return NULL;
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(triples, raptor_sequence, NULL);
   
-  gp=rasqal_new_graph_pattern(query, RASQAL_GRAPH_PATTERN_OPERATOR_BASIC);
+  gp = rasqal_new_graph_pattern(query, RASQAL_GRAPH_PATTERN_OPERATOR_BASIC);
   if(!gp)
     return NULL;
 
@@ -118,7 +117,7 @@ rasqal_new_basic_graph_pattern(rasqal_query* query,
 /*
  * rasqal_new_graph_pattern_from_sequence:
  * @query: #rasqal_graph_pattern query object
- * @graph_patterns: sequence containing the graph patterns
+ * @graph_patterns: sequence containing the graph patterns (or NULL)
  * @operator: enum #rasqal_graph_pattern_operator such as
  * RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL
  *
@@ -133,9 +132,11 @@ rasqal_new_graph_pattern_from_sequence(rasqal_query* query,
 {
   rasqal_graph_pattern* gp;
 
-  gp=rasqal_new_graph_pattern(query, op);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+  
+  gp = rasqal_new_graph_pattern(query, op);
   if(!gp) {
-  	if(graph_patterns)
+    if(graph_patterns)
       raptor_free_sequence(graph_patterns);
     return NULL;
   }
@@ -160,7 +161,10 @@ rasqal_new_filter_graph_pattern(rasqal_query* query,
 {
   rasqal_graph_pattern* gp;
 
-  gp=rasqal_new_graph_pattern(query, RASQAL_GRAPH_PATTERN_OPERATOR_FILTER);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(expr, rasqal_expression, NULL);
+  
+  gp = rasqal_new_graph_pattern(query, RASQAL_GRAPH_PATTERN_OPERATOR_FILTER);
   if(!gp) {
     rasqal_free_expression(expr);
     return NULL;
@@ -192,6 +196,10 @@ rasqal_new_let_graph_pattern(rasqal_query *query,
 {
   rasqal_graph_pattern* gp;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(var, rasqal_variable, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(expr, rasqal_expression, NULL);
+  
   gp = rasqal_new_graph_pattern(query, RASQAL_GRAPH_PATTERN_OPERATOR_LET);
   if(!gp) {
     rasqal_free_expression(expr);
@@ -259,6 +267,9 @@ int
 rasqal_graph_pattern_set_filter_expression(rasqal_graph_pattern* gp,
                                            rasqal_expression* expr)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(gp, rasqal_graph_pattern, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(expr, rasqal_expression, 1);
+  
   if(gp->filter_expression)
     rasqal_free_expression(gp->filter_expression);
   gp->filter_expression = expr;
@@ -277,6 +288,8 @@ rasqal_graph_pattern_set_filter_expression(rasqal_graph_pattern* gp,
 rasqal_expression*
 rasqal_graph_pattern_get_filter_expression(rasqal_graph_pattern* gp)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(gp, rasqal_graph_pattern, NULL);
+
   return gp->filter_expression;
 }
 
@@ -295,6 +308,8 @@ rasqal_graph_pattern_get_filter_expression(rasqal_graph_pattern* gp)
 rasqal_graph_pattern_operator
 rasqal_graph_pattern_get_operator(rasqal_graph_pattern* graph_pattern)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern, rasqal_graph_pattern, RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN);
+
   return graph_pattern->op;
 }
 
@@ -344,7 +359,7 @@ rasqal_graph_pattern_write_indent(raptor_iostream *iostr, int indent)
 {
   while(indent > 0) {
     int sp=(indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
-    raptor_iostream_write_bytes(iostr, spaces, sizeof(char), sp);
+    raptor_iostream_write_bytes(spaces, sizeof(char), sp, iostr);
     indent -= sp;
   }
 }
@@ -354,11 +369,11 @@ static void
 rasqal_graph_pattern_write_plurals(raptor_iostream *iostr,
                                    const char* label, int value)
 {
-  raptor_iostream_write_decimal(iostr, value);
-  raptor_iostream_write_byte(iostr, ' ');
-  raptor_iostream_write_string(iostr, label);
+  raptor_iostream_decimal_write(value, iostr);
+  raptor_iostream_write_byte(' ', iostr);
+  raptor_iostream_string_write(label, iostr);
   if(value != 1)
-    raptor_iostream_write_byte(iostr, 's');
+    raptor_iostream_write_byte('s', iostr);
 }
 
 
@@ -377,16 +392,16 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
 {
   int pending_nl=0;
   
-  raptor_iostream_write_counted_string(iostr, "graph pattern", 13);
+  raptor_iostream_counted_string_write("graph pattern", 13, iostr);
   if(gp->gp_index >= 0) {
-    raptor_iostream_write_byte(iostr, '[');
-    raptor_iostream_write_decimal(iostr, gp->gp_index);
-    raptor_iostream_write_byte(iostr, ']');
+    raptor_iostream_write_byte('[', iostr);
+    raptor_iostream_decimal_write(gp->gp_index, iostr);
+    raptor_iostream_write_byte(']', iostr);
   }
-  raptor_iostream_write_byte(iostr, ' ');
-  raptor_iostream_write_string(iostr, 
-                               rasqal_graph_pattern_operator_as_string(gp->op));
-  raptor_iostream_write_byte(iostr, '(');
+  raptor_iostream_write_byte(' ', iostr);
+  raptor_iostream_string_write(rasqal_graph_pattern_operator_as_string(gp->op),
+                               iostr);
+  raptor_iostream_write_byte('(', iostr);
 
   if(indent >= 0)
     indent+= 2;
@@ -394,12 +409,12 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
   if(gp->triples) {
     int size=gp->end_column - gp->start_column +1;
     int i;
-    raptor_iostream_write_counted_string(iostr, "over ", 5);
+    raptor_iostream_counted_string_write("over ", 5, iostr);
     rasqal_graph_pattern_write_plurals(iostr, "triple", size);
-    raptor_iostream_write_byte(iostr, '[');
+    raptor_iostream_write_byte('[', iostr);
 
     if(indent >= 0) {
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
       indent+= 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
@@ -407,36 +422,36 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
     for(i=gp->start_column; i <= gp->end_column; i++) {
       rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(gp->triples, i);
       if(i > gp->start_column) {
-        raptor_iostream_write_counted_string(iostr, " ,", 2);
+        raptor_iostream_counted_string_write(" ,", 2, iostr);
 
         if(indent >= 0) {
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
           rasqal_graph_pattern_write_indent(iostr, indent);
         }
       }
       rasqal_triple_write(t, iostr);
     }
     if(indent >= 0) {
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
       indent-= 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
-    raptor_iostream_write_byte(iostr, ']');
+    raptor_iostream_write_byte(']', iostr);
 
     pending_nl=1;
   }
 
   if(gp->origin) {
     if(pending_nl) {
-      raptor_iostream_write_counted_string(iostr, " ,", 2);
+      raptor_iostream_counted_string_write(" ,", 2, iostr);
 
       if(indent >= 0) {
-        raptor_iostream_write_byte(iostr, '\n');
+        raptor_iostream_write_byte('\n', iostr);
         rasqal_graph_pattern_write_indent(iostr, indent);
       }
     }
 
-    raptor_iostream_write_counted_string(iostr, "origin ", 7);
+    raptor_iostream_counted_string_write("origin ", 7, iostr);
 
     rasqal_literal_write(gp->origin, iostr);
 
@@ -448,19 +463,19 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
     int i;
 
     if(pending_nl) {
-      raptor_iostream_write_counted_string(iostr, " ,", 2);
+      raptor_iostream_counted_string_write(" ,", 2, iostr);
       if(indent >= 0) {
-        raptor_iostream_write_byte(iostr, '\n');
+        raptor_iostream_write_byte('\n', iostr);
         rasqal_graph_pattern_write_indent(iostr, indent);
       }
     }
     
-    raptor_iostream_write_counted_string(iostr, "over ", 5);
+    raptor_iostream_counted_string_write("over ", 5, iostr);
     rasqal_graph_pattern_write_plurals(iostr, "graph pattern", size);
-    raptor_iostream_write_byte(iostr, '[');
+    raptor_iostream_write_byte('[', iostr);
 
     if(indent >= 0) {
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
       indent+= 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
@@ -469,48 +484,48 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
       rasqal_graph_pattern* sgp;
       sgp=(rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
       if(i) {
-        raptor_iostream_write_counted_string(iostr, " ,", 2);
+        raptor_iostream_counted_string_write(" ,", 2, iostr);
         if(indent >= 0) {
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
           rasqal_graph_pattern_write_indent(iostr, indent);
         }
       }
       if(sgp)
         rasqal_graph_pattern_write_internal(sgp, iostr, indent);
       else
-        raptor_iostream_write_counted_string(iostr, "(empty)", 7);
+        raptor_iostream_counted_string_write("(empty)", 7, iostr);
     }
     if(indent >= 0) {
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
       indent-= 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
-    raptor_iostream_write_byte(iostr, ']');
+    raptor_iostream_write_byte(']', iostr);
 
     pending_nl=1;
   }
 
   if(gp->var) {
     rasqal_variable_write(gp->var, iostr);
-    raptor_iostream_write_counted_string(iostr, " := ", 4);
+    raptor_iostream_counted_string_write(" := ", 4, iostr);
     pending_nl = 0;
   }
   
   if(gp->filter_expression) {
     if(pending_nl) {
-      raptor_iostream_write_counted_string(iostr, " ,", 2);
+      raptor_iostream_counted_string_write(" ,", 2, iostr);
 
       if(indent >= 0) {
-        raptor_iostream_write_byte(iostr, '\n');
+        raptor_iostream_write_byte('\n', iostr);
         rasqal_graph_pattern_write_indent(iostr, indent);
       }
     }
     
     if(gp->triples || gp->graph_patterns)
-      raptor_iostream_write_counted_string(iostr, "with ", 5);
+      raptor_iostream_counted_string_write("with ", 5, iostr);
   
     if(indent >= 0) {
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
       indent+= 2;
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
@@ -527,12 +542,12 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
 
   if(pending_nl) {
     if(indent >= 0) {
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
       rasqal_graph_pattern_write_indent(iostr, indent);
     }
   }
   
-  raptor_iostream_write_byte(iostr, ')');
+  raptor_iostream_write_byte(')', iostr);
 
   return 0;
 }
@@ -541,20 +556,27 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
 /**
  * rasqal_graph_pattern_print:
  * @gp: the #rasqal_graph_pattern object
- * @fh: the #FILE* handle to print to
+ * @fh: the FILE* handle to print to
  *
  * Print a #rasqal_graph_pattern in a debug format.
  * 
  * The print debug format may change in any release.
  * 
+ * Return value: non-0 on failure
  **/
-void
+int
 rasqal_graph_pattern_print(rasqal_graph_pattern* gp, FILE* fh)
 {
   raptor_iostream* iostr;
-  iostr=raptor_new_iostream_to_file_handle(fh);
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(gp, rasqal_graph_pattern, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(fh, FILE*, 1);
+
+  iostr = raptor_new_iostream_to_file_handle(gp->query->world->raptor_world_ptr, fh);
   rasqal_graph_pattern_write_internal(gp, iostr, DO_INDENTING);
   raptor_free_iostream(iostr);
+
+  return 0;
 }
 
 
@@ -580,7 +602,11 @@ rasqal_graph_pattern_visit(rasqal_query *query,
   raptor_sequence *seq;
   int result;
   
-  result=fn(query, gp, user_data);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(gp, rasqal_graph_pattern, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(fn, rasqal_graph_pattern_visit_fn, 1);
+
+  result = fn(query, gp, user_data);
   if(result)
     return result;
   
@@ -618,6 +644,8 @@ rasqal_graph_pattern_visit(rasqal_query *query,
 int
 rasqal_graph_pattern_get_index(rasqal_graph_pattern* gp)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(gp, rasqal_graph_pattern, -1);
+
   return gp->gp_index;
 }
 
@@ -635,8 +663,15 @@ int
 rasqal_graph_pattern_add_sub_graph_pattern(rasqal_graph_pattern* graph_pattern,
                                            rasqal_graph_pattern* sub_graph_pattern)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern, rasqal_graph_pattern, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(sub_graph_pattern, rasqal_graph_pattern, 1);
+
   if(!graph_pattern->graph_patterns) {
-    graph_pattern->graph_patterns=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+#ifdef HAVE_RAPTOR2_API
+    graph_pattern->graph_patterns = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
+#else
+    graph_pattern->graph_patterns = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+#endif
     if(!graph_pattern->graph_patterns) {
       if(sub_graph_pattern)
         rasqal_free_graph_pattern(sub_graph_pattern);
@@ -659,6 +694,8 @@ rasqal_graph_pattern_add_sub_graph_pattern(rasqal_graph_pattern* graph_pattern,
 rasqal_triple*
 rasqal_graph_pattern_get_triple(rasqal_graph_pattern* graph_pattern, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern, rasqal_graph_pattern, NULL);
+
   if(!graph_pattern->triples)
     return NULL;
 
@@ -682,6 +719,8 @@ rasqal_graph_pattern_get_triple(rasqal_graph_pattern* graph_pattern, int idx)
 raptor_sequence*
 rasqal_graph_pattern_get_sub_graph_pattern_sequence(rasqal_graph_pattern* graph_pattern)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern, rasqal_graph_pattern, NULL);
+
   return graph_pattern->graph_patterns;
 }
 
@@ -698,6 +737,8 @@ rasqal_graph_pattern_get_sub_graph_pattern_sequence(rasqal_graph_pattern* graph_
 rasqal_graph_pattern*
 rasqal_graph_pattern_get_sub_graph_pattern(rasqal_graph_pattern* graph_pattern, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern, rasqal_graph_pattern, NULL);
+
   if(!graph_pattern->graph_patterns)
     return NULL;
 
@@ -716,6 +757,8 @@ rasqal_graph_pattern_get_sub_graph_pattern(rasqal_graph_pattern* graph_pattern,
 rasqal_literal*
 rasqal_graph_pattern_get_origin(rasqal_graph_pattern* graph_pattern)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_pattern, rasqal_graph_pattern, NULL);
+
   return graph_pattern->origin;
 }
   
@@ -797,7 +840,11 @@ rasqal_new_2_group_graph_pattern(rasqal_query* query,
 {
   raptor_sequence *seq;
 
-  seq=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
+#else
+  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+#endif
   if(!seq) {
     if(first_gp)
       rasqal_free_graph_pattern(first_gp);
@@ -835,12 +882,56 @@ int
 rasqal_graph_pattern_variable_bound_in(rasqal_graph_pattern *gp,
                                        rasqal_variable *v) 
 {
-  rasqal_query* query = gp->query;
+  rasqal_query* query;
+  int column;
+  
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(gp, rasqal_graph_pattern, 0);
 
-  int column = query->variables_bound_in[v->offset];
+  query = gp->query;
+  column = query->variables_bound_in[v->offset];
   if(column < 0)
     return 0;
   
   return (column >= gp->start_column && column <= gp->end_column);
 }
 
+
+
+/**
+ * rasqal_new_basic_graph_pattern_from_triples:
+ * @query: #rasqal_graph_pattern query object
+ * @triples: triples sequence containing the graph pattern
+ *
+ * INTERNAL - Create a new graph pattern object from a sequence of triples.
+ *
+ * The @triples become owned by the graph pattern
+ * 
+ * Return value: a new #rasqal_graph_pattern object or NULL on failure
+ **/
+rasqal_graph_pattern*
+rasqal_new_basic_graph_pattern_from_triples(rasqal_query* query,
+                                            raptor_sequence* triples)
+{
+  rasqal_graph_pattern* gp;
+  raptor_sequence *graph_triples = query->triples;
+  int offset = raptor_sequence_size(graph_triples);
+  int triple_pattern_size = 0;
+
+  if(triples) {
+    /* Move triples to end of graph triples sequence */
+    triple_pattern_size = raptor_sequence_size(triples);
+    if(raptor_sequence_join(graph_triples, triples)) {
+      raptor_free_sequence(triples);
+      return NULL;
+    }
+  }
+
+  raptor_free_sequence(triples);
+
+  gp = rasqal_new_basic_graph_pattern(query, graph_triples,
+                                      offset, 
+                                      offset + triple_pattern_size - 1);
+  return gp;
+}
+
+
