@@ -35,15 +35,6 @@ extern "C" {
 
 #ifdef RASQAL_INTERNAL
 
-/* for the memory allocation functions */
-#if defined(HAVE_DMALLOC_H) && defined(RASQAL_MEMORY_DEBUG_DMALLOC)
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#undef HAVE_STDLIB_H
-#endif
-#include <dmalloc.h>
-#endif
-
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
 #define RASQAL_PRINTF_FORMAT(string_index, first_to_check_index) \
   __attribute__((__format__(__printf__, string_index, first_to_check_index)))
@@ -344,9 +335,10 @@ struct rasqal_query_s {
   /* stuff for our user */
   void* user_data;
 
-  int default_generate_bnodeid_handler_base;
-  char *default_generate_bnodeid_handler_prefix;
-  size_t default_generate_bnodeid_handler_prefix_length;
+  /* former state for generating blank node IDs; now in world object */
+  int unused1;
+  char *unused2;
+  size_t unused3;
 
   void *generate_bnodeid_handler_user_data;
   rasqal_generate_bnodeid_handler generate_bnodeid_handler;
@@ -412,6 +404,14 @@ struct rasqal_query_s {
    *   4  RASQAL_USE_BOUND_HERE
    */
   short* variables_use_map;
+
+  /* sequence of #rasqal_update_operation when @verb is
+   * INSERT (deprecated), DELETE (deprecated) or UPDATE
+   */
+  raptor_sequence* updates;
+
+  /* sequence of (group by ...) having condition expressions */
+  raptor_sequence* having_conditions_sequence;
 };
 
 
@@ -466,7 +466,7 @@ typedef struct rasqal_rowsource_s rasqal_rowsource;
 /*
  * A row of values from a query result, usually generated by a rowsource
  */
-typedef struct {
+struct rasqal_row_s {
   /* reference count */
   int usage;
 
@@ -484,29 +484,11 @@ typedef struct {
   /* number of expressions (can be 0) */
   int order_size;
   rasqal_literal** order_values;
-} rasqal_row;
+};
 
 
 typedef struct rasqal_map_s rasqal_map;
 
-/**
- * rasqal_query_results_type:
- * @RASQAL_QUERY_RESULTS_BINDINGS: variable binding
- * @RASQAL_QUERY_RESULTS_BOOLEAN: a single boolean
- * @RASQAL_QUERY_RESULTS_GRAPH: an RDF graph
- * @RASQAL_QUERY_RESULTS_SYNTAX: a syntax
- *
- * Query result type.
- */
-
-typedef enum {
-  RASQAL_QUERY_RESULTS_BINDINGS,
-  RASQAL_QUERY_RESULTS_BOOLEAN,
-  RASQAL_QUERY_RESULTS_GRAPH,
-  RASQAL_QUERY_RESULTS_SYNTAX
-} rasqal_query_results_type;
-
-
 /**
  * rasqal_join_type:
  * @RASQAL_JOIN_TYPE_UNKNOWN: unknown join type
@@ -860,6 +842,7 @@ struct rasqal_query_results_formatter_s {
 };
 
 typedef struct {
+  rasqal_world *world;
   raptor_sequence *triples;
   rasqal_literal *value;
 } rasqal_formula;
@@ -867,26 +850,30 @@ typedef struct {
 
 /* rasqal_datetime.c */
 int rasqal_xsd_datetime_check(const unsigned char* string);
+int rasqal_xsd_date_check(const unsigned char* string);
 
 
 /* rasqal_general.c */
 char* rasqal_vsnprintf(const char* message, va_list arguments);
+unsigned char* rasqal_world_generate_bnodeid(rasqal_world* world, unsigned char *user_bnodeid);
 
 int rasqal_query_language_register_factory(rasqal_world*, const char* name, const char* label, const char* alias, const unsigned char* uri_string, void (*factory) (rasqal_query_language_factory*));
 rasqal_query_language_factory* rasqal_get_query_language_factory (rasqal_world*, const char* name, const unsigned char* uri);
 void rasqal_log_error_simple(rasqal_world* world, raptor_log_level level, raptor_locator* locator, const char* message, ...) RASQAL_PRINTF_FORMAT(4, 5);
 void rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level, raptor_locator* locator, const char* message, va_list arguments) RASQAL_PRINTF_FORMAT(4, 0);
-void rasqal_log_error(raptor_log_level level, raptor_message_handler handler, void* handler_data, raptor_locator* locator, const char* message);
 void rasqal_query_simple_error(void* query, const char *message, ...) RASQAL_PRINTF_FORMAT(2, 3);
 
 const char* rasqal_basename(const char* name);
+unsigned char* rasqal_world_default_generate_bnodeid_handler(void *user_data, unsigned char *user_bnodeid);
 
+unsigned char* rasqal_escaped_name_to_utf8_string(const unsigned char* src, size_t len, size_t* dest_lenp, int (*error_handler)(rasqal_query *error_data, const char *message, ...) RASQAL_PRINTF_FORMAT(2, 3), rasqal_query* error_data);
 
 /* rasqal_graph_pattern.c */
-unsigned char* rasqal_escaped_name_to_utf8_string(const unsigned char* src, size_t len, size_t* dest_lenp, raptor_simple_message_handler error_handler, void* error_data);
 unsigned char* rasqal_query_generate_bnodeid(rasqal_query* rdf_query, unsigned char *user_bnodeid);
 
 rasqal_graph_pattern* rasqal_new_basic_graph_pattern_from_formula(rasqal_query* query, rasqal_formula* formula);
+rasqal_graph_pattern* rasqal_new_basic_graph_pattern_from_triples(rasqal_query* query, raptor_sequence* triples);
+
 rasqal_graph_pattern* rasqal_new_2_group_graph_pattern(rasqal_query* query, rasqal_graph_pattern* first_gp, rasqal_graph_pattern* second_gp);
 
 /* rdql_parser.y */
@@ -917,7 +904,7 @@ int rasqal_literal_as_boolean(rasqal_literal* literal, int* error);
 int rasqal_literal_as_integer(rasqal_literal* l, int* error);
 double rasqal_literal_as_floating(rasqal_literal* l, int* error);
 raptor_uri* rasqal_literal_as_uri(rasqal_literal* l);
-int rasqal_literal_string_to_native(rasqal_literal *l, raptor_simple_message_handler error_handler, void *error_data, int flags);
+int rasqal_literal_string_to_native(rasqal_literal *l, int flags);
 int rasqal_literal_has_qname(rasqal_literal* l);
 int rasqal_literal_expand_qname(void* user_data, rasqal_literal* l);
 int rasqal_literal_is_constant(rasqal_literal* l);
@@ -961,9 +948,9 @@ int rasqal_uri_init(rasqal_world*);
 void rasqal_uri_finish(rasqal_world*);
 
 /* rasqal_literal.c */
-rasqal_formula* rasqal_new_formula(void);
+rasqal_formula* rasqal_new_formula(rasqal_world* world);
 void rasqal_free_formula(rasqal_formula* formula);
-void rasqal_formula_print(rasqal_formula* formula, FILE *stream);
+int rasqal_formula_print(rasqal_formula* formula, FILE *stream);
 rasqal_formula* rasqal_formula_join(rasqal_formula* first_formula, rasqal_formula* second_formula);
 
 /* The following should be public eventually in rasqal.h or raptor.h or ...? */
@@ -994,15 +981,19 @@ void rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr);
 /* rasqal_map.c */
 typedef void (*rasqal_map_visit_fn)(void *key, void *value, void *user_data);
 
+#ifdef HAVE_RAPTOR2_API
+rasqal_map* rasqal_new_map(rasqal_compare_fn* compare_fn, void* compare_user_data, rasqal_compare_free_user_data_fn* free_compare_user_data, rasqal_kv_free_fn* free_fn, raptor_data_print_handler print_key_fn, raptor_data_print_handler print_value_fn, int flags);
+#else
 rasqal_map* rasqal_new_map(rasqal_compare_fn* compare_fn, void* compare_user_data, rasqal_compare_free_user_data_fn* free_compare_user_data, rasqal_kv_free_fn* free_fn, raptor_sequence_print_handler* print_key_fn, raptor_sequence_print_handler* print_value_fn, int flags);
+#endif
 void rasqal_free_map(rasqal_map *map);
 int rasqal_map_add_kv(rasqal_map* map, void* key, void *value);
 void rasqal_map_visit(rasqal_map* map, rasqal_map_visit_fn fn, void *user_data);
-void rasqal_map_print(rasqal_map* map, FILE* fh);
+int rasqal_map_print(rasqal_map* map, FILE* fh);
 
 /* rasqal_query.c */
 rasqal_query_results* rasqal_query_execute_with_engine(rasqal_query* query, const rasqal_query_execution_factory* engine);
-void rasqal_query_remove_query_result(rasqal_query* query, rasqal_query_results* query_results);
+int rasqal_query_remove_query_result(rasqal_query* query, rasqal_query_results* query_results);
 int rasqal_query_declare_prefix(rasqal_query* rq, rasqal_prefix* prefix);
 int rasqal_query_declare_prefixes(rasqal_query* rq);
 unsigned char* rasqal_query_get_genid(rasqal_query* query, const unsigned char* base, int counter);
@@ -1015,13 +1006,13 @@ int rasqal_query_variable_bound_in_triple(rasqal_query *query, rasqal_variable *
 /* rasqal_query_results.c */
 int rasqal_init_query_results(void);
 void rasqal_finish_query_results(void);
-rasqal_query_results* rasqal_new_query_results(rasqal_world* world, rasqal_query* query, rasqal_query_results_type type, rasqal_variables_table* vars_table);
 rasqal_query_results* rasqal_query_results_execute_with_engine(rasqal_query* query, const rasqal_query_execution_factory* factory);
-void rasqal_query_results_add_row(rasqal_query_results* query_results, rasqal_row* row);
 int rasqal_query_results_check_limit_offset(rasqal_query_results* query_results);
 void rasqal_query_results_remove_query_reference(rasqal_query_results* query_results);
 rasqal_variables_table* rasqal_query_results_get_variables_table(rasqal_query_results* query_results);
 rasqal_row* rasqal_query_results_get_current_row(rasqal_query_results* query_results);
+rasqal_world* rasqal_query_results_get_world(rasqal_query_results* query_results);
+
 
 /* rasqal_query_write.c */
 int rasqal_query_write_sparql_20060406(raptor_iostream *iostr, rasqal_query* query, raptor_uri *base_uri);
@@ -1043,13 +1034,13 @@ int rasqal_init_result_format_sparql_xml(rasqal_world*);
 /* rasqal_format_table.c */
 int rasqal_init_result_format_table(rasqal_world*);
 
+/* rasqal_format_html.c */
+int rasqal_init_result_format_html(rasqal_world*);
+
 /* rasqal_row.c */
 rasqal_row* rasqal_new_row(rasqal_rowsource* rowsource);
-rasqal_row* rasqal_new_row_for_size(int size);
 rasqal_row* rasqal_new_row_from_row(rasqal_row* row);
-void rasqal_free_row(rasqal_row* row);
-void rasqal_row_print(rasqal_row* row, FILE* fh);
-void rasqal_row_set_value_at(rasqal_row* row, int offset, rasqal_literal* value);
+int rasqal_row_print(rasqal_row* row, FILE* fh);
 raptor_sequence* rasqal_new_row_sequence(rasqal_world* world, rasqal_variables_table* vt, const char* const row_data[], int vars_count, raptor_sequence** vars_seq_p);
 int rasqal_row_to_nodes(rasqal_row* row);
 void rasqal_row_set_values_from_variables_table(rasqal_row* row, rasqal_variables_table* vars_table);
@@ -1081,6 +1072,8 @@ int rasqal_xsd_datatype_check(rasqal_literal_type native_type, const unsigned ch
 const char* rasqal_xsd_datatype_label(rasqal_literal_type native_type);
 int rasqal_xsd_is_datatype_uri(rasqal_world*, raptor_uri* uri);
 const unsigned char* rasqal_xsd_datetime_string_to_canonical(const unsigned char* datetime_string);
+const unsigned char* rasqal_xsd_date_string_to_canonical(const unsigned char* date_string);
+
 int rasqal_xsd_datatype_is_numeric(rasqal_literal_type type);
 unsigned char* rasqal_xsd_format_integer(int i, size_t *len_p);
 unsigned char* rasqal_xsd_format_float(float f, size_t *len_p);
@@ -1100,8 +1093,14 @@ struct rasqal_world_s {
   /* should rasqal free the raptor_world */
   int raptor_world_allocated_here;
 
+#ifdef HAVE_RAPTOR2_API
+  /* log handler */
+  raptor_log_handler log_handler;
+  void *log_handler_user_data;
+#else
   /* error handlers structure */
   raptor_error_handlers error_handlers;
+#endif
 
   /* linked list of query languages */
   rasqal_query_language_factory *query_languages;
@@ -1125,6 +1124,13 @@ struct rasqal_world_s {
   /* graph factory */
   rasqal_graph_factory *graph_factory;
   void *graph_factory_user_data;
+
+  int default_generate_bnodeid_handler_base;
+  char *default_generate_bnodeid_handler_prefix;
+  size_t default_generate_bnodeid_handler_prefix_length;
+
+  void *generate_bnodeid_handler_user_data;
+  rasqal_generate_bnodeid_handler2 generate_bnodeid_handler;
 };
 
 
@@ -1238,16 +1244,13 @@ void rasqal_free_algebra_node(rasqal_algebra_node* node);
 rasqal_algebra_node_operator rasqal_algebra_node_get_operator(rasqal_algebra_node* node);
 const char* rasqal_algebra_node_operator_as_string(rasqal_algebra_node_operator op);
 int rasqal_algebra_algebra_node_write(rasqal_algebra_node *node, raptor_iostream* iostr);
-void rasqal_algebra_node_print(rasqal_algebra_node* node, FILE* fh);
+int rasqal_algebra_node_print(rasqal_algebra_node* node, FILE* fh);
 int rasqal_algebra_node_visit(rasqal_query *query, rasqal_algebra_node* node, rasqal_algebra_node_visit_fn fn, void *user_data);
 rasqal_algebra_node* rasqal_algebra_query_to_algebra(rasqal_query* query);
 int rasqal_algebra_node_is_empty(rasqal_algebra_node* node);
 
 /* rasqal_variable.c */
-rasqal_variables_table* rasqal_new_variables_table(rasqal_world* world);
 rasqal_variables_table* rasqal_new_variables_table_from_variables_table(rasqal_variables_table* vt);
-void rasqal_free_variables_table(rasqal_variables_table* vt);
-rasqal_variable* rasqal_variables_table_add(rasqal_variables_table* vt, rasqal_variable_type type, const unsigned char *name, rasqal_literal *value);
 rasqal_variable* rasqal_variables_table_get(rasqal_variables_table* vt, int idx);
 rasqal_variable* rasqal_variables_table_get_by_name(rasqal_variables_table* vt, const unsigned char *name);
 rasqal_literal* rasqal_variables_table_get_value(rasqal_variables_table* vt, int idx);
@@ -1356,6 +1359,44 @@ const char* rasqal_engine_get_parts_string(rasqal_triple_parts parts);
 
 /* New query engine based on executing over query algebra */
 extern const rasqal_query_execution_factory rasqal_query_engine_algebra;
+
+
+/* rasqal_update.c */
+const char* rasqal_update_type_label(rasqal_update_type type);
+rasqal_update_operation* rasqal_new_update_operation(rasqal_update_type type, raptor_uri* graph_uri, raptor_uri* document_uri, raptor_sequence* insert_templates, raptor_sequence* delete_templates, rasqal_graph_pattern* graph_pattern, int flags);
+void rasqal_free_update_operation(rasqal_update_operation *update);
+int rasqal_update_operation_print(rasqal_update_operation *update, FILE* stream);
+int rasqal_query_add_update_operation(rasqal_query* query, rasqal_update_operation *update);
+
+
+/* raptor1 -> raptor2 migration */
+/* use preprocessor macros to map raptor2 calls to raptor1 if raptor2 is not available */
+#ifndef HAVE_RAPTOR2_API
+
+#define raptor_new_iostream_to_file_handle(world, fh) raptor_new_iostream_to_file_handle(fh)
+#define raptor_iostream_write_bytes(ptr, size, nmemb, iostr) raptor_iostream_write_bytes(iostr, ptr, size, nmemb)
+#define raptor_iostream_write_byte(byte, iostr) raptor_iostream_write_byte(iostr, byte)
+#define raptor_iostream_counted_string_write(string, len, iostr) raptor_iostream_write_counted_string(iostr, string, len)
+#define raptor_iostream_string_write(string, iostr) raptor_iostream_write_string(iostr, string)
+#define raptor_iostream_decimal_write(decimal, iostr) raptor_iostream_write_decimal(iostr, decimal)
+#define raptor_iostream_read_bytes(ptr, size, nmemb, iostr) raptor_iostream_read_bytes(iostr, ptr, size, nmemb)
+#define raptor_xml_escape_string_write(string, len, quote, iostr) raptor_iostream_write_xml_escaped_string(iostr, string, len, quote, NULL, NULL)
+
+#define raptor_string_ntriples_write(str, len, delim, iostr) raptor_iostream_write_string_ntriples(iostr, str, len, delim)
+
+#define raptor_new_qname_from_namespace_uri(nstack, uri, xml_version) raptor_namespaces_qname_from_uri(nstack, uri, xml_version)
+
+#define raptor_unicode_utf8_string_get_char(input, length, output) raptor_utf8_to_unicode_char(output, input, length)
+#define raptor_unicode_utf8_string_put_char(char, output, length) raptor_unicode_char_to_utf8(char, output)
+
+#define raptor_new_uri(world, uri_string) raptor_new_uri(uri_string)
+#define raptor_new_uri_from_uri_local_name(world, base_uri, name) raptor_new_uri_from_uri_local_name(base_uri, name)
+#define raptor_uri_write(uri, iostr) raptor_iostream_write_uri(iostr, uri)
+
+#define RAPTOR_LOG_LEVEL_WARN RAPTOR_LOG_LEVEL_WARNING
+
+#endif /* !HAVE_RAPTOR2_API */
+
   
 /* end of RASQAL_INTERNAL */
 #endif
