@@ -411,6 +411,7 @@ rasqal_new_numeric_literal(rasqal_world* world, rasqal_literal_type type,
 
   switch(type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       return rasqal_new_integer_literal(world, type, (int)d);
       break;
 
@@ -501,21 +502,24 @@ rasqal_literal_set_typed_value(rasqal_literal* l, rasqal_literal_type type,
     l->string = (unsigned char*)RASQAL_MALLOC(cstring, l->string_len+1);
     if(!l->string)
       return 1;
-    strncpy((char*)l->string, (const char*)string, l->string_len+1);
+    memcpy((void*)l->string, string, l->string_len + 1);
   }
 
-  dt_uri = rasqal_xsd_datatype_type_to_uri(l->world, l->type);
-  if(!dt_uri)
-    return 1;
+  if(l->type != RASQAL_LITERAL_INTEGER_SUBTYPE) {
+    dt_uri = rasqal_xsd_datatype_type_to_uri(l->world, l->type);
+    if(!dt_uri)
+      return 1;
 
-  if(l->datatype)
-    raptor_free_uri(l->datatype);
-  l->datatype = raptor_uri_copy(dt_uri);
+    if(l->datatype)
+      raptor_free_uri(l->datatype);
+    l->datatype = raptor_uri_copy(dt_uri);
 
-  l->parent_type = rasqal_xsd_datatype_parent_type(type);
+    l->parent_type = rasqal_xsd_datatype_parent_type(type);
+  }
 
   switch(type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       eptr = NULL;
       i = (int)strtol((const char*)l->string, &eptr, 10);
       if(*eptr)
@@ -754,8 +758,7 @@ rasqal_new_string_literal_common(rasqal_world* world,
  * prefixes have not yet been declared or checked.
  * 
  * If the string literal is datatyped and of certain types recognised
- * it may be converted to a different literal type by
- * rasqal_literal_string_to_native()
+ * it may be converted to a different literal type.
  *
  * Return value: New #rasqal_literal or NULL on failure
  **/
@@ -926,7 +929,8 @@ rasqal_new_literal_from_literal(rasqal_literal* l)
 void
 rasqal_free_literal(rasqal_literal* l)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(l, rasqal_literal);
+  if(!l)
+    return;
   
   if(--l->usage)
     return;
@@ -946,6 +950,7 @@ rasqal_free_literal(rasqal_literal* l)
     case RASQAL_LITERAL_FLOAT:
     case RASQAL_LITERAL_DATETIME:
     case RASQAL_LITERAL_UDT:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       if(l->string)
         RASQAL_FREE(cstring, (void*)l->string);
       if(l->language)
@@ -1009,7 +1014,8 @@ static const char* const rasqal_literal_type_labels[RASQAL_LITERAL_LAST+1]={
   "udt",
   "pattern",
   "qname",
-  "variable"
+  "variable",
+  "<integer subtype>"
 };
 
 
@@ -1027,14 +1033,14 @@ rasqal_literal_write_type(rasqal_literal* l, raptor_iostream* iostr)
   rasqal_literal_type type;
 
   if(!l) {
-    raptor_iostream_write_counted_string(iostr, "null", 4);
+    raptor_iostream_counted_string_write("null", 4, iostr);
     return;
   }
   
   type = l->type;
   if(type > RASQAL_LITERAL_LAST)
     type = RASQAL_LITERAL_UNKNOWN;
-  raptor_iostream_write_string(iostr, rasqal_literal_type_labels[(int)type]);
+  raptor_iostream_string_write(rasqal_literal_type_labels[(int)type], iostr);
 }
 
 
@@ -1049,17 +1055,20 @@ rasqal_literal_write_type(rasqal_literal* l, raptor_iostream* iostr)
 void
 rasqal_literal_print_type(rasqal_literal* l, FILE* fh)
 {
-  rasqal_literal_type type;
+  raptor_iostream *iostr;
 
   if(!l) {
     fputs("null", fh);
     return;
   }
-  
-  type = l->type;
-  if(type > RASQAL_LITERAL_LAST)
-    type = RASQAL_LITERAL_UNKNOWN;
-  fputs(rasqal_literal_type_labels[(int)type], fh);
+
+  iostr = raptor_new_iostream_to_file_handle(l->world->raptor_world_ptr, fh);
+  if(!iostr)
+    return;
+
+  rasqal_literal_write_type(l, iostr);
+
+  raptor_free_iostream(iostr);
 }
 
 
@@ -1079,50 +1088,50 @@ rasqal_literal_write(rasqal_literal* l, raptor_iostream* iostr)
   size_t len;
   
   if(!l) {
-    raptor_iostream_write_counted_string(iostr, "null", 4);
+    raptor_iostream_counted_string_write("null", 4, iostr);
     return;
   }
 
   if(!l->valid)
-    raptor_iostream_write_counted_string(iostr, "INV:", 4);
+    raptor_iostream_counted_string_write("INV:", 4, iostr);
 
   if(l->type != RASQAL_LITERAL_VARIABLE)
     rasqal_literal_write_type(l, iostr);
 
   switch(l->type) {
     case RASQAL_LITERAL_URI:
-      raptor_iostream_write_byte(iostr, '<');
+      raptor_iostream_write_byte('<', iostr);
       str = raptor_uri_as_counted_string(l->value.uri, &len);
-      raptor_iostream_write_string_ntriples(iostr, str, len, '>');
-      raptor_iostream_write_byte(iostr, '>');
+      raptor_string_ntriples_write(str, len, '>', iostr);
+      raptor_iostream_write_byte('>', iostr);
       break;
     case RASQAL_LITERAL_BLANK:
-      raptor_iostream_write_byte(iostr, ' ');
-      raptor_iostream_write_counted_string(iostr, l->string, l->string_len);
+      raptor_iostream_write_byte(' ', iostr);
+      raptor_iostream_counted_string_write(l->string, l->string_len, iostr);
       break;
     case RASQAL_LITERAL_PATTERN:
-      raptor_iostream_write_byte(iostr, '/');
-      raptor_iostream_write_counted_string(iostr, l->string, l->string_len);
-      raptor_iostream_write_byte(iostr, '/');
+      raptor_iostream_write_byte('/', iostr);
+      raptor_iostream_counted_string_write(l->string, l->string_len, iostr);
+      raptor_iostream_write_byte('/', iostr);
       if(l->flags)
-        raptor_iostream_write_string(iostr, l->flags);
+        raptor_iostream_string_write(l->flags, iostr);
       break;
     case RASQAL_LITERAL_STRING:
     case RASQAL_LITERAL_UDT:
-      raptor_iostream_write_counted_string(iostr, "(\"", 2);
-      raptor_iostream_write_string_ntriples(iostr, l->string, l->string_len, '"');
-      raptor_iostream_write_byte(iostr, '"');
+      raptor_iostream_counted_string_write("(\"", 2, iostr);
+      raptor_string_ntriples_write(l->string, l->string_len, '"', iostr);
+      raptor_iostream_write_byte('"', iostr);
       if(l->language) {
-        raptor_iostream_write_byte(iostr, '@');
-        raptor_iostream_write_string(iostr, l->language);
+        raptor_iostream_write_byte('@', iostr);
+        raptor_iostream_string_write(l->language, iostr);
       }
       if(l->datatype) {
-        raptor_iostream_write_counted_string(iostr, "^^<", 3);
+        raptor_iostream_counted_string_write("^^<", 3, iostr);
         str = raptor_uri_as_counted_string(l->datatype, &len);
-        raptor_iostream_write_string_ntriples(iostr, str, len, '>');
-        raptor_iostream_write_byte(iostr, '>');
+        raptor_string_ntriples_write(str, len, '>', iostr);
+        raptor_iostream_write_byte('>', iostr);
       }
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_write_byte(')', iostr);
       break;
     case RASQAL_LITERAL_VARIABLE:
       rasqal_variable_write(l->value.variable, iostr);
@@ -1136,9 +1145,10 @@ rasqal_literal_write(rasqal_literal* l, raptor_iostream* iostr)
     case RASQAL_LITERAL_FLOAT:
     case RASQAL_LITERAL_DECIMAL:
     case RASQAL_LITERAL_DATETIME:
-      raptor_iostream_write_byte(iostr, '(');
-      raptor_iostream_write_counted_string(iostr, l->string, l->string_len);
-      raptor_iostream_write_byte(iostr, ')');
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
+      raptor_iostream_write_byte('(', iostr);
+      raptor_iostream_counted_string_write(l->string, l->string_len, iostr);
+      raptor_iostream_write_byte(')', iostr);
       break;
 
     case RASQAL_LITERAL_UNKNOWN:
@@ -1163,77 +1173,16 @@ rasqal_literal_write(rasqal_literal* l, raptor_iostream* iostr)
 int
 rasqal_literal_print(rasqal_literal* l, FILE* fh)
 {
+  raptor_iostream *iostr;
+
   if(!l) {
     fputs("null", fh);
     return 0;
   }
 
-  if(!l->valid)
-    fputs("INV:", fh);
-
-  if(l->type != RASQAL_LITERAL_VARIABLE)
-    rasqal_literal_print_type(l, fh);
-
-  switch(l->type) {
-    case RASQAL_LITERAL_URI:
-      fputc('<', fh);
-#ifdef RAPTOR_V2_AVAILABLE
-      raptor_print_ntriples_string(raptor_uri_as_string(l->value.uri), '>', fh);
-#else
-      raptor_print_ntriples_string(fh, raptor_uri_as_string(l->value.uri), '>');
-#endif
-      fputc('>', fh);
-      break;
-    case RASQAL_LITERAL_BLANK:
-      fprintf(fh, " %s", l->string);
-      break;
-    case RASQAL_LITERAL_PATTERN:
-      fprintf(fh, "/%s/%s", l->string, l->flags ? (const char*)l->flags : "");
-      break;
-    case RASQAL_LITERAL_STRING:
-    case RASQAL_LITERAL_UDT:
-      fputs("(\"", fh);
-#ifdef RAPTOR_V2_AVAILABLE
-      raptor_print_ntriples_string(l->string, '"', fh);
-#else
-      raptor_print_ntriples_string(fh, l->string, '"');
-#endif
-      fputc('"', fh);
-      if(l->language)
-        fprintf(fh, "@%s", l->language);
-      if(l->datatype) {
-        fputs("^^<", fh);
-#ifdef RAPTOR_V2_AVAILABLE
-        raptor_print_ntriples_string(raptor_uri_as_string(l->datatype), '>', fh);
-#else
-        raptor_print_ntriples_string(fh, raptor_uri_as_string(l->datatype), '>');
-#endif
-        fputc('>', fh);
-      }
-      fputc(')', fh);
-      break;
-    case RASQAL_LITERAL_VARIABLE:
-      rasqal_variable_print(l->value.variable, fh);
-      break;
-
-    case RASQAL_LITERAL_QNAME:
-    case RASQAL_LITERAL_INTEGER:
-    case RASQAL_LITERAL_XSD_STRING:
-    case RASQAL_LITERAL_BOOLEAN:
-    case RASQAL_LITERAL_DOUBLE:
-    case RASQAL_LITERAL_FLOAT:
-    case RASQAL_LITERAL_DECIMAL:
-    case RASQAL_LITERAL_DATETIME:
-      fputc('(', fh);
-      fwrite(l->string, 1, l->string_len, fh);
-      fputc(')', fh);
-      break;
-
-    case RASQAL_LITERAL_UNKNOWN:
-    default:
-      RASQAL_FATAL2("Unknown literal type %d", l->type);
-      return 1;
-  }
+  iostr = raptor_new_iostream_to_file_handle(l->world->raptor_world_ptr, fh);
+  rasqal_literal_write(l, iostr);
+  raptor_free_iostream(iostr);
 
   return 0;
 }
@@ -1297,6 +1246,7 @@ rasqal_literal_as_boolean(rasqal_literal* l, int *error)
 
     case RASQAL_LITERAL_INTEGER:
     case RASQAL_LITERAL_BOOLEAN:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       return l->value.integer != 0;
       break;
 
@@ -1337,6 +1287,7 @@ rasqal_literal_as_integer(rasqal_literal* l, int *error)
   
   switch(l->type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       return l->value.integer;
       break;
 
@@ -1418,6 +1369,7 @@ rasqal_literal_as_floating(rasqal_literal* l, int *error)
   switch(l->type) {
     case RASQAL_LITERAL_INTEGER:
     case RASQAL_LITERAL_BOOLEAN:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       return (double)l->value.integer;
       break;
 
@@ -1523,6 +1475,7 @@ rasqal_literal_as_string_flags(rasqal_literal* l, int flags, int *error)
     case RASQAL_LITERAL_DECIMAL:
     case RASQAL_LITERAL_DATETIME:
     case RASQAL_LITERAL_UDT:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       return l->string;
 
     case RASQAL_LITERAL_URI:
@@ -1728,7 +1681,7 @@ rasqal_new_literal_from_promotion(rasqal_literal* lit,
       new_s = (unsigned char*)RASQAL_MALLOC(sstring, len+1);
       if(new_s) {
         raptor_uri* dt_uri = NULL;
-        strncpy((char*)new_s, (const char*)s, len+1);
+        memcpy(new_s, s, len + 1);
         if(lit->datatype) {
           dt_uri = raptor_uri_copy(lit->datatype);
         }
@@ -1764,6 +1717,7 @@ rasqal_new_literal_from_promotion(rasqal_literal* lit,
       
 
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       i = rasqal_literal_as_integer(lit, &errori);
       /* failure always means no match */
       if(errori)
@@ -1789,7 +1743,7 @@ rasqal_new_literal_from_promotion(rasqal_literal* lit,
       len = strlen((const char*)s);
       new_s = (unsigned char*)RASQAL_MALLOC(sstring, len+1);
       if(new_s) {
-        strncpy((char*)new_s, (const char*)s, len+1);
+        memcpy(new_s, s, len + 1);
         new_lit = rasqal_new_string_literal(lit->world, new_s, NULL, NULL, NULL);
       }
       break;
@@ -1800,7 +1754,7 @@ rasqal_new_literal_from_promotion(rasqal_literal* lit,
       new_s = (unsigned char*)RASQAL_MALLOC(sstring, len+1);
       if(new_s) {
         raptor_uri* dt_uri;
-        strncpy((char*)new_s, (const char*)s, len+1);
+        memcpy(new_s, s, len + 1);
         dt_uri = rasqal_xsd_datatype_type_to_uri(lit->world, lit->type);
         dt_uri = raptor_uri_copy(dt_uri);
         new_lit = rasqal_new_string_literal(lit->world, new_s, NULL, dt_uri,
@@ -1924,6 +1878,7 @@ rasqal_literal_rdql_promote_calculate(rasqal_literal* l1, rasqal_literal* l2)
       break;
       
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       seen_int++;
       break;
       
@@ -2136,6 +2091,7 @@ rasqal_literal_compare(rasqal_literal* l1, rasqal_literal* l2, int flags,
 
     case RASQAL_LITERAL_INTEGER:
     case RASQAL_LITERAL_BOOLEAN:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       result = new_lits[0]->value.integer - new_lits[1]->value.integer;
       break;
 
@@ -2328,14 +2284,14 @@ rasqal_literal_equals(rasqal_literal* l1, rasqal_literal* l2)
 }
 
 
-/**
+/*
  * rasqal_literal_equals_flags:
  * @l1: #rasqal_literal literal
  * @l2: #rasqal_literal data literal
  * @flags: comparison flags
  * @error_p: type error
  *
- * Compare two literals with optional type promotion.
+ * INTERNAL - Compare two literals with optional type promotion.
  * 
  * flag bits affects equality:
  *   RASQAL_COMPARE_XQUERY: use XQuery comparison and type promotion rules
@@ -2452,6 +2408,7 @@ rasqal_literal_equals_flags(rasqal_literal* l1, rasqal_literal* l2,
       
     case RASQAL_LITERAL_INTEGER:
     case RASQAL_LITERAL_BOOLEAN:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       result = l1_p->value.integer == l2_p->value.integer;
       break;
 
@@ -2520,7 +2477,7 @@ rasqal_literal_expand_qname(void *user_data, rasqal_literal *l)
     raptor_uri *uri;
 
     /* expand a literal qname */
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
     uri = raptor_qname_string_to_uri(rq->namespaces,
                                      l->string, l->string_len);
 #else
@@ -2539,7 +2496,7 @@ rasqal_literal_expand_qname(void *user_data, rasqal_literal *l)
     
     if(l->flags) {
       /* expand a literal string datatype qname */
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
       uri = raptor_qname_string_to_uri(rq->namespaces,
                                        l->flags,
                                        strlen((const char*)l->flags));
@@ -2628,6 +2585,7 @@ rasqal_literal_as_node(rasqal_literal* l)
     case RASQAL_LITERAL_DECIMAL:
     case RASQAL_LITERAL_DATETIME:
     case RASQAL_LITERAL_UDT:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       new_l = (rasqal_literal*)RASQAL_CALLOC(rasqal_literal, 1, sizeof(*new_l));
       if(new_l) {
         new_l->valid = 1;
@@ -2640,7 +2598,7 @@ rasqal_literal_as_node(rasqal_literal* l)
           rasqal_free_literal(new_l);
           return NULL; 
         }
-        strncpy((char*)new_l->string, (const char*)l->string, l->string_len+1);
+        memcpy((void*)new_l->string, l->string, l->string_len + 1);
         dt_uri = rasqal_xsd_datatype_type_to_uri(l->world, l->type);
         if(!dt_uri) {
           rasqal_free_literal(new_l);
@@ -2700,7 +2658,7 @@ rasqal_literal_ebv(rasqal_literal* l)
             !l->datatype && !l->string_len) {
     /* ... The operand is a 0-length untyped RDF literal or xs:string. */
     b = 0;
-  } else if((l->type == RASQAL_LITERAL_INTEGER && !l->value.integer) ||
+  } else if(((l->type == RASQAL_LITERAL_INTEGER || l->type == RASQAL_LITERAL_INTEGER_SUBTYPE) && !l->value.integer) ||
             ((l->type == RASQAL_LITERAL_DOUBLE || 
               l->type == RASQAL_LITERAL_FLOAT) &&
              !l->value.floating)
@@ -2751,6 +2709,7 @@ rasqal_literal_is_constant(rasqal_literal* l)
     case RASQAL_LITERAL_DECIMAL:
     case RASQAL_LITERAL_DATETIME:
     case RASQAL_LITERAL_UDT:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       return 1;
 
     case RASQAL_LITERAL_VARIABLE:
@@ -2825,6 +2784,7 @@ rasqal_literal_cast(rasqal_literal* l, raptor_uri* to_datatype, int flags,
       case RASQAL_LITERAL_DOUBLE:
       case RASQAL_LITERAL_FLOAT:
       case RASQAL_LITERAL_DECIMAL:
+      case RASQAL_LITERAL_INTEGER_SUBTYPE:
         /* XSD (boolean, integer, decimal, double, float) may NOT be
          * cast to dateTime */
         if(to_native_type == RASQAL_LITERAL_DATETIME) {
@@ -2966,6 +2926,7 @@ rasqal_literal_add(rasqal_literal* l1, rasqal_literal* l2, int *error_p)
   type = rasqal_literal_promote_numerics(l1, l2, flags);
   switch(type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       i = rasqal_literal_as_integer(l1, &error);
       if(error)
         break;
@@ -3052,6 +3013,7 @@ rasqal_literal_subtract(rasqal_literal* l1, rasqal_literal* l2, int *error_p)
   type = rasqal_literal_promote_numerics(l1, l2, flags);
   switch(type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       i = rasqal_literal_as_integer(l1, &error);
       if(error)
         break;
@@ -3138,6 +3100,7 @@ rasqal_literal_multiply(rasqal_literal* l1, rasqal_literal* l2, int *error_p)
   type = rasqal_literal_promote_numerics(l1, l2, flags);
   switch(type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       i = rasqal_literal_as_integer(l1, &error);
       if(error)
         break;
@@ -3224,6 +3187,7 @@ rasqal_literal_divide(rasqal_literal* l1, rasqal_literal* l2, int *error_p)
   type = rasqal_literal_promote_numerics(l1, l2, flags);
   switch(type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       i2 = rasqal_literal_as_integer(l2, &error);
       if(!i2)
         /* division by zero error */
@@ -3316,6 +3280,7 @@ rasqal_literal_negate(rasqal_literal* l, int *error_p)
 
   switch(l->type) {
     case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       i = rasqal_literal_as_integer(l, &error);
       if(error)
         break;
