@@ -41,7 +41,7 @@
 #include "rasqal_internal.h"
 
 
-/**
+/*
  *
  * Query Class Internals
  *
@@ -98,11 +98,13 @@ rasqal_new_query(rasqal_world *world, const char *name,
 {
   rasqal_query_language_factory* factory;
   rasqal_query* query;
-#ifndef RAPTOR_V2_AVAILABLE
+#ifndef HAVE_RAPTOR2_API
   const raptor_uri_handler *uri_handler;
   void *uri_context;
 #endif
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
+
   /* for compatibility with older binaries that do not call it */
   rasqal_world_open(world);
 
@@ -131,11 +133,8 @@ rasqal_new_query(rasqal_world *world, const char *name,
   if(!query->context)
     goto tidy;
   
-#ifdef RAPTOR_V2_AVAILABLE
-  query->namespaces = raptor_new_namespaces_v2(world->raptor_world_ptr,
-                                               (raptor_simple_message_handler)rasqal_query_simple_error,
-                                               query,
-                                               0);
+#ifdef HAVE_RAPTOR2_API
+  query->namespaces = raptor_new_namespaces(world->raptor_world_ptr, 0);
 #else
   raptor_uri_get_handler(&uri_handler, &uri_context);
   query->namespaces = raptor_new_namespaces(uri_handler, uri_context,
@@ -150,19 +149,35 @@ rasqal_new_query(rasqal_world *world, const char *name,
   if(!query->vars_table)
     goto tidy;
 
+#ifdef HAVE_RAPTOR2_API
+  query->triples = raptor_new_sequence((raptor_data_free_handler)rasqal_free_triple, (raptor_data_print_handler)rasqal_triple_print);
+#else
   query->triples = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_triple, (raptor_sequence_print_handler*)rasqal_triple_print);
+#endif
   if(!query->triples)
     goto tidy;
-  
+
+#ifdef HAVE_RAPTOR2_API
+  query->prefixes = raptor_new_sequence((raptor_data_free_handler)rasqal_free_prefix, (raptor_data_print_handler)rasqal_prefix_print);
+#else
   query->prefixes = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
+#endif
   if(!query->prefixes)
     goto tidy;
 
+#ifdef HAVE_RAPTOR2_API
+  query->data_graphs = raptor_new_sequence((raptor_data_free_handler)rasqal_free_data_graph, (raptor_data_print_handler)rasqal_data_graph_print);
+#else
   query->data_graphs = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_data_graph, (raptor_sequence_print_handler*)rasqal_data_graph_print);
+#endif
   if(!query->data_graphs)
     goto tidy;
 
+#ifdef HAVE_RAPTOR2_API
+  query->results = raptor_new_sequence((raptor_data_free_handler)rasqal_query_results_remove_query_reference, NULL);
+#else
   query->results = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_query_results_remove_query_reference, NULL);
+#endif
   if(!query->results)
     goto tidy;
 
@@ -187,7 +202,8 @@ rasqal_new_query(rasqal_world *world, const char *name,
 void
 rasqal_free_query(rasqal_query* query) 
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+  if(!query)
+    return;
   
   if(--query->usage)
     return;
@@ -202,11 +218,7 @@ rasqal_free_query(rasqal_query* query)
     raptor_free_namespaces(query->namespaces);
 
   if(query->base_uri)
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_free_uri_v2(query->world->raptor_world_ptr, query->base_uri);
-#else
     raptor_free_uri(query->base_uri);
-#endif
 
   if(query->query_string)
     RASQAL_FREE(cstring, query->query_string);
@@ -244,6 +256,9 @@ rasqal_free_query(rasqal_query* query)
   if(query->group_conditions_sequence)
     raptor_free_sequence(query->group_conditions_sequence);
 
+  if(query->having_conditions_sequence)
+    raptor_free_sequence(query->having_conditions_sequence);
+
   if(query->graph_patterns_sequence)
     raptor_free_sequence(query->graph_patterns_sequence);
 
@@ -253,6 +268,10 @@ rasqal_free_query(rasqal_query* query)
   /* Do this last since most everything above could refer to a variable */
   if(query->vars_table)
     rasqal_free_variables_table(query->vars_table);
+
+  if(query->updates)
+    raptor_free_sequence(query->updates);
+  
   RASQAL_FREE(rasqal_query, query);
 }
 
@@ -270,6 +289,8 @@ rasqal_free_query(rasqal_query* query)
 const char*
 rasqal_query_get_name(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->factory->name;
 }
 
@@ -285,6 +306,8 @@ rasqal_query_get_name(rasqal_query* query)
 const char*
 rasqal_query_get_label(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->factory->label;
 }
 
@@ -304,14 +327,20 @@ void
 rasqal_query_set_fatal_error_handler(rasqal_query* query, void *user_data,
                                      raptor_message_handler handler)
 {
+#ifdef HAVE_RAPTOR2_API
+#else
   raptor_error_handlers* error_handlers;
-  if(!query || !query->world)
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  if(!query->world)
     return;
 
   error_handlers = &query->world->error_handlers;
   
   error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].handler = handler;
+  error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].handler   = handler;
+#endif
 }
 
 
@@ -330,14 +359,20 @@ void
 rasqal_query_set_error_handler(rasqal_query* query, void *user_data,
                                raptor_message_handler handler)
 {
+#ifdef HAVE_RAPTOR2_API
+#else
   raptor_error_handlers* error_handlers;
-  if(!query || !query->world)
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  if(!query->world)
     return;
 
   error_handlers = &query->world->error_handlers;
   
   error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].handler = handler;
+  error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].handler   = handler;
+#endif
 }
 
 
@@ -356,14 +391,20 @@ void
 rasqal_query_set_warning_handler(rasqal_query* query, void *user_data,
                                  raptor_message_handler handler)
 {
+#ifdef HAVE_RAPTOR2_API
+#else
   raptor_error_handlers* error_handlers;
-  if(!query || !query->world)
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  if(!query->world)
     return;
 
   error_handlers = &query->world->error_handlers;
   
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].handler  =handler;
+  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARN].user_data = user_data;
+  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARN].handler   = handler;
+#endif
 }
 
 
@@ -382,6 +423,8 @@ rasqal_query_set_warning_handler(rasqal_query* query, void *user_data,
 int
 rasqal_query_set_feature(rasqal_query* query, rasqal_feature feature, int value)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
   switch(feature) {
     case RASQAL_FEATURE_NO_NET:
       query->features[(int)feature] = value;
@@ -414,6 +457,9 @@ rasqal_query_set_feature_string(rasqal_query *query,
                                 const unsigned char *value)
 {
   int value_is_string = (rasqal_feature_value_type(feature) == 1);
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
   if(!value_is_string)
     return rasqal_query_set_feature(query, feature, atoi((const char*)value));
 
@@ -439,6 +485,8 @@ rasqal_query_get_feature(rasqal_query *query, rasqal_feature feature)
 {
   int result= -1;
   
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
   switch(feature) {
     case RASQAL_FEATURE_NO_NET:
       result = (query->features[(int)feature] != 0);
@@ -469,6 +517,9 @@ rasqal_query_get_feature_string(rasqal_query *query,
                                 rasqal_feature feature)
 {
   int value_is_string = (rasqal_feature_value_type(feature) == 1);
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!value_is_string)
     return NULL;
   
@@ -489,6 +540,8 @@ rasqal_query_get_feature_string(rasqal_query *query,
 int
 rasqal_query_get_distinct(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+
   return query->distinct;
 }
 
@@ -509,6 +562,8 @@ rasqal_query_get_distinct(rasqal_query* query)
 void
 rasqal_query_set_distinct(rasqal_query* query, int distinct_mode)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
   if(distinct_mode >= 0 && distinct_mode <= 2)
     query->distinct= distinct_mode;
   else
@@ -527,6 +582,8 @@ rasqal_query_set_distinct(rasqal_query* query, int distinct_mode)
 int
 rasqal_query_get_explain(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+
   return query->explain;
 }
 
@@ -542,7 +599,9 @@ rasqal_query_get_explain(rasqal_query* query)
 void
 rasqal_query_set_explain(rasqal_query* query, int is_explain)
 {
-  query->explain= (is_explain != 0) ? 1 : 0;
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  query->explain = (is_explain != 0) ? 1 : 0;
 }
 
 
@@ -559,6 +618,8 @@ rasqal_query_set_explain(rasqal_query* query, int is_explain)
 int
 rasqal_query_get_limit(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+
   return query->limit;
 }
 
@@ -575,6 +636,8 @@ rasqal_query_get_limit(rasqal_query* query)
 void
 rasqal_query_set_limit(rasqal_query* query, int limit)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
   query->limit = limit;
 }
 
@@ -592,6 +655,8 @@ rasqal_query_set_limit(rasqal_query* query, int limit)
 int
 rasqal_query_get_offset(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+
   return query->offset;
 }
 
@@ -608,6 +673,8 @@ rasqal_query_get_offset(rasqal_query* query)
 void
 rasqal_query_set_offset(rasqal_query* query, int offset)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
   query->offset = offset;
 }
 
@@ -634,6 +701,9 @@ rasqal_query_add_data_graph(rasqal_query* query,
 {
   rasqal_data_graph *dg;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, raptor_uri, 1);
+
   if((flags & RASQAL_DATA_GRAPH_NAMED) && !name_uri)
     return 1;
   
@@ -657,6 +727,8 @@ rasqal_query_add_data_graph(rasqal_query* query,
 raptor_sequence*
 rasqal_query_get_data_graph_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->data_graphs;
 }
 
@@ -673,6 +745,8 @@ rasqal_query_get_data_graph_sequence(rasqal_query* query)
 rasqal_data_graph*
 rasqal_query_get_data_graph(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!query->data_graphs)
     return NULL;
   
@@ -697,15 +771,11 @@ rasqal_query_dataset_contains_named_graph(rasqal_query* query,
   int idx;
   int found = 0;
   
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(graph_uri, raptor_uri, 1);
+
   for(idx = 0; (dg = rasqal_query_get_data_graph(query, idx)); idx++) {
-    if(dg->name_uri &&
-#ifdef RAPTOR_V2_AVAILABLE
-       raptor_uri_equals_v2(query->world->raptor_world_ptr, dg->name_uri, graph_uri)
-#else
-       raptor_uri_equals(dg->name_uri, graph_uri)
-#endif
-       )
-    {
+    if(dg->name_uri && raptor_uri_equals(dg->name_uri, graph_uri)) {
       /* graph_uri is a graph name in the dataset */
       found = 1;
       break;
@@ -730,8 +800,15 @@ rasqal_query_dataset_contains_named_graph(rasqal_query* query,
 int
 rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(var, rasqal_variable, 1);
+
   if(!query->selects) {
+#ifdef HAVE_RAPTOR2_API
+    query->selects = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
+#else
     query->selects = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
     if(!query->selects)
       return 1;
   }
@@ -754,6 +831,8 @@ rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var)
 raptor_sequence*
 rasqal_query_get_bound_variable_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->selects;
 }
 
@@ -772,6 +851,8 @@ rasqal_query_get_bound_variable_sequence(rasqal_query* query)
 raptor_sequence*
 rasqal_query_get_describe_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->describes;
 }
 
@@ -787,6 +868,8 @@ rasqal_query_get_describe_sequence(rasqal_query* query)
 raptor_sequence*
 rasqal_query_get_anonymous_variable_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return rasqal_variables_table_get_anonymous_variables_sequence(query->vars_table);
 }
 
@@ -802,6 +885,8 @@ rasqal_query_get_anonymous_variable_sequence(rasqal_query* query)
 raptor_sequence*
 rasqal_query_get_all_variable_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return rasqal_variables_table_get_named_variables_sequence(query->vars_table);
 }
 
@@ -818,6 +903,8 @@ rasqal_query_get_all_variable_sequence(rasqal_query* query)
 rasqal_variable*
 rasqal_query_get_variable(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!query->selects || idx < 0 || idx >= query->select_variables_count)
     return NULL;
   
@@ -837,6 +924,9 @@ rasqal_query_get_variable(rasqal_query* query, int idx)
 int
 rasqal_query_has_variable(rasqal_query* query, const unsigned char *name)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, char*, 0);
+
   return rasqal_variables_table_has(query->vars_table, name);
 }
 
@@ -860,6 +950,10 @@ rasqal_query_set_variable(rasqal_query* query, const unsigned char *name,
 {
   int i;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, char*, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(value, rasqal_literal, 1);
+
   if(!query->selects)
     return 1;
   
@@ -888,6 +982,8 @@ rasqal_query_set_variable(rasqal_query* query, const unsigned char *name,
 raptor_sequence*
 rasqal_query_get_triple_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->triples;
 }
 
@@ -904,6 +1000,8 @@ rasqal_query_get_triple_sequence(rasqal_query* query)
 rasqal_triple*
 rasqal_query_get_triple(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!query->triples)
     return NULL;
   
@@ -914,16 +1012,15 @@ rasqal_query_get_triple(rasqal_query* query, int idx)
 int
 rasqal_query_declare_prefix(rasqal_query *rq, rasqal_prefix *p)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(rq, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(p, rasqal_prefix, 1);
+
   if(p->declared)
     return 0;
 
   if(raptor_namespaces_start_namespace_full(rq->namespaces, 
                                             p->prefix, 
-#ifdef RAPTOR_V2_AVAILABLE
-                                            raptor_uri_as_string_v2(rq->world->raptor_world_ptr, p->uri),
-#else
                                             raptor_uri_as_string(p->uri),
-#endif
                                             rq->prefix_depth))
     return 1;
   p->declared = 1;
@@ -935,6 +1032,9 @@ rasqal_query_declare_prefix(rasqal_query *rq, rasqal_prefix *p)
 static int
 rasqal_query_undeclare_prefix(rasqal_query *rq, rasqal_prefix *prefix)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(rq, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(prefix, rasqal_prefix, 1);
+
   if(!prefix->declared) {
     prefix->declared = 1;
     return 0;
@@ -950,6 +1050,8 @@ rasqal_query_declare_prefixes(rasqal_query *rq)
 {
   int i;
   
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(rq, rasqal_query, 1);
+
   if(!rq->prefixes)
     return 0;
   
@@ -977,8 +1079,15 @@ rasqal_query_declare_prefixes(rasqal_query *rq)
 int
 rasqal_query_add_prefix(rasqal_query* query, rasqal_prefix* prefix)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(prefix, rasqal_prefix, 1);
+
   if(!query->prefixes) {
+#ifdef HAVE_RAPTOR2_API
+    query->prefixes = raptor_new_sequence((raptor_data_free_handler)rasqal_free_prefix, (raptor_data_print_handler)rasqal_prefix_print);
+#else
     query->prefixes = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
+#endif
     if(!query->prefixes)
       return 1;
   } else {
@@ -1012,6 +1121,8 @@ rasqal_query_add_prefix(rasqal_query* query, rasqal_prefix* prefix)
 raptor_sequence*
 rasqal_query_get_prefix_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->prefixes;
 }
 
@@ -1028,6 +1139,8 @@ rasqal_query_get_prefix_sequence(rasqal_query* query)
 rasqal_prefix*
 rasqal_query_get_prefix(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!query->prefixes)
     return NULL;
 
@@ -1046,6 +1159,8 @@ rasqal_query_get_prefix(rasqal_query* query, int idx)
 rasqal_graph_pattern*
 rasqal_query_get_query_graph_pattern(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->query_graph_pattern;
 }
 
@@ -1061,6 +1176,8 @@ rasqal_query_get_query_graph_pattern(rasqal_query* query)
 raptor_sequence*
 rasqal_query_get_graph_pattern_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return rasqal_graph_pattern_get_sub_graph_pattern_sequence(query->query_graph_pattern);
 }
 
@@ -1077,6 +1194,8 @@ rasqal_query_get_graph_pattern_sequence(rasqal_query* query)
 rasqal_graph_pattern*
 rasqal_query_get_graph_pattern(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return rasqal_graph_pattern_get_sub_graph_pattern(query->query_graph_pattern, idx);
 }
 
@@ -1092,6 +1211,8 @@ rasqal_query_get_graph_pattern(rasqal_query* query, int idx)
 raptor_sequence*
 rasqal_query_get_construct_triples_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->constructs;
 }
 
@@ -1108,6 +1229,8 @@ rasqal_query_get_construct_triples_sequence(rasqal_query* query)
 rasqal_triple*
 rasqal_query_get_construct_triple(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!query->constructs)
     return NULL;
 
@@ -1141,6 +1264,8 @@ rasqal_query_prepare(rasqal_query* query,
 {
   int rc = 0;
   
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
   if(query->failed)
     return 1;
 
@@ -1175,18 +1300,10 @@ rasqal_query_prepare(rasqal_query* query,
   }
 
   if(base_uri)
-#ifdef RAPTOR_V2_AVAILABLE
-    base_uri = raptor_uri_copy_v2(query->world->raptor_world_ptr, base_uri);
-#else
     base_uri = raptor_uri_copy(base_uri);
-#endif
   else {
     unsigned char *uri_string = raptor_uri_filename_to_uri_string("");
-#ifdef RAPTOR_V2_AVAILABLE
-    base_uri = raptor_new_uri_v2(query->world->raptor_world_ptr, uri_string);
-#else
-    base_uri = raptor_new_uri(uri_string);
-#endif
+    base_uri = raptor_new_uri(query->world->raptor_world_ptr, uri_string);
     if(uri_string)
       raptor_free_memory(uri_string);
   }
@@ -1221,6 +1338,7 @@ const rasqal_query_execution_factory*
 rasqal_query_get_engine_by_name(const char* name)
 {
   const rasqal_query_execution_factory* engine;
+
 #if RASQAL_QUERY_ENGINE_VERSION == 1
   engine = &rasqal_query_engine_1;
 #else
@@ -1249,7 +1367,7 @@ rasqal_query_get_engine_by_name(const char* name)
 /**
  * rasqal_query_execute_with_engine:
  * @query: the #rasqal_query object
- * @engine: execution engine factory
+ * @engine: execution engine factory (or NULL)
  *
  * INTERNAL - Excecute a query with a given factory and return results.
  *
@@ -1261,6 +1379,8 @@ rasqal_query_execute_with_engine(rasqal_query* query,
 {
   rasqal_query_results *query_results = NULL;
   
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(query->failed)
     return NULL;
 
@@ -1299,7 +1419,8 @@ static const char* const rasqal_query_verb_labels[RASQAL_QUERY_VERB_LAST+1] = {
   "DESCRIBE",
   "ASK",
   "DELETE",
-  "INSERT"
+  "INSERT",
+  "UPDATE"
 };
 
 /* Utility methods */
@@ -1326,17 +1447,21 @@ rasqal_query_verb_as_string(rasqal_query_verb verb)
 /**
  * rasqal_query_print:
  * @query: the #rasqal_query object
- * @fh: the #FILE* handle to print to.
+ * @fh: the FILE* handle to print to.
  *
  * Print a query in a debug format.
  * 
+ * Return value: non-0 on failure
  **/
-void
+int
 rasqal_query_print(rasqal_query* query, FILE *fh)
 {
   rasqal_variables_table* vars_table = query->vars_table;
   raptor_sequence* seq;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(fh, FILE*, 1);
+
   fprintf(fh, "query verb: %s\n", rasqal_query_verb_as_string(query->verb));
   
   if(query->distinct)
@@ -1398,7 +1523,17 @@ rasqal_query_print(rasqal_query* query, FILE *fh)
     fputs("\nquery group conditions: ", fh);
     raptor_sequence_print(query->group_conditions_sequence, fh);
   }
+  if(query->having_conditions_sequence) {
+    fputs("\nquery having conditions: ", fh);
+    raptor_sequence_print(query->having_conditions_sequence, fh);
+  }
+  if(query->updates) {
+    fputs("\nupdate operations: ", fh);
+    raptor_sequence_print(query->updates, fh);
+  }
   fputc('\n', fh);
+
+  return 0;
 }
 
 
@@ -1406,6 +1541,9 @@ static int
 rasqal_query_add_query_result(rasqal_query* query,
                               rasqal_query_results* query_results) 
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, rasqal_query_results, 1);
+
   /* add reference to ensure query lives as long as this runs */
 
   /* query->results sequence has rasqal_query_results_remove_query_reference()
@@ -1419,13 +1557,16 @@ rasqal_query_add_query_result(rasqal_query* query,
 
 
 
-void
+int
 rasqal_query_remove_query_result(rasqal_query* query,
                                  rasqal_query_results* query_results) 
 {
   int i;
   int size;
   
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, rasqal_query_results, 1);
+
   size = raptor_sequence_size(query->results);
   for(i = 0 ; i < size; i++) {
     rasqal_query_results *result;
@@ -1436,6 +1577,8 @@ rasqal_query_remove_query_result(rasqal_query* query,
       break;
     }
   }
+
+  return 0;
 }
 
 
@@ -1451,6 +1594,8 @@ rasqal_query_remove_query_result(rasqal_query* query,
 void*
 rasqal_query_get_user_data(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->user_data;
 }
 
@@ -1466,6 +1611,8 @@ rasqal_query_get_user_data(rasqal_query* query)
 void
 rasqal_query_set_user_data(rasqal_query* query, void *user_data)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
   query->user_data = user_data;
 }
 
@@ -1481,6 +1628,8 @@ rasqal_query_set_user_data(rasqal_query* query, void *user_data)
 rasqal_query_verb
 rasqal_query_get_verb(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, RASQAL_QUERY_VERB_UNKNOWN);
+
   return query->verb;
 }
 
@@ -1496,6 +1645,8 @@ rasqal_query_get_verb(rasqal_query* query)
 int
 rasqal_query_get_wildcard(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+
   return query->wildcard;
 }
 
@@ -1511,6 +1662,8 @@ rasqal_query_get_wildcard(rasqal_query* query)
 raptor_sequence*
 rasqal_query_get_order_conditions_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->order_conditions_sequence;
 }
 
@@ -1527,6 +1680,8 @@ rasqal_query_get_order_conditions_sequence(rasqal_query* query)
 rasqal_expression*
 rasqal_query_get_order_condition(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!query->order_conditions_sequence)
     return NULL;
   
@@ -1545,6 +1700,8 @@ rasqal_query_get_order_condition(rasqal_query* query, int idx)
 raptor_sequence*
 rasqal_query_get_group_conditions_sequence(rasqal_query* query)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return query->group_conditions_sequence;
 }
 
@@ -1561,6 +1718,8 @@ rasqal_query_get_group_conditions_sequence(rasqal_query* query)
 rasqal_expression*
 rasqal_query_get_group_condition(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   if(!query->group_conditions_sequence)
     return NULL;
   
@@ -1568,6 +1727,44 @@ rasqal_query_get_group_condition(rasqal_query* query, int idx)
 }
 
 
+/**
+ * rasqal_query_get_having_conditions_sequence:
+ * @query: #rasqal_query query object
+ *
+ * Get the sequence of query having conditions.
+ *
+ * Return value: a #raptor_sequence of #rasqal_expression pointers.
+ **/
+raptor_sequence*
+rasqal_query_get_having_conditions_sequence(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  return query->having_conditions_sequence;
+}
+
+
+/**
+ * rasqal_query_get_having_condition:
+ * @query: #rasqal_query query object
+ * @idx: index into the sequence (0 or larger)
+ *
+ * Get a query having expression in the sequence of query havinging conditions.
+ *
+ * Return value: a #rasqal_expression pointer or NULL if out of the sequence range
+ **/
+rasqal_expression*
+rasqal_query_get_having_condition(rasqal_query* query, int idx)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(!query->having_conditions_sequence)
+    return NULL;
+  
+  return (rasqal_expression*)raptor_sequence_get_at(query->having_conditions_sequence, idx);
+}
+
+
 /**
  * rasqal_query_graph_pattern_visit:
  * @query: query
@@ -1583,7 +1780,11 @@ rasqal_query_graph_pattern_visit(rasqal_query* query,
                                  rasqal_graph_pattern_visit_fn visit_fn, 
                                  void* data)
 {
-  rasqal_graph_pattern* gp = rasqal_query_get_query_graph_pattern(query);
+  rasqal_graph_pattern* gp;
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  gp = rasqal_query_get_query_graph_pattern(query);
   if(!gp)
     return;
 
@@ -1613,13 +1814,11 @@ rasqal_query_write(raptor_iostream* iostr, rasqal_query* query,
 {
   const char *format_uri_str = NULL;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(iostr, raptor_iostream, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
   if(format_uri)
-#ifdef RAPTOR_V2_AVAILABLE
-    format_uri_str = (const char*)raptor_uri_as_string_v2(query->world->raptor_world_ptr,
-                                                          format_uri);
-#else
     format_uri_str = (const char*)raptor_uri_as_string(format_uri);
-#endif
 
   if(!format_uri ||
      !strcmp(format_uri_str,
@@ -1651,6 +1850,10 @@ rasqal_query_iostream_write_escaped_counted_string(rasqal_query* query,
                                                    const unsigned char* string,
                                                    size_t len)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(iostr, raptor_iostream, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, char*, 1);
+
   if(query->factory->iostream_write_escaped_counted_string)
     return query->factory->iostream_write_escaped_counted_string(query, iostr, 
                                                                  string, len);
@@ -1683,8 +1886,17 @@ rasqal_query_escape_counted_string(rasqal_query* query,
   void* output_string = NULL;
   int rc;
   
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, char*, NULL);
+
+#ifdef HAVE_RAPTOR2_API
+  iostr = raptor_new_iostream_to_string(query->world->raptor_world_ptr,
+                                        &output_string, output_len_p,
+                                        rasqal_alloc_memory);
+#else
   iostr = raptor_new_iostream_to_string(&output_string, output_len_p,
                                         rasqal_alloc_memory);
+#endif
   if(!iostr)
     return NULL;
   rc = rasqal_query_iostream_write_escaped_counted_string(query, iostr,
@@ -1707,6 +1919,8 @@ rasqal_query_get_genid(rasqal_query* query, const unsigned char* base,
   int length;
   unsigned char *buffer;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   /* This is read-only and thread safe */
   if(counter < 0)
     counter= query->genid_counter++;
@@ -1728,12 +1942,11 @@ rasqal_query_get_genid(rasqal_query* query, const unsigned char* base,
 void
 rasqal_query_set_base_uri(rasqal_query* query, raptor_uri* base_uri)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(base_uri, raptor_uri);
+
   if(query->base_uri)
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_free_uri_v2(query->world->raptor_world_ptr, query->base_uri);
-#else
     raptor_free_uri(query->base_uri);
-#endif
   query->base_uri = base_uri;
   query->locator.uri = base_uri;
 }
@@ -1742,6 +1955,8 @@ rasqal_query_set_base_uri(rasqal_query* query, raptor_uri* base_uri)
 void
 rasqal_query_set_store_results(rasqal_query* query, int store_results)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
   query->store_results = store_results;
 }
 
@@ -1749,6 +1964,8 @@ rasqal_query_set_store_results(rasqal_query* query, int store_results)
 rasqal_variable* 
 rasqal_query_get_variable_by_offset(rasqal_query* query, int idx)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
   return rasqal_variables_table_get(query->vars_table, idx);
 }
 
@@ -1758,10 +1975,189 @@ rasqal_query_variable_bound_in_triple(rasqal_query *query,
                                       rasqal_variable *v,
                                       int column) 
 {
-  int v_column = query->variables_bound_in[v->offset];
+  int v_column;
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(v, rasqal_variable, 0);
+
+  v_column = query->variables_bound_in[v->offset];
   if(v_column < 0)
     return 0;
 
   return v_column == column;
 }
 
+
+/*
+ * rasqal_query_add_update_operation:
+ * @query: #rasqal_query query object
+ * @update: #rasqal_update_operation to add
+ *
+ * Add a update operation to the query.
+ *
+ * INTERNAL - The @update object becomes owned by the query.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_query_add_update_operation(rasqal_query* query, 
+                                  rasqal_update_operation *update)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
+  if(!update)
+    return 1;
+
+  if(!query->updates) {
+#ifdef HAVE_RAPTOR2_API
+    query->updates = raptor_new_sequence((raptor_data_free_handler)rasqal_free_update_operation, (raptor_data_print_handler)rasqal_update_operation_print);
+#else
+    query->updates = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_update_operation, (raptor_sequence_print_handler*)rasqal_update_operation_print);
+#endif
+    if(!query->updates) {
+      rasqal_free_update_operation(update);
+      return 1;
+    }
+  }
+
+  if(raptor_sequence_push(query->updates, (void*)update))
+    return 1;
+  return 0;
+}
+
+
+/**
+ * rasqal_query_get_update_operations_sequence:
+ * @query: #rasqal_query query object
+ *
+ * Get the sequence of update operations
+ *
+ * Return value: a #raptor_sequence of #rasqal_update_operation pointers.
+ **/
+raptor_sequence*
+rasqal_query_get_update_operations_sequence(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  return query->updates;
+}
+
+
+/**
+ * rasqal_query_get_update_operation:
+ * @query: #rasqal_query query object
+ * @idx: index into the sequence (0 or larger)
+ *
+ * Get a query update operation in the sequence of update operations
+ *
+ * Return value: a #rasqal_update_operation pointer or NULL if out of the sequence range
+ **/
+rasqal_update_operation*
+rasqal_query_get_update_operation(rasqal_query* query, int idx)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(!query->updates)
+    return NULL;
+  
+  return (rasqal_update_operation*)raptor_sequence_get_at(query->updates, idx);
+}
+
+
+/**
+ * rasqal_query_set_default_generate_bnodeid_parameters:
+ * @rdf_query: #rasqal_query object
+ * @prefix: prefix string
+ * @base: integer base identifier
+ *
+ * Set default bnodeid generation parameters
+ *
+ * Sets the parameters for the default algorithm used to generate
+ * blank node IDs.  The default algorithm uses both @prefix and @base
+ * to generate a new identifier.  The exact identifier generated is
+ * not guaranteed to be a strict concatenation of @prefix and @base
+ * but will use both parts.
+ *
+ * For finer control of the generated identifiers, use
+ * rasqal_world_set_generate_bnodeid_handler() on the world class (or
+ * the deprecated rasqal_query_set_generate_bnodeid_handler() on the
+ * query class).
+ *
+ * If prefix is NULL, the default prefix is used (currently "bnodeid")
+ * If base is less than 1, it is initialised to 1.
+ * 
+ * This calls rasqal_world_set_default_generate_bnodeid_parameters()
+ * method on the world class; there is no separate configuration per
+ * query object.
+ *  
+ * @Deprecated: Use
+ * rasqal_world_set_default_generate_bnodeid_parameters() on the
+ * world class.
+ *
+ **/
+void
+rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, 
+                                                     char *prefix, int base)
+{
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(rdf_query, rasqal_query);
+
+  rasqal_world_set_default_generate_bnodeid_parameters(rdf_query->world, 
+                                                       prefix, base);
+}
+
+
+/**
+ * rasqal_query_set_generate_bnodeid_handler:
+ * @query: #rasqal_query query object
+ * @user_data: user data pointer for callback
+ * @handler: generate blank ID callback function
+ *
+ * Set the generate blank node ID handler function for the query.
+ *
+ * Sets the function to generate blank node IDs for the query.
+ * The handler is called with a pointer to the rasqal_query, the
+ * @user_data pointer and a user_bnodeid which is the value of
+ * a user-provided blank node identifier (may be NULL).
+ * It can either be returned directly as the generated value when present or
+ * modified.  The passed in value must be free()d if it is not used.
+ *
+ * If handler is NULL, the default method is used
+ *
+ * Any hander set by rasqal_world_set_generate_bnodeid_handler()
+ * overrides this.
+ *  
+ * @Deprecated: Use
+ * rasqal_world_set_generate_bnodeid_handler() on the world class.
+ * 
+ **/
+void
+rasqal_query_set_generate_bnodeid_handler(rasqal_query* query,
+                                          void *user_data,
+                                          rasqal_generate_bnodeid_handler handler)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  query->generate_bnodeid_handler_user_data = user_data;
+  query->generate_bnodeid_handler = handler;
+}
+
+
+/*
+ * rasqal_query_generate_bnodeid - Default generate id - internal
+ */
+unsigned char*
+rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
+                              unsigned char *user_bnodeid)
+{
+  /* prefer world generate handler/data */
+  if(rdf_query->world->generate_bnodeid_handler)
+    return rasqal_world_generate_bnodeid(rdf_query->world, user_bnodeid);
+
+  if(rdf_query->generate_bnodeid_handler)
+    return rdf_query->generate_bnodeid_handler(rdf_query, 
+                                               rdf_query->generate_bnodeid_handler_user_data, user_bnodeid);
+  else
+    return rasqal_world_default_generate_bnodeid_handler(rdf_query->world,
+                                                         user_bnodeid);
+}
