@@ -51,7 +51,7 @@
 /* prototype for later */
 static int rasqal_query_build_variables_use_map(rasqal_query* query);
 static void rasqal_query_graph_build_variables_use_map_in_internal(rasqal_query* query, short *use_map, rasqal_literal *origin);
-static void rasqal_query_filter_build_variables_use_map_in_internal(rasqal_query* query, short *use_map, rasqal_expression* e);
+static void rasqal_query_expression_build_variables_use_map(short *use_map, rasqal_expression* e);
 static void rasqal_query_let_build_variables_use_map_in_internal(rasqal_query* query, short *use_map, rasqal_variable *var, rasqal_expression* e);
 
 
@@ -252,7 +252,11 @@ rasqal_query_expand_wildcards(rasqal_query* rq)
     return 0;
   
   /* If 'SELECT *' was given, make the selects be a list of all variables */
+#ifdef HAVE_RAPTOR2_API
+  rq->selects = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
+#else
   rq->selects = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
   if(!rq->selects)
     return 1;
   
@@ -294,8 +298,12 @@ rasqal_query_remove_duplicate_select_vars(rasqal_query* rq)
   size = raptor_sequence_size(seq);
   if(!size)
     return 0;
-  
+
+#ifdef HAVE_RAPTOR2_API
+  new_seq = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
+#else
   new_seq = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
   if(!new_seq)
     return 1;
   
@@ -320,7 +328,8 @@ rasqal_query_remove_duplicate_select_vars(rasqal_query* rq)
       
       if(v == v2) {
         if(!warned) {
-          rasqal_log_error_simple(rq->world, RAPTOR_LOG_LEVEL_WARNING,
+          rasqal_log_error_simple(rq->world,
+                                  RAPTOR_LOG_LEVEL_WARN,
                                   &rq->locator,
                                   "Variable %s duplicated in SELECT.", 
                                   v->name);
@@ -553,7 +562,8 @@ rasqal_query_check_unused_variables(rasqal_query* query, int *bound_in)
 
     v = rasqal_variables_table_get(query->vars_table, i);
     if(column == BOUND_IN_UNBOUND)
-      rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_WARNING,
+      rasqal_log_error_simple(query->world,
+                              RAPTOR_LOG_LEVEL_WARN,
                               &query->locator,
                               "Variable %s was selected but is unused in the query.", 
                               v->name);
@@ -674,7 +684,11 @@ rasqal_query_merge_triple_patterns(rasqal_query* query,
 
     RASQAL_DEBUG3("Initial columns %d to %d\n", gp->start_column, gp->end_column);
   #endif
+#ifdef HAVE_RAPTOR2_API
+    seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
+#else
     seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+#endif
     if(!seq)
       return 1;
     for(i = 0; raptor_sequence_size(gp->graph_patterns) > 0; i++) {
@@ -789,8 +803,11 @@ rasqal_query_remove_empty_group_graph_patterns(rasqal_query* query,
     return 0;
   }
   
-  
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
+#else
   seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+#endif
   if(!seq) {
     RASQAL_DEBUG1("Cannot create new gp sequence\n");
     *modified = -1;
@@ -1324,8 +1341,13 @@ rasqal_graph_patterns_join(rasqal_graph_pattern *dest_gp,
 
   if(src_gp->graph_patterns) {
     if(!dest_gp->graph_patterns) {
+#ifdef HAVE_RAPTOR2_API
+      dest_gp->graph_patterns = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern,
+                                                    (raptor_data_print_handler)rasqal_graph_pattern_print);
+#else
       dest_gp->graph_patterns = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern,
                                                     (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+#endif
       if(!dest_gp->graph_patterns)
         return -1;
     }
@@ -1461,9 +1483,8 @@ rasqal_query_graph_pattern_build_variables_use_map(rasqal_query* query,
       break;
       
     case RASQAL_GRAPH_PATTERN_OPERATOR_FILTER:
-      rasqal_query_filter_build_variables_use_map_in_internal(query, 
-                                                              &use_map[offset],
-                                                              gp->filter_expression);
+      rasqal_query_expression_build_variables_use_map(&use_map[offset],
+                                                      gp->filter_expression);
       break;
 
     case RASQAL_GRAPH_PATTERN_OPERATOR_LET:
@@ -1554,8 +1575,8 @@ rasqal_query_print_variables_use_map(FILE* fh, rasqal_query* query)
 
 /* for use with rasqal_expression_visit and user_data=rasqal_query */
 static int
-rasqal_expression_expr_build_variables_use_map_row(short *use_map_row,
-                                                   rasqal_expression *e)
+rasqal_query_expression_build_variables_use_map_row(short *use_map_row,
+                                                    rasqal_expression *e)
 {
   if(e->literal) {
     rasqal_variable* v;
@@ -1589,11 +1610,7 @@ rasqal_query_build_variables_sequence_use_map_row(short* use_map_row,
 
     e = v->expression;
     if(e) {
-      rc = rasqal_expression_expr_build_variables_use_map_row(use_map_row,
-                                                              v->expression);
-      if(rc)
-        break;
-
+      rasqal_query_expression_build_variables_use_map(use_map_row, e);
       flags |= RASQAL_VAR_USE_BOUND_HERE;
     }
 
@@ -1634,8 +1651,7 @@ rasqal_query_build_literals_sequence_use_map_row(short* use_map_row,
  * Mark variables seen in a sequence of expressions
  */
 static int
-rasqal_query_build_expressions_sequence_use_map_row(rasqal_query *query,
-                                                    short* use_map_row,
+rasqal_query_build_expressions_sequence_use_map_row(short* use_map_row,
                                                     raptor_sequence *exprs_seq)
 {
   int rc = 0;
@@ -1648,10 +1664,7 @@ rasqal_query_build_expressions_sequence_use_map_row(rasqal_query *query,
     if(!e)
       break;
 
-    rc = rasqal_expression_expr_build_variables_use_map_row(use_map_row, e);
-    if(rc)
-      break;
-
+    rasqal_query_expression_build_variables_use_map(use_map_row, e);
   }
 
   return rc;
@@ -1696,6 +1709,7 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   int height;
   int rc = 0;
   short *use_map;
+  raptor_sequence* seq;
   
   width = rasqal_variables_table_get_total_variables_count(query->vars_table);
   height = RASQAL_VAR_USE_MAP_OFFSET_LAST + 1 + query->graph_pattern_count;
@@ -1733,7 +1747,8 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
 
     case RASQAL_QUERY_VERB_DELETE:
     case RASQAL_QUERY_VERB_INSERT:
-      /* FIXME - should mark these triple patterns as using vars */
+    case RASQAL_QUERY_VERB_UPDATE:
+      /* FIXME - should mark the verbs using triple patterns as using vars */
       break;
       
     case RASQAL_QUERY_VERB_UNKNOWN:
@@ -1746,15 +1761,30 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
     goto done;
   
 
-  /* FIXME: record variable use for 2) GROUP BY expr/var (SPARQL 1.1 TBD) */
+  /* Record variable use for 2) GROUP BY expressions (SPARQL 1.1) */
+  seq = query->group_conditions_sequence;
+  if(seq) {
+    rc = rasqal_query_build_expressions_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_GROUP_BY],
+                                                             seq);
+    if(rc)
+      goto done;
+  }
+  
 
-  /* FIXME: record variable use for 3) HAVING expr (SPARQL 1.1 TBD) */
+  /* Record variable use for 3) HAVING expr (SPARQL 1.1) */
+  seq = query->having_conditions_sequence;
+  if(seq) {
+    rc = rasqal_query_build_expressions_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_HAVING],
+                                                             seq);
+    if(rc)
+      goto done;
+  }
 
   /* record variable use for 4) ORDER list-of-expr (SPARQL 1.0) */
-  if(query->order_conditions_sequence) {
-    rc = rasqal_query_build_expressions_sequence_use_map_row(query,
-                                                             &use_map[RASQAL_VAR_USE_MAP_OFFSET_ORDER_BY],
-                                                             query->order_conditions_sequence);
+  seq = query->order_conditions_sequence;
+  if(seq) {
+    rc = rasqal_query_build_expressions_sequence_use_map_row(&use_map[RASQAL_VAR_USE_MAP_OFFSET_ORDER_BY],
+                                                             seq);
     if(rc)
       goto done;
   }
@@ -1799,21 +1829,19 @@ rasqal_query_graph_build_variables_use_map_in_internal(rasqal_query* query,
 
 
 /**
- * rasqal_query_filter_build_variables_use_map_in_internal:
- * @query: the #rasqal_query to find the variables in
+ * rasqal_query_expression_build_variables_use_map:
  * @use_map_row: 1D array of size num. variables to write
  * @e: filter expression to use
  *
- * INTERNAL - Mark variables mentioned in a FILTER graph pattern
+ * INTERNAL - Mark variables mentioned in an expression
  * 
  **/
 static void
-rasqal_query_filter_build_variables_use_map_in_internal(rasqal_query* query,
-                                                        short *use_map_row,
-                                                        rasqal_expression* e)
+rasqal_query_expression_build_variables_use_map(short *use_map_row,
+                                                rasqal_expression* e)
 {
   rasqal_expression_visit(e, 
-                          (rasqal_expression_visit_fn)rasqal_expression_expr_build_variables_use_map_row,
+                          (rasqal_expression_visit_fn)rasqal_query_expression_build_variables_use_map_row,
                           use_map_row);
 }
 
@@ -1837,7 +1865,7 @@ rasqal_query_let_build_variables_use_map_in_internal(rasqal_query* query,
                               RASQAL_VAR_USE_MENTIONED_HERE;
 
   rasqal_expression_visit(e, 
-                          (rasqal_expression_visit_fn)rasqal_expression_expr_build_variables_use_map_row,
+                          (rasqal_expression_visit_fn)rasqal_query_expression_build_variables_use_map_row,
                           use_map_row);
 }
 
