@@ -81,6 +81,7 @@ static int rasqal_raptor_triple_present(rasqal_triples_source *rts, void *user_d
 static void rasqal_raptor_free_triples_source(void *user_data);
 
 
+#ifndef HAVE_RAPTOR2_API
 static raptor_uri* 
 ordinal_as_uri(rasqal_world* world, int ordinal) 
 {
@@ -97,17 +98,72 @@ ordinal_as_uri(rasqal_world* world, int ordinal)
     return NULL;
   
   sprintf((char*)buffer, "%s_%d", raptor_rdf_namespace_uri, ordinal);
-#ifdef RAPTOR_V2_AVAILABLE
-  uri = raptor_new_uri_v2(world->raptor_world_ptr, buffer);
-#else
-  uri = raptor_new_uri(buffer);
-#endif
+  uri = raptor_new_uri(world->raptor_world_ptr, buffer);
   RASQAL_FREE(cstring, buffer);
 
   return uri;
 }
+#endif
 
+
+#ifdef HAVE_RAPTOR2_API
+static rasqal_triple*
+raptor_statement_as_rasqal_triple(rasqal_world* world,
+                                  const raptor_statement *statement)
+{
+  rasqal_literal *s, *p, *o;
+  raptor_uri *uri;
+  unsigned char *new_str;
+  size_t len;
+
+  /* subject */
+  if(statement->subject->type == RAPTOR_TERM_TYPE_BLANK) {
+    len = strlen((char*)statement->subject->value.blank.string);
+    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    memcpy(new_str, statement->subject->value.blank.string, len + 1);
+    s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
+  } else {
+    uri = raptor_uri_copy((raptor_uri*)statement->subject->value.uri);
+    s = rasqal_new_uri_literal(world, uri);
+  }
+
+  /* predicate */
+  uri = raptor_uri_copy((raptor_uri*)statement->predicate->value.uri);
+  p = rasqal_new_uri_literal(world, uri);
   
+  /* object */
+  if(statement->object->type == RAPTOR_TERM_TYPE_LITERAL) {
+    char *language = NULL;
+
+    len = strlen((char*)statement->object->value.literal.string);
+    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    memcpy(new_str, statement->object->value.literal.string, len + 1);
+
+    if(statement->object->value.literal.language) {
+      len = strlen((const char*)statement->object->value.literal.language);
+      language = (char*)RASQAL_MALLOC(cstring, len + 1);
+      memcpy(language, statement->object->value.literal.language, len + 1);
+    }
+
+    if(statement->object->value.literal.datatype)
+      uri = raptor_uri_copy(statement->object->value.literal.datatype);
+    else
+      uri = NULL;
+
+    o = rasqal_new_string_literal(world, new_str, language, uri, NULL);
+  } else if(statement->object->type == RAPTOR_TERM_TYPE_BLANK) {
+    len = strlen((char*)statement->object->value.blank.string);
+    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    memcpy(new_str, statement->object->value.blank.string, len + 1);
+    o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
+  } else {
+    uri = raptor_uri_copy((raptor_uri*)statement->object->value.uri);
+    o = rasqal_new_uri_literal(world, uri);
+  }
+
+  return rasqal_new_triple(s, p, o);
+}
+#else /* ifdef HAVE_RAPTOR_API */
 static rasqal_triple*
 raptor_statement_as_rasqal_triple(rasqal_world* world,
                                   const raptor_statement *statement)
@@ -120,7 +176,7 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     blank_len = strlen((char*)statement->subject);
     new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    strncpy((char*)new_blank, (const char*)statement->subject, blank_len + 1);
+    memcpy(new_blank, statement->subject, blank_len + 1);
     s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
   } else if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
     raptor_uri* uri;
@@ -132,12 +188,7 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
   } else {
     raptor_uri *uri;
 
-#ifdef RAPTOR_V2_AVAILABLE
-    uri = raptor_uri_copy_v2(world->raptor_world_ptr,
-                             (raptor_uri*)statement->subject);
-#else
     uri = raptor_uri_copy((raptor_uri*)statement->subject);
-#endif
     s = rasqal_new_uri_literal(world, uri);
   }
 
@@ -150,12 +201,7 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
   } else {
     raptor_uri *uri;
 
-#ifdef RAPTOR_V2_AVAILABLE
-    uri = raptor_uri_copy_v2(world->raptor_world_ptr,
-                             (raptor_uri*)statement->predicate);
-#else
     uri = raptor_uri_copy((raptor_uri*)statement->predicate);
-#endif
     p = rasqal_new_uri_literal(world, uri);
   }
   
@@ -168,33 +214,24 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     literal_len = strlen((char*)statement->object);
     string = (unsigned char*)RASQAL_MALLOC(cstring, literal_len + 1);
-    strncpy((char*)string, (const char*)statement->object, literal_len + 1);
+    memcpy(string, statement->object, literal_len + 1);
 
     if(statement->object_literal_language) {
       size_t language_len;
       language_len = strlen((const char*)statement->object_literal_language);
       language = (char*)RASQAL_MALLOC(cstring, language_len + 1);
-      strncpy(language, (const char*)statement->object_literal_language,
-              language_len + 1);
+      memcpy(language, statement->object_literal_language, language_len + 1);
     }
 
     if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
       const unsigned char* dt_uri_string;
 
       dt_uri_string = (const unsigned char*)raptor_xml_literal_datatype_uri_string;
-#ifdef RAPTOR_V2_AVAILABLE
-      uri = raptor_new_uri_v2(world->raptor_world_ptr, dt_uri_string);
-#else
-      uri = raptor_new_uri(dt_uri_string);
-#endif
+      uri = raptor_new_uri(world->raptor_world_ptr, dt_uri_string);
     } else if(statement->object_literal_datatype) {
       raptor_uri *dt_uri = (raptor_uri*)statement->object_literal_datatype;
 
-#ifdef RAPTOR_V2_AVAILABLE
-      uri = raptor_uri_copy_v2(world->raptor_world_ptr, dt_uri);
-#else
       uri = raptor_uri_copy(dt_uri);
-#endif
     }
     o = rasqal_new_string_literal(world, string, language, uri, NULL);
   } else if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
@@ -204,7 +241,7 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     blank_len = strlen(blank);
     new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    strncpy((char*)new_blank, (const char*)blank, blank_len + 1);
+    memcpy(new_blank, blank, blank_len + 1);
     o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
   } else if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
     raptor_uri* uri = ordinal_as_uri(world, *((int*)statement->object));
@@ -214,24 +251,21 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
     o = rasqal_new_uri_literal(world, uri);
   } else {
     raptor_uri *uri;
-
-#ifdef RAPTOR_V2_AVAILABLE
-    uri = raptor_uri_copy_v2(world->raptor_world_ptr,
-                             (raptor_uri*)statement->object);
-#else
     uri = raptor_uri_copy((raptor_uri*)statement->object);
-#endif
     o = rasqal_new_uri_literal(world, uri);
   }
 
   return rasqal_new_triple(s, p, o);
 }
-
+#endif /* ifdef HAVE_RAPTOR2_API ... else */
 
 
 static void
-rasqal_raptor_statement_handler(void *user_data, 
-                                const raptor_statement *statement) 
+rasqal_raptor_statement_handler(void *user_data,
+#ifndef HAVE_RAPTOR2_API
+                                const
+#endif
+                                raptor_statement *statement)
 {
   rasqal_raptor_triples_source_user_data* rtsc;
   rasqal_raptor_triple *triple;
@@ -259,6 +293,7 @@ rasqal_raptor_statement_handler(void *user_data,
 }
 
 
+#ifndef HAVE_RAPTOR2_API
 static void
 rasqal_raptor_error_handler(void *user_data, 
                             raptor_locator* locator, const char *message)
@@ -269,17 +304,15 @@ rasqal_raptor_error_handler(void *user_data,
   query->failed = 1;
 
   if(locator &&
-#ifdef RAPTOR_V2_AVAILABLE
-     (locator_len = raptor_format_locator_v2(query->world->raptor_world_ptr,
-                                             NULL, 0, locator)) > 0
+#ifdef HAVE_RAPTOR2_API
+     (locator_len = raptor_locator_format(NULL, 0, locator)) > 0
 #else
      (locator_len = raptor_format_locator(NULL, 0, locator)) > 0
 #endif
     ) {
     char *buffer = (char*)RASQAL_MALLOC(cstring, locator_len + 1);
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_format_locator_v2(query->world->raptor_world_ptr, buffer,
-                             locator_len, locator);
+#ifdef HAVE_RAPTOR2_API
+    raptor_locator_format(buffer, locator_len, locator);
 #else
     raptor_format_locator(buffer, locator_len, locator);
 #endif
@@ -293,11 +326,15 @@ rasqal_raptor_error_handler(void *user_data,
                             &query->locator,
                             "Failed to parse - %s", message);
 }
+#endif
 
 
 static unsigned char*
 rasqal_raptor_generate_id_handler(void *user_data,
+#ifdef HAVE_RAPTOR2_API
+#else
                                   raptor_genid_type type,
+#endif
                                   unsigned char *user_bnodeid) 
 {
   rasqal_raptor_triples_source_user_data* rtsc;
@@ -311,11 +348,10 @@ rasqal_raptor_generate_id_handler(void *user_data,
     mapped_id = (unsigned char*)RASQAL_MALLOC(cstring, 
                                               rtsc->mapped_id_base_len + 1 + 
                                               user_bnodeid_len + 1);
-    strncpy((char*)mapped_id, (const char*)rtsc->mapped_id_base, 
-            rtsc->mapped_id_base_len);
+    memcpy(mapped_id, rtsc->mapped_id_base, rtsc->mapped_id_base_len);
     mapped_id[rtsc->mapped_id_base_len] = '_';
-    strncpy((char*)(mapped_id+rtsc->mapped_id_base_len + 1),
-            (const char*)user_bnodeid, user_bnodeid_len + 1);
+    memcpy(mapped_id + rtsc->mapped_id_base_len + 1,
+           user_bnodeid, user_bnodeid_len + 1);
 
     raptor_free_memory(user_bnodeid);
     return mapped_id;
@@ -369,20 +405,8 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
     name_uri = dg->name_uri;
 
     rtsc->source_index = i;
-
-#ifdef RAPTOR_V2_AVAILABLE
-    rtsc->source_uri = raptor_uri_copy_v2(rdf_query->world->raptor_world_ptr,
-                                          uri);
-    if(name_uri)
-      rtsc->source_literals[i] = rasqal_new_uri_literal(rdf_query->world, 
-                                                        raptor_uri_copy_v2(rdf_query->world->raptor_world_ptr, name_uri)
-                                                        );
-    else {
-      name_uri = raptor_uri_copy_v2(rdf_query->world->raptor_world_ptr, uri);
-      free_name_uri = 1;
-    }
-#else
     rtsc->source_uri = raptor_uri_copy(uri);
+
     if(name_uri)
       rtsc->source_literals[i] = rasqal_new_uri_literal(rdf_query->world, 
                                                         raptor_uri_copy(name_uri)
@@ -391,22 +415,25 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
       name_uri = raptor_uri_copy(uri);
       free_name_uri = 1;
     }
-#endif
 
     rtsc->mapped_id_base = rasqal_query_get_genid(rdf_query,
                                                   (const unsigned char*)"graphid",
                                                   i);
     rtsc->mapped_id_base_len = strlen((const char*)rtsc->mapped_id_base);
 
-#ifdef RAPTOR_V2_AVAILABLE
-    parser = raptor_new_parser_v2(rdf_query->world->raptor_world_ptr, "guess");
+#ifdef HAVE_RAPTOR2_API
+    parser = raptor_new_parser(rdf_query->world->raptor_world_ptr, "guess");
+    raptor_parser_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
+    raptor_world_set_generate_bnodeid_handler(rdf_query->world->raptor_world_ptr,
+                                              rtsc,
+                                              rasqal_raptor_generate_id_handler);
 #else
     parser = raptor_new_parser("guess");
-#endif
     raptor_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
     raptor_set_error_handler(parser, rdf_query, rasqal_raptor_error_handler);
     raptor_set_generate_id_handler(parser, rtsc,
                                    rasqal_raptor_generate_id_handler);
+#endif
 
 #ifdef RAPTOR_FEATURE_NO_NET
     if(rdf_query->features[RASQAL_FEATURE_NO_NET])
@@ -414,22 +441,17 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
                          rdf_query->features[RASQAL_FEATURE_NO_NET]);
 #endif
 
+#ifdef HAVE_RAPTOR2_API
+    raptor_parser_parse_uri(parser, uri, name_uri);
+#else
     raptor_parse_uri(parser, uri, name_uri);
+#endif
     raptor_free_parser(parser);
 
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_free_uri_v2(rdf_query->world->raptor_world_ptr, rtsc->source_uri);
-#else
     raptor_free_uri(rtsc->source_uri);
-#endif
 
-    if(free_name_uri) {
-#ifdef RAPTOR_V2_AVAILABLE
-      raptor_free_uri_v2(rdf_query->world->raptor_world_ptr, name_uri);
-#else
+    if(free_name_uri)
       raptor_free_uri(name_uri);
-#endif
-    }
 
     /* This is freed in rasqal_raptor_free_triples_source() */
     /* rasqal_free_literal(rtsc->source_literal); */
@@ -503,13 +525,8 @@ rasqal_raptor_triple_match(rasqal_world* world,
       if(match->origin->type == RASQAL_LITERAL_URI ) {
         raptor_uri* triple_uri = triple->origin->value.uri;
         raptor_uri* match_uri = match->origin->value.uri;
-#ifdef RAPTOR_V2_AVAILABLE
-        if(!raptor_uri_equals_v2(world->raptor_world_ptr, triple_uri, match_uri))
-          goto done;
-#else
         if(!raptor_uri_equals(triple_uri, match_uri))
           goto done;
-#endif
       }
     }
     
