@@ -87,8 +87,8 @@ rasqal_init_result_formats(rasqal_world* world)
 {
   int rc = 0;
   
-#ifdef RAPTOR_V2_AVAILABLE
-  world->query_results_formats = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_query_results_format_factory, NULL);
+#ifdef HAVE_RAPTOR2_API
+  world->query_results_formats = raptor_new_sequence((raptor_data_free_handler)rasqal_free_query_results_format_factory, NULL);
 #else
   world->query_results_formats = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_query_results_format_factory, NULL);
 #endif
@@ -103,6 +103,8 @@ rasqal_init_result_formats(rasqal_world* world)
 
   rc += rasqal_init_result_format_sv(world) != 0;
 
+  rc += rasqal_init_result_format_html(world) != 0;
+
   return rc;
 }
 
@@ -377,7 +379,8 @@ rasqal_new_query_results_formatter_by_mime_type(rasqal_world* world,
 void
 rasqal_free_query_results_formatter(rasqal_query_results_formatter* formatter) 
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(formatter, rasqal_query_results_formatter);
+  if(!formatter)
+    return;
 
   RASQAL_FREE(rasqal_query_results_formatter, formatter);
 }
@@ -405,9 +408,12 @@ rasqal_query_results_formatter_get_mime_type(rasqal_query_results_formatter *for
  * @iostr: #raptor_iostream to write the query to
  * @formatter: #rasqal_query_results_formatter object
  * @results: #rasqal_query_results query results format
- * @base_uri: #raptor_uri base URI of the output format
+ * @base_uri: #raptor_uri base URI of the output format (or NULL)
  *
  * Write the query results using the given formatter to an iostream
+ *
+ * Note that after calling this method, the query results will be
+ * empty and rasqal_query_results_finished() will return true (non-0)
  * 
  * See rasqal_query_results_formats_enumerate() to get the
  * list of syntax URIs and their description. 
@@ -423,7 +429,6 @@ rasqal_query_results_formatter_write(raptor_iostream *iostr,
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(iostr, raptor_iostream, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(formatter, rasqal_query_results_formatter, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(results, rasqal_query_results, 1);
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, raptor_uri, 1);
 
   if(!formatter->factory->writer)
      return 1;
