@@ -96,7 +96,11 @@ rasqal_new_rowsource_from_handler(rasqal_world* world,
     rowsource->vars_table = NULL;
 
   /* no free method here - the variables are owned by rowsource->vars_table */
+#ifdef HAVE_RAPTOR2_API
+  rowsource->variables_sequence = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
+#else
   rowsource->variables_sequence = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
   if(!rowsource->variables_sequence) {
     rasqal_free_rowsource(rowsource);
     return NULL;
@@ -116,7 +120,7 @@ rasqal_new_rowsource_from_handler(rasqal_world* world,
  * rasqal_free_rowsource:
  * @rowsource: rowsource object
  *
- * Destructor - destroy an rowsource.
+ * INTERNAL - Destructor - destroy an rowsource.
  **/
 void
 rasqal_free_rowsource(rasqal_rowsource *rowsource)
@@ -295,13 +299,23 @@ rasqal_rowsource_read_all_rows(rasqal_rowsource *rowsource)
   if(rowsource->handler->read_all_rows) {
     seq = rowsource->handler->read_all_rows(rowsource, rowsource->user_data);
     if(!seq)
+#ifdef HAVE_RAPTOR2_API
+      seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
+                                (raptor_data_print_handler)rasqal_row_print);
+#else
       seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row,
                                 (raptor_sequence_print_handler*)rasqal_row_print);
+#endif
     return seq;
   }
-  
+
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
+                            (raptor_data_print_handler)rasqal_row_print);
+#else
   seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row,
                             (raptor_sequence_print_handler*)rasqal_row_print);
+#endif
   if(!seq)
     return NULL;
   
@@ -580,7 +594,7 @@ rasqal_rowsource_write_indent(raptor_iostream *iostr, int indent)
 {
   while(indent > 0) {
     int sp = (indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
-    raptor_iostream_write_bytes(iostr, spaces, sizeof(char), sp);
+    raptor_iostream_write_bytes(spaces, sizeof(char), sp, iostr);
     indent -= sp;
   }
 }
@@ -598,8 +612,8 @@ rasqal_rowsource_write_internal(rasqal_rowsource *rowsource,
   
   indent_delta = strlen(rs_name);
 
-  raptor_iostream_write_counted_string(iostr, rs_name, indent_delta);
-  raptor_iostream_write_counted_string(iostr, "(\n", 2);
+  raptor_iostream_counted_string_write(rs_name, indent_delta, iostr);
+  raptor_iostream_counted_string_write("(\n", 2, iostr);
   indent_delta++;
   
   indent += indent_delta;
@@ -610,18 +624,18 @@ rasqal_rowsource_write_internal(rasqal_rowsource *rowsource,
       (inner_rowsource = rasqal_rowsource_get_inner_rowsource(rowsource, offset));
       offset++) {
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_rowsource_write_indent(iostr, indent);
       }
       rasqal_rowsource_write_internal(inner_rowsource, iostr, indent);
       arg_count++;
   }
 
-  raptor_iostream_write_byte(iostr, '\n');
+  raptor_iostream_write_byte('\n', iostr);
   indent-= indent_delta;
 
   rasqal_rowsource_write_indent(iostr, indent);
-  raptor_iostream_write_byte(iostr, ')');
+  raptor_iostream_write_byte(')', iostr);
 
   return 0;
 }
@@ -637,7 +651,7 @@ rasqal_rowsource_write(rasqal_rowsource *rowsource, raptor_iostream *iostr)
 /**
  * rasqal_rowsource_print:
  * @rs: the #rasqal_rowsource object
- * @fh: the #FILE* handle to print to
+ * @fh: the FILE* handle to print to
  *
  * Print a #rasqal_rowsource in a debug format.
  * 
@@ -649,7 +663,7 @@ rasqal_rowsource_print(rasqal_rowsource *rowsource, FILE* fh)
 {
   raptor_iostream *iostr;
 
-  iostr = raptor_new_iostream_to_file_handle(fh);
+  iostr = raptor_new_iostream_to_file_handle(rowsource->world->raptor_world_ptr, fh);
   rasqal_rowsource_write(rowsource, iostr);
   raptor_free_iostream(iostr);
 }
