@@ -157,7 +157,8 @@ rasqal_new_variable_from_variable(rasqal_variable* v)
 void
 rasqal_free_variable(rasqal_variable* v)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(v, rasqal_variable);
+  if(!v)
+    return;
   
   if(v->name)
     RASQAL_FREE(cstring, (void*)v->name);
@@ -183,33 +184,34 @@ void
 rasqal_variable_write(rasqal_variable* v, raptor_iostream* iostr)
 {
   if(v->type == RASQAL_VARIABLE_TYPE_ANONYMOUS)
-    raptor_iostream_write_counted_string(iostr, "anon-variable(", 14);
+    raptor_iostream_counted_string_write("anon-variable(", 14, iostr);
   else
-    raptor_iostream_write_counted_string(iostr, "variable(", 9);
-  raptor_iostream_write_string(iostr, v->name);
+    raptor_iostream_counted_string_write("variable(", 9, iostr);
+  raptor_iostream_string_write(v->name, iostr);
   if(v->expression) {
-    raptor_iostream_write_byte(iostr, '=');
+    raptor_iostream_write_byte('=', iostr);
     rasqal_expression_write(v->expression, iostr);
   }
   if(v->value) {
-    raptor_iostream_write_byte(iostr, '=');
+    raptor_iostream_write_byte('=', iostr);
     rasqal_literal_write(v->value, iostr);
   }
-  raptor_iostream_write_byte(iostr, ')');
+  raptor_iostream_write_byte(')', iostr);
 }
 
 
 /**
  * rasqal_variable_print:
  * @v: the #rasqal_variable object
- * @fh: the #FILE* handle to print to
+ * @fh: the FILE* handle to print to
  *
  * Print a Rasqal variable in a debug format.
  * 
  * The print debug format may change in any release.
  * 
+ * Return value: non-0 on failure
  **/
-void
+int
 rasqal_variable_print(rasqal_variable* v, FILE* fh)
 {
   if(v->type == RASQAL_VARIABLE_TYPE_ANONYMOUS)
@@ -225,6 +227,8 @@ rasqal_variable_print(rasqal_variable* v, FILE* fh)
     rasqal_literal_print(v->value, fh);
   }
   fputc(')', fh);
+
+  return 0;
 }
 
 
@@ -294,6 +298,14 @@ struct rasqal_variables_table_s {
 
 
 
+/**
+ * rasqal_new_variables_table:
+ * @world: rasqal world
+ *
+ * Constructor - create a new variables table
+ *
+ * Return value: new variables table or NULL On failure
+ */
 rasqal_variables_table*
 rasqal_new_variables_table(rasqal_world* world)
 {
@@ -306,12 +318,24 @@ rasqal_new_variables_table(rasqal_world* world)
 
   vt->usage = 1;
   vt->world = world;
-  
-  vt->variables_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable, (raptor_sequence_print_handler*)rasqal_variable_print);
+
+#ifdef HAVE_RAPTOR2_API
+  vt->variables_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                               (raptor_data_print_handler)rasqal_variable_print);
+#else
+  vt->variables_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                                               (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
   if(!vt->variables_sequence)
     goto tidy;
 
-  vt->anon_variables_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable, (raptor_sequence_print_handler*)rasqal_variable_print);
+#ifdef HAVE_RAPTOR2_API
+  vt->anon_variables_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                                    (raptor_data_print_handler)rasqal_variable_print);
+#else
+  vt->anon_variables_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                                                    (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
   if(!vt->anon_variables_sequence)
     goto tidy;
 
@@ -335,10 +359,17 @@ rasqal_new_variables_table_from_variables_table(rasqal_variables_table* vt)
 }
 
 
+/**
+ * rasqal_free_variables_table:
+ * @vt: rasqal variables table
+ *
+ * Destructor - destroy a new variables table
+ */
 void
 rasqal_free_variables_table(rasqal_variables_table* vt)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(vt, rasqal_variables_table);
+  if(!vt)
+    return;
 
   if(--vt->usage)
     return;
@@ -366,7 +397,7 @@ rasqal_free_variables_table(rasqal_variables_table* vt)
  * @name: variable name
  * @value: variable #rasqal_literal value (or NULL)
  *
- * Constructor - Add a variable to the variables table
+ * Constructor - Create a new variable and add it to the variables table
  * 
  * The @name and @value become owned by the rasqal_variable structure
  *
@@ -622,7 +653,7 @@ main(int argc, char *argv[])
   for(i = 0; i < NUM_VARS; i++) {
     size_t len = strlen(var_names[i]);
     names[i] = (unsigned char*)malloc(len+1);
-    strncpy((char*)names[i], var_names[i], len+1);
+    memcpy(names[i], var_names[i], len + 1);
   }
   
   vars[0] = rasqal_variables_table_add(vt, RASQAL_VARIABLE_TYPE_NORMAL,
