@@ -580,7 +580,7 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   202,   202,   206,   235,   240,   245,   254,   258,   265,
+       0,   198,   198,   202,   231,   236,   241,   254,   258,   265,
      269,   274,   283,   288,   293,   306,   310,   314,   318,   329,
      334,   339,   343,   347,   355,   360,   365,   370,   375,   383,
      387,   393,   398,   404,   408,   412,   416,   420,   426,   430,
@@ -1265,11 +1265,10 @@ yysyntax_error (char *yyresult, int yystate, int yychar)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yydestruct (void *YYPARSE_PARAM, const char *yymsg, int yytype, YYSTYPE *yyvaluep)
+yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
 #else
 static void
-yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
-    void *YYPARSE_PARAM;
+yydestruct (yymsg, yytype, yyvaluep)
     const char *yymsg;
     int yytype;
     YYSTYPE *yyvaluep;
@@ -1293,7 +1292,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1297 "rdql_parser.c"
+#line 1296 "rdql_parser.c"
 	break;
       case 46: /* "\"string literal\"" */
 
@@ -1305,7 +1304,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1309 "rdql_parser.c"
+#line 1308 "rdql_parser.c"
 	break;
       case 47: /* "\"integer literal\"" */
 
@@ -1317,7 +1316,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1321 "rdql_parser.c"
+#line 1320 "rdql_parser.c"
 	break;
       case 48: /* "\"pattern literal\"" */
 
@@ -1329,7 +1328,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1333 "rdql_parser.c"
+#line 1332 "rdql_parser.c"
 	break;
       case 49: /* "\"boolean literal\"" */
 
@@ -1341,7 +1340,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1345 "rdql_parser.c"
+#line 1344 "rdql_parser.c"
 	break;
       case 50: /* "\"null\"" */
 
@@ -1353,7 +1352,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1357 "rdql_parser.c"
+#line 1356 "rdql_parser.c"
 	break;
       case 51: /* "\"URI literal\"" */
 
@@ -1361,39 +1360,35 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 #line 185 "./rdql_parser.y"
 	{
   if((yyvaluep->uri))
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_free_uri_v2(((rasqal_query*)rq)->world->raptor_world_ptr, (yyvaluep->uri));
-#else    
     raptor_free_uri((yyvaluep->uri));
-#endif
 };
 
 /* Line 1000 of yacc.c  */
-#line 1373 "rdql_parser.c"
+#line 1368 "rdql_parser.c"
 	break;
       case 52: /* "\"QName literal\"" */
 
 /* Line 1000 of yacc.c  */
-#line 194 "./rdql_parser.y"
+#line 190 "./rdql_parser.y"
 	{
   if((yyvaluep->name))
     RASQAL_FREE(cstring, (yyvaluep->name));
 };
 
 /* Line 1000 of yacc.c  */
-#line 1385 "rdql_parser.c"
+#line 1380 "rdql_parser.c"
 	break;
       case 53: /* "\"identifier\"" */
 
 /* Line 1000 of yacc.c  */
-#line 194 "./rdql_parser.y"
+#line 190 "./rdql_parser.y"
 	{
   if((yyvaluep->name))
     RASQAL_FREE(cstring, (yyvaluep->name));
 };
 
 /* Line 1000 of yacc.c  */
-#line 1397 "rdql_parser.c"
+#line 1392 "rdql_parser.c"
 	break;
 
       default:
@@ -1697,7 +1692,7 @@ yyreduce:
         case 3:
 
 /* Line 1455 of yacc.c  */
-#line 207 "./rdql_parser.y"
+#line 203 "./rdql_parser.y"
     {
   ((rasqal_query*)rq)->selects=(yyvsp[(2) - (7)].seq);
   ((rasqal_query*)rq)->verb=RASQAL_QUERY_VERB_SELECT;
@@ -1729,7 +1724,7 @@ yyreduce:
   case 4:
 
 /* Line 1455 of yacc.c  */
-#line 236 "./rdql_parser.y"
+#line 232 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (3)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(3) - (3)].variable));
@@ -1739,7 +1734,7 @@ yyreduce:
   case 5:
 
 /* Line 1455 of yacc.c  */
-#line 241 "./rdql_parser.y"
+#line 237 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (2)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(2) - (2)].variable));
@@ -1749,10 +1744,14 @@ yyreduce:
   case 6:
 
 /* Line 1455 of yacc.c  */
-#line 246 "./rdql_parser.y"
+#line 242 "./rdql_parser.y"
     {
   /* The variables are freed from the rasqal_query field variables */
-  (yyval.seq)=raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+#ifdef HAVE_RAPTOR2_API
+  (yyval.seq) = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
+#else
+  (yyval.seq) = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].variable));
 }
     break;
@@ -2402,10 +2401,12 @@ yyreduce:
 /* Line 1455 of yacc.c  */
 #line 610 "./rdql_parser.y"
     {
-#ifdef RAPTOR_V2_AVAILABLE
-  (yyval.seq)=raptor_new_sequence_v2((raptor_sequence_free_handler_v2*)raptor_free_uri_v2, (raptor_sequence_print_handler_v2*)raptor_uri_print_v2, (void*)((rasqal_query*)rq)->world->raptor_world_ptr);
+#ifdef HAVE_RAPTOR2_API
+  (yyval.seq) = raptor_new_sequence((raptor_data_free_handler)raptor_free_uri,
+                           (raptor_data_print_handler)raptor_uri_print);
 #else
-  (yyval.seq)=raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri, (raptor_sequence_print_handler*)raptor_uri_print);
+  (yyval.seq) = raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri,
+                           (raptor_sequence_print_handler*)raptor_uri_print);
 #endif
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].uri));
 }
@@ -2414,7 +2415,7 @@ yyreduce:
 
 
 /* Line 1455 of yacc.c  */
-#line 2418 "rdql_parser.c"
+#line 2419 "rdql_parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2500,7 +2501,7 @@ yyerrlab:
 	}
       else
 	{
-	  yydestruct (YYPARSE_PARAM, "Error: discarding",
+	  yydestruct ("Error: discarding",
 		      yytoken, &yylval);
 	  yychar = YYEMPTY;
 	}
@@ -2556,7 +2557,7 @@ yyerrlab1:
 	YYABORT;
 
 
-      yydestruct (YYPARSE_PARAM, "Error: popping",
+      yydestruct ("Error: popping",
 		  yystos[yystate], yyvsp);
       YYPOPSTACK (1);
       yystate = *yyssp;
@@ -2599,7 +2600,7 @@ yyexhaustedlab:
 
 yyreturn:
   if (yychar != YYEMPTY)
-     yydestruct (YYPARSE_PARAM, "Cleanup: discarding lookahead",
+     yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
      this YYABORT or YYACCEPT.  */
@@ -2607,7 +2608,7 @@ yyreturn:
   YY_STACK_PRINT (yyss, yyssp);
   while (yyssp != yyss)
     {
-      yydestruct (YYPARSE_PARAM, "Cleanup: popping",
+      yydestruct ("Cleanup: popping",
 		  yystos[*yyssp], yyvsp);
       YYPOPSTACK (1);
     }
@@ -2626,7 +2627,7 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 620 "./rdql_parser.y"
+#line 622 "./rdql_parser.y"
 
 
 
@@ -2830,7 +2831,9 @@ rdql_syntax_warning(rasqal_query *rq, const char *message, ...)
 #endif
 
   va_start(arguments, message);
-  rasqal_log_error_varargs(rq->world, RAPTOR_LOG_LEVEL_WARNING, &rq->locator,
+  rasqal_log_error_varargs(rq->world,
+                           RAPTOR_LOG_LEVEL_WARN,
+                           &rq->locator,
                            message, arguments);
   va_end(arguments);
 
@@ -2926,11 +2929,7 @@ main(int argc, char *argv[])
   query=rasqal_new_query(world, "rdql", NULL);
 
   uri_string=raptor_uri_filename_to_uri_string(filename);
-#ifdef RAPTOR_V2_AVAILABLE
-  base_uri = raptor_new_uri_v2(world->raptor_world_ptr, uri_string);
-#else
-  base_uri = raptor_new_uri(uri_string);
-#endif
+  base_uri = raptor_new_uri(world->raptor_world_ptr, uri_string);
   
   rc=rasqal_query_prepare(query, (const unsigned char*)query_string, base_uri);
 
@@ -2938,11 +2937,7 @@ main(int argc, char *argv[])
 
   rasqal_free_query(query);
 
-#ifdef RAPTOR_V2_AVAILABLE
-  raptor_free_uri_v2(world->raptor_world_ptr, base_uri);
-#else
   raptor_free_uri(base_uri);
-#endif
 
   raptor_free_memory(uri_string);
 
