@@ -64,59 +64,72 @@
      DESC = 280,
      LANGMATCHES = 281,
      A = 282,
-     STR = 283,
-     LANG = 284,
-     DATATYPE = 285,
-     ISURI = 286,
-     ISBLANK = 287,
-     ISLITERAL = 288,
-     SAMETERM = 289,
-     GROUP = 290,
-     COUNT = 291,
-     SUM = 292,
-     AVG = 293,
-     MIN = 294,
-     MAX = 295,
-     DELETE = 296,
-     INSERT = 297,
-     WITH = 298,
-     CLEAR = 299,
-     CREATE = 300,
-     SILENT = 301,
-     DATA = 302,
-     DROP = 303,
-     LOAD = 304,
-     INTO = 305,
-     EXPLAIN = 306,
-     LET = 307,
-     AS = 308,
-     COALESCE = 309,
-     SC_OR = 310,
-     SC_AND = 311,
-     EQ = 312,
-     NEQ = 313,
-     LT = 314,
-     GT = 315,
-     LE = 316,
-     GE = 317,
-     ASSIGN = 318,
-     STRING_LITERAL = 319,
-     DOUBLE_LITERAL = 320,
-     DOUBLE_POSITIVE_LITERAL = 321,
-     DOUBLE_NEGATIVE_LITERAL = 322,
-     INTEGER_LITERAL = 323,
-     INTEGER_POSITIVE_LITERAL = 324,
-     INTEGER_NEGATIVE_LITERAL = 325,
-     DECIMAL_LITERAL = 326,
-     DECIMAL_POSITIVE_LITERAL = 327,
-     DECIMAL_NEGATIVE_LITERAL = 328,
-     BOOLEAN_LITERAL = 329,
-     URI_LITERAL = 330,
-     URI_LITERAL_BRACE = 331,
-     QNAME_LITERAL = 332,
-     QNAME_LITERAL_BRACE = 333,
-     BLANK_LITERAL = 334,
-     IDENTIFIER = 335
+     STRLANG = 283,
+     STRDT = 284,
+     STR = 285,
+     IRI = 286,
+     URI = 287,
+     BNODE = 288,
+     LANG = 289,
+     DATATYPE = 290,
+     ISURI = 291,
+     ISBLANK = 292,
+     ISLITERAL = 293,
+     SAMETERM = 294,
+     GROUP = 295,
+     HAVING = 296,
+     COUNT = 297,
+     SUM = 298,
+     AVG = 299,
+     MIN = 300,
+     MAX = 301,
+     GROUP_CONCAT = 302,
+     SAMPLE = 303,
+     SEPARATOR = 304,
+     DELETE = 305,
+     INSERT = 306,
+     WITH = 307,
+     CLEAR = 308,
+     CREATE = 309,
+     SILENT = 310,
+     DATA = 311,
+     DROP = 312,
+     LOAD = 313,
+     INTO = 314,
+     DEFAULT = 315,
+     COALESCE = 316,
+     AS = 317,
+     IF = 318,
+     NOT = 319,
+     IN = 320,
+     EXPLAIN = 321,
+     LET = 322,
+     SC_OR = 323,
+     SC_AND = 324,
+     EQ = 325,
+     NEQ = 326,
+     LT = 327,
+     GT = 328,
+     LE = 329,
+     GE = 330,
+     ASSIGN = 331,
+     STRING_LITERAL = 332,
+     DOUBLE_LITERAL = 333,
+     DOUBLE_POSITIVE_LITERAL = 334,
+     DOUBLE_NEGATIVE_LITERAL = 335,
+     INTEGER_LITERAL = 336,
+     INTEGER_POSITIVE_LITERAL = 337,
+     INTEGER_NEGATIVE_LITERAL = 338,
+     DECIMAL_LITERAL = 339,
+     DECIMAL_POSITIVE_LITERAL = 340,
+     DECIMAL_NEGATIVE_LITERAL = 341,
+     BOOLEAN_LITERAL = 342,
+     URI_LITERAL = 343,
+     URI_LITERAL_BRACE = 344,
+     QNAME_LITERAL = 345,
+     QNAME_LITERAL_BRACE = 346,
+     BLANK_LITERAL = 347,
+     IDENTIFIER = 348
    };
 #endif
 /* Tokens.  */
@@ -145,59 +158,72 @@
 #define DESC 280
 #define LANGMATCHES 281
 #define A 282
-#define STR 283
-#define LANG 284
-#define DATATYPE 285
-#define ISURI 286
-#define ISBLANK 287
-#define ISLITERAL 288
-#define SAMETERM 289
-#define GROUP 290
-#define COUNT 291
-#define SUM 292
-#define AVG 293
-#define MIN 294
-#define MAX 295
-#define DELETE 296
-#define INSERT 297
-#define WITH 298
-#define CLEAR 299
-#define CREATE 300
-#define SILENT 301
-#define DATA 302
-#define DROP 303
-#define LOAD 304
-#define INTO 305
-#define EXPLAIN 306
-#define LET 307
-#define AS 308
-#define COALESCE 309
-#define SC_OR 310
-#define SC_AND 311
-#define EQ 312
-#define NEQ 313
-#define LT 314
-#define GT 315
-#define LE 316
-#define GE 317
-#define ASSIGN 318
-#define STRING_LITERAL 319
-#define DOUBLE_LITERAL 320
-#define DOUBLE_POSITIVE_LITERAL 321
-#define DOUBLE_NEGATIVE_LITERAL 322
-#define INTEGER_LITERAL 323
-#define INTEGER_POSITIVE_LITERAL 324
-#define INTEGER_NEGATIVE_LITERAL 325
-#define DECIMAL_LITERAL 326
-#define DECIMAL_POSITIVE_LITERAL 327
-#define DECIMAL_NEGATIVE_LITERAL 328
-#define BOOLEAN_LITERAL 329
-#define URI_LITERAL 330
-#define URI_LITERAL_BRACE 331
-#define QNAME_LITERAL 332
-#define QNAME_LITERAL_BRACE 333
-#define BLANK_LITERAL 334
-#define IDENTIFIER 335
+#define STRLANG 283
+#define STRDT 284
+#define STR 285
+#define IRI 286
+#define URI 287
+#define BNODE 288
+#define LANG 289
+#define DATATYPE 290
+#define ISURI 291
+#define ISBLANK 292
+#define ISLITERAL 293
+#define SAMETERM 294
+#define GROUP 295
+#define HAVING 296
+#define COUNT 297
+#define SUM 298
+#define AVG 299
+#define MIN 300
+#define MAX 301
+#define GROUP_CONCAT 302
+#define SAMPLE 303
+#define SEPARATOR 304
+#define DELETE 305
+#define INSERT 306
+#define WITH 307
+#define CLEAR 308
+#define CREATE 309
+#define SILENT 310
+#define DATA 311
+#define DROP 312
+#define LOAD 313
+#define INTO 314
+#define DEFAULT 315
+#define COALESCE 316
+#define AS 317
+#define IF 318
+#define NOT 319
+#define IN 320
+#define EXPLAIN 321
+#define LET 322
+#define SC_OR 323
+#define SC_AND 324
+#define EQ 325
+#define NEQ 326
+#define LT 327
+#define GT 328
+#define LE 329
+#define GE 330
+#define ASSIGN 331
+#define STRING_LITERAL 332
+#define DOUBLE_LITERAL 333
+#define DOUBLE_POSITIVE_LITERAL 334
+#define DOUBLE_NEGATIVE_LITERAL 335
+#define INTEGER_LITERAL 336
+#define INTEGER_POSITIVE_LITERAL 337
+#define INTEGER_NEGATIVE_LITERAL 338
+#define DECIMAL_LITERAL 339
+#define DECIMAL_POSITIVE_LITERAL 340
+#define DECIMAL_NEGATIVE_LITERAL 341
+#define BOOLEAN_LITERAL 342
+#define URI_LITERAL 343
+#define URI_LITERAL_BRACE 344
+#define QNAME_LITERAL 345
+#define QNAME_LITERAL_BRACE 346
+#define BLANK_LITERAL 347
+#define IDENTIFIER 348
 
 
 
@@ -220,11 +246,12 @@ typedef union YYSTYPE
   unsigned char *name;
   rasqal_formula *formula;
   rasqal_update_operation *update;
+  unsigned int uinteger;
 
 
 
 /* Line 1676 of yacc.c  */
-#line 228 "sparql_parser.tab.h"
+#line 255 "sparql_parser.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
