@@ -137,12 +137,7 @@ main(int argc, char *argv[])
   
 
   query_file=argv[1];
-#ifdef RAPTOR_V2_AVAILABLE
   base_uri = raptor_new_uri(world->raptor_world_ptr, (const unsigned char*)argv[2]);
-#else
-  base_uri = raptor_new_uri((const unsigned char*)argv[2]);
-#endif
-
   query=rasqal_new_query(world, query_language_name, NULL);
   if(!query) {
     fprintf(stderr, "%s: creating query in language %s FAILED\n", program,
@@ -167,18 +162,14 @@ main(int argc, char *argv[])
     FAIL;
   }
   
-#ifdef RAPTOR_V2_AVAILABLE
   iostr = raptor_new_iostream_to_file_handle(world->raptor_world_ptr, stdout);
-#else
-  iostr = raptor_new_iostream_to_file_handle(stdout);
-#endif
   if(!iostr) {
     fprintf(stderr, "%s: Failed to make iostream\n", program);
     FAIL;
   }
   
   rasqal_algebra_algebra_node_write(node, iostr);
-  raptor_iostream_write_byte(iostr, '\n');
+  raptor_iostream_write_byte('\n', iostr);
   raptor_free_iostream(iostr); iostr=NULL;
 
   rasqal_free_algebra_node(node); node=NULL;
